@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,8 +26,8 @@
 package com.sun.jdi;
 
 import java.util.List;
-
 import com.sun.jdi.event.EventSet;
+import jdk.internal.javac.PreviewFeature;
 
 /**
  * A thread object from the target VM.
@@ -47,11 +47,14 @@ public interface ThreadReference extends ObjectReference {
     public final int THREAD_STATUS_ZOMBIE = 0;
     /** Thread is runnable */
     public final int THREAD_STATUS_RUNNING = 1;
-    /** Thread is sleeping - Thread.sleep() or JVM_Sleep() was called */
+    /** Thread is sleeping - {@link Thread#sleep(long)}. */
     public final int THREAD_STATUS_SLEEPING = 2;
     /** Thread is waiting on a java monitor */
     public final int THREAD_STATUS_MONITOR = 3;
-    /** Thread is waiting - Object.wait() or JVM_MonitorWait() was called */
+    /** Thread is waiting - {@link Object#wait()} or
+     * {@link java.util.concurrent.locks.LockSupport#park()}.
+     * A virtual thread that is sleeping in {@link Thread#sleep(long)} may
+     * have this thread status instead of {@link #THREAD_STATUS_SLEEPING}. */
     public final int THREAD_STATUS_WAIT = 4;
     /** Thread has not yet been started */
     public final int THREAD_STATUS_NOT_STARTED = 5;
@@ -68,23 +71,21 @@ public interface ThreadReference extends ObjectReference {
      * {@link #resume} or resumed with other threads through
      * {@link VirtualMachine#resume}.
      * <p>
-     * Unlike {@link java.lang.Thread#suspend},
-     * suspends of both the virtual machine and individual threads are
+     * Suspends of both the virtual machine and individual threads are
      * counted. Before a thread will run again, it must be resumed
-     * (through {@link #resume} or {@link ThreadReference#resume})
+     * (through {@link #resume} or {@link VirtualMachine#resume})
      * the same number of times it has been suspended.
      * <p>
-     * Suspending single threads with this method has the same dangers
-     * as {@link java.lang.Thread#suspend()}. If the suspended thread
-     * holds a monitor needed by another running thread, deadlock is
-     * possible in the target VM (at least until the suspended thread
+     * Suspending single threads with this method is inherently deadlock-prone.
+     * If the suspended thread holds a monitor needed by another running thread,
+     * deadlock is possible in the target VM (at least until the suspended thread
      * is resumed again).
      * <p>
      * The suspended thread is guaranteed to remain suspended until
-     * resumed through one of the JDI resume methods mentioned above;
-     * the application in the target VM cannot resume the suspended thread
-     * through {@link java.lang.Thread#resume}.
-     * @throws VMCannotBeModifiedException if the VirtualMachine is read-only - see {@link VirtualMachine#canBeModified()}.
+     * resumed through one of the JDI resume methods mentioned above.
+     *
+     * @throws VMCannotBeModifiedException if the VirtualMachine is read-only
+     * @see VirtualMachine#canBeModified()
      */
     @SuppressWarnings("javadoc")
     void suspend();
@@ -98,7 +99,9 @@ public interface ThreadReference extends ObjectReference {
      * the thread will continue to execute.
      * Note: the normal way to resume from an event related suspension is
      * via {@link EventSet#resume}.
-     * @throws VMCannotBeModifiedException if the VirtualMachine is read-only - see {@link VirtualMachine#canBeModified()}.
+     *
+     * @throws VMCannotBeModifiedException if the VirtualMachine is read-only
+     * @see VirtualMachine#canBeModified()
      */
     void resume();
 
@@ -113,12 +116,21 @@ public interface ThreadReference extends ObjectReference {
      * Stops this thread with an asynchronous exception.
      * A debugger thread in the target VM will stop this thread
      * with the given {@link java.lang.Throwable} object.
+     * <p>
+     * The target VM may not support, or may only provide limited support,
+     * for stopping a virtual thread with an asynchronous exception. It may,
+     * for example, only support this operation when the virtual thread is
+     * suspended at a breakpoint or singlestep event.
      *
-     * @param throwable the asynchronous exception to throw.
+     * @param throwable the asynchronous exception to throw
      * @throws InvalidTypeException if <code>throwable</code> is not
-     * an instance of java.lang.Throwable in the target VM.
-     * @throws VMCannotBeModifiedException if the VirtualMachine is read-only - see {@link VirtualMachine#canBeModified()}.
-     * @see java.lang.Thread#stop(Throwable)
+     * an instance of java.lang.Throwable in the target VM
+     * @throws IllegalThreadStateException if the thread has terminated
+     * @throws UnsupportedOperationException if the thread is a virtual
+     * thread and the target VM does not support this operation on
+     * virtual threads
+     * @throws VMCannotBeModifiedException if the VirtualMachine is read-only
+     * @see VirtualMachine#canBeModified()
      */
     @SuppressWarnings("javadoc")
     void stop(ObjectReference throwable) throws InvalidTypeException;
@@ -126,9 +138,11 @@ public interface ThreadReference extends ObjectReference {
     /**
      * Interrupts this thread unless the thread has been suspended by the
      * debugger.
-     * @throws VMCannotBeModifiedException if the VirtualMachine is read-only - see {@link VirtualMachine#canBeModified()}.
+     *
+     * @throws VMCannotBeModifiedException if the VirtualMachine is read-only
      *
      * @see java.lang.Thread#interrupt()
+     * @see VirtualMachine#canBeModified()
      */
     void interrupt();
 
@@ -234,7 +248,7 @@ public interface ThreadReference extends ObjectReference {
      * @throws IncompatibleThreadStateException if the thread is
      * not suspended in the target VM
      * @throws IndexOutOfBoundsException if the specified range is not
-     * within the range of stack frame indicies.
+     * within the range of stack frame indices.
      * That is, the exception is thrown if any of the following are true:
      * <pre>    start &lt; 0
      *    start &gt;= {@link #frameCount}
@@ -359,6 +373,11 @@ public interface ThreadReference extends ObjectReference {
      * <P>
      * All <code>StackFrame</code> objects for this thread are
      * invalidated.
+     * <p>
+     * The target VM may not support, or may only provide limited support,
+     * for popping stack frames when the thread is a virtual thread.
+     * It may, for example, only support this operation when the virtual
+     * thread is suspended at a breakpoint or singlestep event.
      * <P>
      * No events are generated by this method.
      * <P>
@@ -383,6 +402,9 @@ public interface ThreadReference extends ObjectReference {
      * @throws java.lang.IllegalArgumentException if <CODE>frame</CODE>
      * is not on this thread's call stack.
      *
+     * @throws OpaqueFrameException if this thread is a virtual thread and the
+     * target VM is unable to pop the frames.
+     *
      * @throws NativeMethodException if one of the frames that would be
      * popped is that of a native method or if the frame previous to
      * <i>frame</i> is native.
@@ -391,9 +413,11 @@ public interface ThreadReference extends ObjectReference {
      * invalid. Once this thread is resumed, the stack frame is
      * no longer valid.  This exception is also thrown if there are no
      * more frames.
-     * @throws VMCannotBeModifiedException if the VirtualMachine is read-only - see {@link VirtualMachine#canBeModified()}.
+     * @throws VMCannotBeModifiedException if the VirtualMachine is read-only.
      *
-     * @since 1.4 */
+     * @since 1.4
+     * @see VirtualMachine#canBeModified()
+     */
     void popFrames(StackFrame frame) throws IncompatibleThreadStateException;
 
     /**
@@ -412,6 +436,11 @@ public interface ThreadReference extends ObjectReference {
      * this method and resumption of thread execution, the
      * state of the stack is undefined.
      * <p>
+     * The target VM may not support, or may only provide limited support,
+     * for forcing a method to return when the thread is a virtual thread.
+     * It may, for example, only support this operation when the virtual
+     * thread is suspended at a breakpoint or singlestep event.
+     * <p>
      * No further instructions are executed in the called
      * method. Specifically, finally blocks are not executed. Note:
      * this can cause inconsistent states in the application.
@@ -455,6 +484,9 @@ public interface ThreadReference extends ObjectReference {
      * @throws IncompatibleThreadStateException if this
      * thread is not suspended.
      *
+     * @throws OpaqueFrameException if this thread is a virtual thread and the
+     * target VM is unable to force the method to return.
+     *
      * @throws NativeMethodException if the frame to be returned from
      * is that of a native method.
      *
@@ -466,12 +498,28 @@ public interface ThreadReference extends ObjectReference {
      * @throws ClassNotLoadedException if the method's return type has not yet
      * been loaded through the appropriate class loader.
      *
-     * @throws VMCannotBeModifiedException if the VirtualMachine is read-only - see {@link VirtualMachine#canBeModified()}.
+     * @throws VMCannotBeModifiedException if the VirtualMachine is read-only.
      *
      * @since 1.6
+     * @see VirtualMachine#canBeModified()
      */
     void forceEarlyReturn(Value value) throws InvalidTypeException,
                                               ClassNotLoadedException,
                                               IncompatibleThreadStateException;
 
+    /**
+     * Returns {@code true} if the thread is a
+     * <a href="{@docRoot}/java.base/java/lang/Thread.html#virtual-threads">virtual thread</a>.
+     *
+     * @return true if the thread is a virtual thread
+     *
+     * @implSpec
+     * The default implementation throws {@code UnsupportedOperationException}.
+     *
+     * @since 19
+     */
+    @PreviewFeature(feature = PreviewFeature.Feature.VIRTUAL_THREADS, reflective = true)
+    default boolean isVirtual() {
+        throw new UnsupportedOperationException("Method not implemented");
+    }
 }
