@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2023, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  */
 /*
@@ -28,119 +28,51 @@ import java.io.IOException;
 import com.sun.org.apache.bcel.internal.Const;
 
 /**
- * This class represents the type of a local variable or item on stack
- * used in the StackMap entries.
+ * This class represents the type of a local variable or item on stack used in the StackMap entries.
  *
- * @version $Id: StackMapType.java 1749603 2016-06-21 20:50:19Z ggregory $
- * @see     StackMapEntry
- * @see     StackMap
- * @see     Const
+ * @see StackMapEntry
+ * @see StackMap
+ * @see Const
  */
 public final class StackMapType implements Cloneable {
 
+    public static final StackMapType[] EMPTY_ARRAY = {}; // must be public because BCELifier code generator writes calls to it
+
     private byte type;
     private int index = -1; // Index to CONSTANT_Class or offset
-    private ConstantPool constant_pool;
-
-
-    /**
-     * Construct object from file stream.
-     * @param file Input stream
-     * @throws IOException
-     */
-    StackMapType(final DataInput file, final ConstantPool constant_pool) throws IOException {
-        this(file.readByte(), -1, constant_pool);
-        if (hasIndex()) {
-            this.index = file.readShort();
-        }
-        this.constant_pool = constant_pool;
-    }
-
+    private ConstantPool constantPool;
 
     /**
      * @param type type tag as defined in the Constants interface
      * @param index index to constant pool, or byte code offset
      */
-    public StackMapType(final byte type, final int index, final ConstantPool constant_pool) {
-        if ((type < Const.ITEM_Bogus) || (type > Const.ITEM_NewObject)) {
-            throw new RuntimeException("Illegal type for StackMapType: " + type);
-        }
-        this.type = type;
+    public StackMapType(final byte type, final int index, final ConstantPool constantPool) {
+        this.type = checkType(type);
         this.index = index;
-        this.constant_pool = constant_pool;
+        this.constantPool = constantPool;
     }
 
-
-    public void setType( final byte t ) {
-        if ((t < Const.ITEM_Bogus) || (t > Const.ITEM_NewObject)) {
-            throw new RuntimeException("Illegal type for StackMapType: " + t);
+    /**
+     * Construct object from file stream.
+     *
+     * @param file Input stream
+     * @throws IOException if an I/O error occurs.
+     */
+    StackMapType(final DataInput file, final ConstantPool constantPool) throws IOException {
+        this(file.readByte(), -1, constantPool);
+        if (hasIndex()) {
+            this.index = file.readUnsignedShort();
         }
-        type = t;
+        this.constantPool = constantPool;
     }
 
-
-    public byte getType() {
+    private byte checkType(final byte type) {
+        if (type < Const.ITEM_Bogus || type > Const.ITEM_NewObject) {
+            throw new ClassFormatException("Illegal type for StackMapType: " + type);
+        }
         return type;
     }
 
-
-    public void setIndex( final int t ) {
-        index = t;
-    }
-
-
-    /** @return index to constant pool if type == ITEM_Object, or offset
-     * in byte code, if type == ITEM_NewObject, and -1 otherwise
-     */
-    public int getIndex() {
-        return index;
-    }
-
-
-    /**
-     * Dump type entries to file.
-     *
-     * @param file Output file stream
-     * @throws IOException
-     */
-    public final void dump( final DataOutputStream file ) throws IOException {
-        file.writeByte(type);
-        if (hasIndex()) {
-            file.writeShort(getIndex());
-        }
-    }
-
-
-    /** @return true, if type is either ITEM_Object or ITEM_NewObject
-     */
-    public final boolean hasIndex() {
-        return type == Const.ITEM_Object || type == Const.ITEM_NewObject;
-    }
-
-
-    private String printIndex() {
-        if (type == Const.ITEM_Object) {
-            if (index < 0) {
-                return ", class=<unknown>";
-            }
-            return ", class=" + constant_pool.constantToString(index, Const.CONSTANT_Class);
-        } else if (type == Const.ITEM_NewObject) {
-            return ", offset=" + index;
-        } else {
-            return "";
-        }
-    }
-
-
-    /**
-     * @return String representation
-     */
-    @Override
-    public final String toString() {
-        return "(type=" + Const.getItemName(type) + printIndex() + ")";
-    }
-
-
     /**
      * @return deep copy of this object
      */
@@ -153,19 +85,78 @@ public final class StackMapType implements Cloneable {
         return null;
     }
 
+    /**
+     * Dump type entries to file.
+     *
+     * @param file Output file stream
+     * @throws IOException if an I/O error occurs.
+     */
+    public void dump(final DataOutputStream file) throws IOException {
+        file.writeByte(type);
+        if (hasIndex()) {
+            file.writeShort(getIndex());
+        }
+    }
 
     /**
      * @return Constant pool used by this object.
      */
-    public final ConstantPool getConstantPool() {
-        return constant_pool;
+    public ConstantPool getConstantPool() {
+        return constantPool;
     }
 
-
     /**
-     * @param constant_pool Constant pool to be used for this object.
+     * @return index to constant pool if type == ITEM_Object, or offset in byte code, if type == ITEM_NewObject, and -1
+     *         otherwise
      */
-    public final void setConstantPool( final ConstantPool constant_pool ) {
-        this.constant_pool = constant_pool;
+    public int getIndex() {
+        return index;
+    }
+
+    public byte getType() {
+        return type;
+    }
+
+    /**
+     * @return true, if type is either ITEM_Object or ITEM_NewObject
+     */
+    public boolean hasIndex() {
+        return type == Const.ITEM_Object || type == Const.ITEM_NewObject;
+    }
+
+    private String printIndex() {
+        if (type == Const.ITEM_Object) {
+            if (index < 0) {
+                return ", class=<unknown>";
+            }
+            return ", class=" + constantPool.constantToString(index, Const.CONSTANT_Class);
+        }
+        if (type == Const.ITEM_NewObject) {
+            return ", offset=" + index;
+        }
+        return "";
+    }
+
+    /**
+     * @param constantPool Constant pool to be used for this object.
+     */
+    public void setConstantPool(final ConstantPool constantPool) {
+        this.constantPool = constantPool;
+    }
+
+    public void setIndex(final int index) {
+        this.index = index;
+    }
+
+    public void setType(final byte type) {
+        this.type = checkType(type);
+    }
+
+    /**
+     * @return String representation
+     */
+    @Override
+    public String toString() {
+        return "(type=" + Const.getItemName(type) + printIndex() + ")";
     }
 }
