@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -105,9 +105,29 @@ public enum Source {
     JDK15("15"),
 
     /**
-      * 16, tbd
+      * 16, records and pattern matching for instanceof
       */
-    JDK16("16");
+    JDK16("16"),
+
+    /**
+      * 17, sealed classes, restoration of always-strict floating-point
+      */
+    JDK17("17"),
+
+    /**
+      * 18, no major changes
+      */
+    JDK18("18"),
+
+    /**
+      * 19, no major changes
+      */
+    JDK19("19"),
+
+    /**
+      * 20, no major changes
+      */
+    JDK20("20");
 
     private static final Context.Key<Source> sourceKey = new Context.Key<>();
 
@@ -143,7 +163,7 @@ public enum Source {
         this.name = name;
     }
 
-    public static final Source MIN = Source.JDK7;
+    public static final Source MIN = Source.JDK8;
 
     private static final Source MAX = values()[values().length - 1];
 
@@ -159,6 +179,10 @@ public enum Source {
 
     public Target requiredTarget() {
         return switch(this) {
+        case JDK20  -> Target.JDK1_20;
+        case JDK19  -> Target.JDK1_19;
+        case JDK18  -> Target.JDK1_18;
+        case JDK17  -> Target.JDK1_17;
         case JDK16  -> Target.JDK1_16;
         case JDK15  -> Target.JDK1_15;
         case JDK14  -> Target.JDK1_14;
@@ -183,26 +207,9 @@ public enum Source {
      */
     public enum Feature {
 
-        DIAMOND(JDK7, Fragments.FeatureDiamond, DiagKind.NORMAL),
         MODULES(JDK9, Fragments.FeatureModules, DiagKind.PLURAL),
         EFFECTIVELY_FINAL_VARIABLES_IN_TRY_WITH_RESOURCES(JDK9, Fragments.FeatureVarInTryWithResources, DiagKind.PLURAL),
         DEPRECATION_ON_IMPORT(MIN, JDK8),
-        POLY(JDK8),
-        LAMBDA(JDK8, Fragments.FeatureLambda, DiagKind.PLURAL),
-        METHOD_REFERENCES(JDK8, Fragments.FeatureMethodReferences, DiagKind.PLURAL),
-        DEFAULT_METHODS(JDK8, Fragments.FeatureDefaultMethods, DiagKind.PLURAL),
-        STATIC_INTERFACE_METHODS(JDK8, Fragments.FeatureStaticIntfMethods, DiagKind.PLURAL),
-        STATIC_INTERFACE_METHODS_INVOKE(JDK8, Fragments.FeatureStaticIntfMethodInvoke, DiagKind.PLURAL),
-        STRICT_METHOD_CLASH_CHECK(JDK8),
-        EFFECTIVELY_FINAL_IN_INNER_CLASSES(JDK8),
-        TYPE_ANNOTATIONS(JDK8, Fragments.FeatureTypeAnnotations, DiagKind.PLURAL),
-        ANNOTATIONS_AFTER_TYPE_PARAMS(JDK8, Fragments.FeatureAnnotationsAfterTypeParams, DiagKind.PLURAL),
-        REPEATED_ANNOTATIONS(JDK8, Fragments.FeatureRepeatableAnnotations, DiagKind.PLURAL),
-        INTERSECTION_TYPES_IN_CAST(JDK8, Fragments.FeatureIntersectionTypesInCast, DiagKind.PLURAL),
-        GRAPH_INFERENCE(JDK8),
-        FUNCTIONAL_INTERFACE_MOST_SPECIFIC(JDK8),
-        POST_APPLICABILITY_VARARGS_ACCESS_CHECK(JDK8),
-        MAP_CAPTURES_TO_BOUNDS(MIN, JDK7),
         PRIVATE_SAFE_VARARGS(JDK9),
         DIAMOND_WITH_ANONYMOUS_CLASS_CREATION(JDK9, Fragments.FeatureDiamondAndAnonClass, DiagKind.NORMAL),
         UNDERSCORE_IDENTIFIER(MIN, JDK8),
@@ -217,7 +224,12 @@ public enum Source {
         PATTERN_MATCHING_IN_INSTANCEOF(JDK16, Fragments.FeaturePatternMatchingInstanceof, DiagKind.NORMAL),
         REIFIABLE_TYPES_INSTANCEOF(JDK16, Fragments.FeatureReifiableTypesInstanceof, DiagKind.PLURAL),
         RECORDS(JDK16, Fragments.FeatureRecords, DiagKind.PLURAL),
-        SEALED_CLASSES(JDK16, Fragments.FeatureSealedClasses, DiagKind.PLURAL),
+        SEALED_CLASSES(JDK17, Fragments.FeatureSealedClasses, DiagKind.PLURAL),
+        CASE_NULL(JDK17, Fragments.FeatureCaseNull, DiagKind.NORMAL),
+        PATTERN_SWITCH(JDK17, Fragments.FeaturePatternSwitch, DiagKind.PLURAL),
+        REDUNDANT_STRICTFP(JDK17),
+        UNCONDITIONAL_PATTERN_IN_INSTANCEOF(JDK19, Fragments.FeatureUnconditionalPatternsInInstanceof, DiagKind.PLURAL),
+        RECORD_PATTERNS(JDK19, Fragments.FeatureDeconstructionPatterns, DiagKind.PLURAL),
         ;
 
         enum DiagKind {
@@ -296,6 +308,10 @@ public enum Source {
         case JDK14  -> RELEASE_14;
         case JDK15  -> RELEASE_15;
         case JDK16  -> RELEASE_16;
+        case JDK17  -> RELEASE_17;
+        case JDK18  -> RELEASE_18;
+        case JDK19  -> RELEASE_19;
+        case JDK20  -> RELEASE_20;
         default     -> null;
         };
     }
