@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2021, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -43,11 +43,13 @@ import com.sun.tools.javac.api.DiagnosticFormatter.PositionKind;
 import com.sun.tools.javac.api.Formattable;
 import com.sun.tools.javac.code.Lint.LintCategory;
 import com.sun.tools.javac.code.Printer;
+import com.sun.tools.javac.code.Source;
 import com.sun.tools.javac.code.Symbol;
 import com.sun.tools.javac.code.Type;
 import com.sun.tools.javac.code.Type.CapturedType;
 import com.sun.tools.javac.file.PathFileObject;
 import com.sun.tools.javac.jvm.Profile;
+import com.sun.tools.javac.jvm.Target;
 import com.sun.tools.javac.main.Option;
 import com.sun.tools.javac.tree.JCTree.*;
 import com.sun.tools.javac.tree.Pretty;
@@ -59,7 +61,7 @@ import static com.sun.tools.javac.util.JCDiagnostic.DiagnosticType.*;
  * by any formatter used by javac. Among the main features provided by AbstractDiagnosticFormatter are:
  *
  * <ul>
- *  <li> Provides a standard implementation of the visitor-like methods defined in the interface DiagnisticFormatter.
+ *  <li> Provides a standard implementation of the visitor-like methods defined in the interface DiagnosticFormatter.
  *  Those implementations are specifically targeting JCDiagnostic objects.
  *  <li> Provides basic support for i18n and a method for executing all locale-dependent conversions
  *  <li> Provides the formatting logic for rendering the arguments of a JCDiagnostic object.
@@ -83,7 +85,7 @@ public abstract class AbstractDiagnosticFormatter implements DiagnosticFormatter
     private SimpleConfiguration config;
 
     /**
-     * Current depth level of the disgnostic being formatted
+     * Current depth level of the diagnostic being formatted
      * (!= 0 for subdiagnostics)
      */
     protected int depth = 0;
@@ -146,8 +148,8 @@ public abstract class AbstractDiagnosticFormatter implements DiagnosticFormatter
             throw new IllegalArgumentException(); // d should have source set
         if (fullname)
             return fo.getName();
-        else if (fo instanceof PathFileObject)
-            return ((PathFileObject) fo).getShortName();
+        else if (fo instanceof PathFileObject pathFileObject)
+            return pathFileObject.getShortName();
         else
             return PathFileObject.getSimpleName(fo);
     }
@@ -176,40 +178,54 @@ public abstract class AbstractDiagnosticFormatter implements DiagnosticFormatter
      * @return string representation of the diagnostic argument
      */
     protected String formatArgument(JCDiagnostic d, Object arg, Locale l) {
-        if (arg instanceof JCDiagnostic) {
+        if (arg instanceof JCDiagnostic diagnostic) {
             String s = null;
             depth++;
             try {
-                s = formatMessage((JCDiagnostic)arg, l);
+                JCDiagnostic rewrittenDiagnostic = null;
+                if (diagnostic.hasRewriter()) {
+                    rewrittenDiagnostic = diagnostic.rewrite();
+                }
+                s = formatMessage(rewrittenDiagnostic != null ? rewrittenDiagnostic : diagnostic, l);
             }
             finally {
                 depth--;
             }
             return s;
         }
-        else if (arg instanceof JCExpression) {
-            return expr2String((JCExpression)arg);
+        else if (arg instanceof JCExpression expression) {
+            return expr2String(expression);
         }
-        else if (arg instanceof Iterable<?> && !(arg instanceof Path)) {
-            return formatIterable(d, (Iterable<?>)arg, l);
+        else if (arg instanceof Iterable<?> iterable && !(arg instanceof Path)) {
+            return formatIterable(d, iterable, l);
         }
-        else if (arg instanceof Type) {
-            return printer.visit((Type)arg, l);
+        else if (arg instanceof Type type) {
+            return printer.visit(type, l);
         }
-        else if (arg instanceof Symbol) {
-            return printer.visit((Symbol)arg, l);
+        else if (arg instanceof Symbol symbol) {
+            return printer.visit(symbol, l);
         }
-        else if (arg instanceof JavaFileObject) {
-            return ((JavaFileObject)arg).getName();
+        else if (arg instanceof JavaFileObject javaFileObject) {
+            return javaFileObject.getName();
         }
-        else if (arg instanceof Profile) {
-            return ((Profile)arg).name;
+        else if (arg instanceof Profile profile) {
+            return profile.name;
         }
-        else if (arg instanceof Option) {
-            return ((Option)arg).primaryName;
+        else if (arg instanceof Option option) {
+            return option.primaryName;
         }
-        else if (arg instanceof Formattable) {
-            return ((Formattable)arg).toString(l, messages);
+        else if (arg instanceof Formattable formattable) {
+            return formattable.toString(l, messages);
+        }
+        else if (arg instanceof Target target) {
+            return target.name;
+        }
+        else if (arg instanceof Source source) {
+            return source.name;
+        }
+        else if (arg instanceof Tag tag) {
+            return messages.getLocalizedString(l, "compiler.misc.tree.tag." +
+                                                  StringUtils.toLowerCase(tag.name()));
         }
         else {
             return String.valueOf(arg);
@@ -389,7 +405,7 @@ public abstract class AbstractDiagnosticFormatter implements DiagnosticFormatter
         return config;
     }
 
-    static public class SimpleConfiguration implements Configuration {
+    public static class SimpleConfiguration implements Configuration {
 
         protected Map<MultilineLimit, Integer> multilineLimits;
         protected EnumSet<DiagnosticPart> visibleParts;
