@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2020, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,6 +25,8 @@
 
 package com.sun.tools.javap;
 
+import java.util.Collection;
+
 import com.sun.tools.classfile.AccessFlags;
 import com.sun.tools.classfile.AnnotationDefault_attribute;
 import com.sun.tools.classfile.Attribute;
@@ -35,10 +37,13 @@ import com.sun.tools.classfile.CharacterRangeTable_attribute.Entry;
 import com.sun.tools.classfile.Code_attribute;
 import com.sun.tools.classfile.CompilationID_attribute;
 import com.sun.tools.classfile.ConstantPool;
+import com.sun.tools.classfile.ConstantPool.CONSTANT_Class_info;
 import com.sun.tools.classfile.ConstantPoolException;
 import com.sun.tools.classfile.ConstantValue_attribute;
 import com.sun.tools.classfile.DefaultAttribute;
 import com.sun.tools.classfile.Deprecated_attribute;
+import com.sun.tools.classfile.Descriptor;
+import com.sun.tools.classfile.Descriptor.InvalidDescriptor;
 import com.sun.tools.classfile.EnclosingMethod_attribute;
 import com.sun.tools.classfile.Exceptions_attribute;
 import com.sun.tools.classfile.InnerClasses_attribute;
@@ -53,6 +58,9 @@ import com.sun.tools.classfile.ModuleMainClass_attribute;
 import com.sun.tools.classfile.ModulePackages_attribute;
 import com.sun.tools.classfile.ModuleResolution_attribute;
 import com.sun.tools.classfile.ModuleTarget_attribute;
+import com.sun.tools.classfile.NestHost_attribute;
+import com.sun.tools.classfile.NestMembers_attribute;
+import com.sun.tools.classfile.Record_attribute;
 import com.sun.tools.classfile.RuntimeInvisibleAnnotations_attribute;
 import com.sun.tools.classfile.RuntimeInvisibleParameterAnnotations_attribute;
 import com.sun.tools.classfile.RuntimeInvisibleTypeAnnotations_attribute;
@@ -60,6 +68,7 @@ import com.sun.tools.classfile.RuntimeParameterAnnotations_attribute;
 import com.sun.tools.classfile.RuntimeVisibleAnnotations_attribute;
 import com.sun.tools.classfile.RuntimeVisibleParameterAnnotations_attribute;
 import com.sun.tools.classfile.RuntimeVisibleTypeAnnotations_attribute;
+import com.sun.tools.classfile.PermittedSubclasses_attribute;
 import com.sun.tools.classfile.Signature_attribute;
 import com.sun.tools.classfile.SourceDebugExtension_attribute;
 import com.sun.tools.classfile.SourceFile_attribute;
@@ -67,6 +76,7 @@ import com.sun.tools.classfile.SourceID_attribute;
 import com.sun.tools.classfile.StackMapTable_attribute;
 import com.sun.tools.classfile.StackMap_attribute;
 import com.sun.tools.classfile.Synthetic_attribute;
+import com.sun.tools.classfile.Type;
 
 import static com.sun.tools.classfile.AccessFlags.*;
 
@@ -123,9 +133,6 @@ public class AttributeWriter extends BasicWriter
 
     @Override
     public Void visitDefault(DefaultAttribute attr, Void ignore) {
-        if (attr.reason != null) {
-            report(attr.reason);
-        }
         byte[] data = attr.info;
         int i = 0;
         int j = 0;
@@ -137,7 +144,11 @@ public class AttributeWriter extends BasicWriter
             print("attribute name = #" + attr.attribute_name_index);
         }
         print(": ");
-        println("length = 0x" + toHex(attr.info.length));
+        print("length = 0x" + toHex(attr.info.length));
+        if (attr.reason != null) {
+            print(" (" + attr.reason + ")");
+        }
+        println();
 
         print("   ");
 
@@ -397,6 +408,14 @@ public class AttributeWriter extends BasicWriter
         return null;
     }
 
+    @Override
+    public Void visitNestHost(NestHost_attribute attr, Void aVoid) {
+        print("NestHost: ");
+        constantWriter.write(attr.top_index);
+        println();
+        return null;
+    }
+
     private String getJavaClassName(ModuleMainClass_attribute a) {
         try {
             return getJavaName(a.getMainClassName(constant_pool));
@@ -686,6 +705,102 @@ public class AttributeWriter extends BasicWriter
         }
     }
 
+    @Override
+    public Void visitNestMembers(NestMembers_attribute attr, Void aVoid) {
+        println("NestMembers:");
+        indent(+1);
+        try {
+            CONSTANT_Class_info[] children = attr.getChildren(constant_pool);
+            for (int i = 0; i < attr.members_indexes.length; i++) {
+                println(constantWriter.stringValue(children[i]));
+            }
+            indent(-1);
+        } catch (ConstantPoolException ex) {
+            throw new AssertionError(ex);
+        }
+        return null;
+    }
+
+    @Override
+    public Void visitRecord(Record_attribute attr, Void p) {
+        println("Record:");
+        indent(+1);
+        for (Record_attribute.ComponentInfo componentInfo : attr.component_info_arr) {
+            Signature_attribute sigAttr = (Signature_attribute) componentInfo.attributes.get(Attribute.Signature);
+
+            if (sigAttr == null)
+                print(getJavaFieldType(componentInfo.descriptor));
+            else {
+                try {
+                    Type t = sigAttr.getParsedSignature().getType(constant_pool);
+                    print(getJavaName(t.toString()));
+                } catch (ConstantPoolException e) {
+                    // report error?
+                    // fall back on non-generic descriptor
+                    print(getJavaFieldType(componentInfo.descriptor));
+                }
+            }
+
+            print(" ");
+            try {
+                print(componentInfo.getName(constant_pool));
+            } catch (ConstantPoolException e) {
+                report(e);
+                return null;
+            }
+            print(";");
+            println();
+            indent(+1);
+            if (options.showDescriptors) {
+                println("descriptor: " + getValue(componentInfo.descriptor));
+            }
+            if (options.showAllAttrs) {
+                for (Attribute componentAttr: componentInfo.attributes)
+                    write(componentInfo, componentAttr, constant_pool);
+                println();
+            }
+            indent(-1);
+        }
+        indent(-1);
+        return null;
+    }
+
+    String getValue(Descriptor d) {
+        try {
+            return d.getValue(constant_pool);
+        } catch (ConstantPoolException e) {
+            return report(e);
+        }
+    }
+
+    void writeList(String prefix, Collection<?> items, String suffix) {
+        print(prefix);
+        String sep = "";
+        for (Object item: items) {
+            print(sep);
+            print(item);
+            sep = ", ";
+        }
+        print(suffix);
+    }
+
+    String getJavaFieldType(Descriptor d) {
+        try {
+            return getJavaName(d.getFieldType(constant_pool));
+        } catch (ConstantPoolException e) {
+            return report(e);
+        } catch (InvalidDescriptor e) {
+            return report(e);
+        }
+    }
+
+    void writeModifiers(Collection<String> items) {
+        for (Object item: items) {
+            print(item);
+            print(" ");
+        }
+    }
+
     @Override
     public Void visitRuntimeVisibleAnnotations(RuntimeVisibleAnnotations_attribute attr, Void ignore) {
         println("RuntimeVisibleAnnotations:");
@@ -766,6 +881,22 @@ public class AttributeWriter extends BasicWriter
         return null;
     }
 
+    @Override
+    public Void visitPermittedSubclasses(PermittedSubclasses_attribute attr, Void ignore) {
+        println("PermittedSubclasses:");
+        indent(+1);
+        try {
+            CONSTANT_Class_info[] subtypes = attr.getSubtypes(constant_pool);
+            for (int i = 0; i < subtypes.length; i++) {
+                println(constantWriter.stringValue(subtypes[i]));
+            }
+            indent(-1);
+        } catch (ConstantPoolException ex) {
+            throw new AssertionError(ex);
+        }
+        return null;
+    }
+
     @Override
     public Void visitSignature(Signature_attribute attr, Void ignore) {
         print("Signature: #" + attr.signature_index);
