@@ -3,9 +3,9 @@ package com.sun.tools.javap.resources;
 public final class version extends java.util.ListResourceBundle {
     protected final Object[][] getContents() {
         return new Object[][] {
-            { "full", "18.0.2.1+1-1" },
-            { "jdk", "18.0.2.1" },
-            { "release", "18.0.2.1" },
+            { "full", "20.0.2+9-78" },
+            { "jdk", "20.0.2" },
+            { "release", "20.0.2" },
         };
     }
 }
