@@ -304,13 +304,17 @@ public class      BeanContextSupport extends BeanContextChildSupport
      * when the BeanContextSupport is serialized.
      */
 
+    /**
+     * A protected nested class containing per-child information
+     * in the {@code children} hashtable.
+     */
     protected class BCSChild implements Serializable {
 
-    /**
-     * Use serialVersionUID from JDK 1.7 for interoperability.
-     */
-    @Serial
-    private static final long serialVersionUID = -5815286101609939109L;
+        /**
+         * Use serialVersionUID from JDK 1.7 for interoperability.
+         */
+        @Serial
+        private static final long serialVersionUID = -5815286101609939109L;
 
         BCSChild(Object bcc, Object peer) {
             super();
@@ -771,17 +775,15 @@ public class      BeanContextSupport extends BeanContextChildSupport
         }
 
         synchronized(children) {
-            for (Iterator<Object> i = children.keySet().iterator(); i.hasNext();) {
-                Object c = i.next();
-
+            for (Object c : children.keySet()) {
                 try {
-                        return ((Visibility)c).needsGui();
-                    } catch (ClassCastException cce) {
-                        // do nothing ...
-                    }
+                    return ((Visibility)c).needsGui();
+                } catch (ClassCastException cce) {
+                    // do nothing ...
+                }
 
-                    if (c instanceof Container || c instanceof Component)
-                        return true;
+                if (c instanceof Container || c instanceof Component)
+                    return true;
             }
         }
 
@@ -798,11 +800,11 @@ public class      BeanContextSupport extends BeanContextChildSupport
 
             // lets also tell the Children that can that they may not use their GUI's
             synchronized(children) {
-                for (Iterator<Object> i = children.keySet().iterator(); i.hasNext();) {
-                    Visibility v = getChildVisibility(i.next());
+                for (Object c : children.keySet()) {
+                    Visibility v = getChildVisibility(c);
 
                     if (v != null) v.dontUseGui();
-               }
+                }
             }
         }
     }
@@ -817,8 +819,8 @@ public class      BeanContextSupport extends BeanContextChildSupport
 
             // lets also tell the Children that can that they may use their GUI's
             synchronized(children) {
-                for (Iterator<Object> i = children.keySet().iterator(); i.hasNext();) {
-                    Visibility v = getChildVisibility(i.next());
+                for (Object c : children.keySet()) {
+                    Visibility v = getChildVisibility(c);
 
                     if (v != null) v.okToUseGui();
                 }
@@ -1325,7 +1327,7 @@ public class      BeanContextSupport extends BeanContextChildSupport
             /*
              * this adaptor is used by the BeanContextSupport class to forward
              * property changes from a child to the BeanContext, avoiding
-             * accidential serialization of the BeanContext by a badly
+             * accidental serialization of the BeanContext by a badly
              * behaved Serializable child.
              */
 
@@ -1339,7 +1341,7 @@ public class      BeanContextSupport extends BeanContextChildSupport
             /*
              * this adaptor is used by the BeanContextSupport class to forward
              * vetoable changes from a child to the BeanContext, avoiding
-             * accidential serialization of the BeanContext by a badly
+             * accidental serialization of the BeanContext by a badly
              * behaved Serializable child.
              */
 
