@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,11 +25,13 @@
 
 package java.lang.reflect;
 
+import jdk.internal.misc.VM;
 import jdk.internal.org.objectweb.asm.ClassWriter;
 import jdk.internal.org.objectweb.asm.Label;
 import jdk.internal.org.objectweb.asm.MethodVisitor;
 import jdk.internal.org.objectweb.asm.Opcodes;
 import jdk.internal.org.objectweb.asm.Type;
+import sun.invoke.util.Wrapper;
 import sun.security.action.GetBooleanAction;
 
 import java.io.IOException;
@@ -38,9 +40,7 @@ import java.nio.file.Files;
 import java.nio.file.Path;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.HashMap;
 import java.util.LinkedHashMap;
-import java.util.LinkedList;
 import java.util.List;
 import java.util.ListIterator;
 import java.util.Map;
@@ -55,7 +55,7 @@ import static jdk.internal.org.objectweb.asm.Opcodes.*;
  * "generateProxyClass" method.
  */
 final class ProxyGenerator extends ClassWriter {
-
+    private static final int CLASSFILE_VERSION = VM.classFileVersion();
     private static final String JL_CLASS = "java/lang/Class";
     private static final String JL_OBJECT = "java/lang/Object";
     private static final String JL_THROWABLE = "java/lang/Throwable";
@@ -73,6 +73,7 @@ final class ProxyGenerator extends ClassWriter {
     private static final String JLR_UNDECLARED_THROWABLE_EX = "java/lang/reflect/UndeclaredThrowableException";
 
     private static final String LJL_CLASS = "Ljava/lang/Class;";
+    private static final String LJL_CLASSLOADER = "Ljava/lang/ClassLoader;";
     private static final String LJLR_METHOD = "Ljava/lang/reflect/Method;";
     private static final String LJLR_INVOCATION_HANDLER = "Ljava/lang/reflect/InvocationHandler;";
 
@@ -92,15 +93,16 @@ final class ProxyGenerator extends ClassWriter {
     /**
      * debugging flag for saving generated class files
      */
+    @SuppressWarnings("removal")
     private static final boolean saveGeneratedFiles =
             java.security.AccessController.doPrivileged(
                     new GetBooleanAction(
                             "jdk.proxy.ProxyGenerator.saveGeneratedFiles"));
 
     /* Preloaded ProxyMethod objects for methods in java.lang.Object */
-    private final static ProxyMethod hashCodeMethod;
-    private final static ProxyMethod equalsMethod;
-    private final static ProxyMethod toStringMethod;
+    private static final ProxyMethod hashCodeMethod;
+    private static final ProxyMethod equalsMethod;
+    private static final ProxyMethod toStringMethod;
 
     static {
         try {
@@ -168,6 +170,7 @@ final class ProxyGenerator extends ClassWriter {
      * @param interfaces  proxy interfaces
      * @param accessFlags access flags of the proxy class
      */
+    @SuppressWarnings("removal")
     static byte[] generateProxyClass(ClassLoader loader,
                                      final String name,
                                      List<Class<?>> interfaces,
@@ -242,7 +245,7 @@ final class ProxyGenerator extends ClassWriter {
          * List of return types that are not yet known to be
          * assignable from ("covered" by) any of the others.
          */
-        LinkedList<Class<?>> uncoveredReturnTypes = new LinkedList<>();
+        List<Class<?>> uncoveredReturnTypes = new ArrayList<>(1);
 
         nextNewReturnType:
         for (ProxyMethod pm : methods) {
@@ -453,7 +456,7 @@ final class ProxyGenerator extends ClassWriter {
      * class file generation process.
      */
     private byte[] generateClassFile() {
-        visit(V14, accessFlags, dotToSlash(className), null,
+        visit(CLASSFILE_VERSION, accessFlags, dotToSlash(className), null,
                 JLR_PROXY, typeNames(interfaces));
 
         /*
@@ -519,7 +522,7 @@ final class ProxyGenerator extends ClassWriter {
      */
     private void addProxyMethod(Method m, Class<?> fromClass) {
         Class<?> returnType = m.getReturnType();
-        Class<?>[] exceptionTypes = m.getExceptionTypes();
+        Class<?>[] exceptionTypes = m.getSharedExceptionTypes();
 
         String sig = m.toShortSignature();
         List<ProxyMethod> sigmethods = proxyMethods.computeIfAbsent(sig,
@@ -541,7 +544,7 @@ final class ProxyGenerator extends ClassWriter {
                 return;
             }
         }
-        sigmethods.add(new ProxyMethod(m, sig, m.getParameterTypes(), returnType,
+        sigmethods.add(new ProxyMethod(m, sig, m.getSharedParameterTypes(), returnType,
                 exceptionTypes, fromClass,
                 "m" + proxyMethodCount++));
     }
@@ -595,6 +598,13 @@ final class ProxyGenerator extends ClassWriter {
         mv.visitTryCatchBlock(L_startBlock, L_endBlock, L_NoClassHandler,
                 JL_CLASS_NOT_FOUND_EX);
 
+        // Put ClassLoader at local variable index 0, used by
+        // Class.forName(String, boolean, ClassLoader) calls
+        mv.visitLdcInsn(Type.getObjectType(dotToSlash(className)));
+        mv.visitMethodInsn(INVOKEVIRTUAL, JL_CLASS,
+                "getClassLoader", "()" + LJL_CLASSLOADER, false);
+        mv.visitVarInsn(ASTORE, 0);
+
         mv.visitLabel(L_startBlock);
         for (List<ProxyMethod> sigmethods : proxyMethods.values()) {
             for (ProxyMethod pm : sigmethods) {
@@ -707,8 +717,8 @@ final class ProxyGenerator extends ClassWriter {
          */
         private ProxyMethod(Method method, String methodFieldName) {
             this(method, method.toShortSignature(),
-                    method.getParameterTypes(), method.getReturnType(),
-                    method.getExceptionTypes(), method.getDeclaringClass(), methodFieldName);
+                    method.getSharedParameterTypes(), method.getReturnType(),
+                    method.getSharedExceptionTypes(), method.getDeclaringClass(), methodFieldName);
         }
 
         /**
@@ -811,21 +821,7 @@ final class ProxyGenerator extends ClassWriter {
             if (type.isPrimitive()) {
                 PrimitiveTypeInfo prim = PrimitiveTypeInfo.get(type);
 
-                if (type == int.class ||
-                        type == boolean.class ||
-                        type == byte.class ||
-                        type == char.class ||
-                        type == short.class) {
-                    mv.visitVarInsn(ILOAD, slot);
-                } else if (type == long.class) {
-                    mv.visitVarInsn(LLOAD, slot);
-                } else if (type == float.class) {
-                    mv.visitVarInsn(FLOAD, slot);
-                } else if (type == double.class) {
-                    mv.visitVarInsn(DLOAD, slot);
-                } else {
-                    throw new AssertionError();
-                }
+                mv.visitVarInsn(prim.loadOpcode, slot);
                 mv.visitMethodInsn(INVOKESTATIC, prim.wrapperClassName, "valueOf",
                         prim.wrapperValueOfDesc, false);
             } else {
@@ -847,21 +843,7 @@ final class ProxyGenerator extends ClassWriter {
                         prim.wrapperClassName,
                         prim.unwrapMethodName, prim.unwrapMethodDesc, false);
 
-                if (type == int.class ||
-                        type == boolean.class ||
-                        type == byte.class ||
-                        type == char.class ||
-                        type == short.class) {
-                    mv.visitInsn(IRETURN);
-                } else if (type == long.class) {
-                    mv.visitInsn(LRETURN);
-                } else if (type == float.class) {
-                    mv.visitInsn(FRETURN);
-                } else if (type == double.class) {
-                    mv.visitInsn(DRETURN);
-                } else {
-                    throw new AssertionError();
-                }
+                mv.visitInsn(prim.returnOpcode);
             } else {
                 mv.visitTypeInsn(CHECKCAST, dotToSlash(type.getName()));
                 mv.visitInsn(ARETURN);
@@ -870,7 +852,8 @@ final class ProxyGenerator extends ClassWriter {
 
         /**
          * Generate code for initializing the static field that stores
-         * the Method object for this proxy method.
+         * the Method object for this proxy method. A class loader is
+         * anticipated at local variable index 0.
          */
         private void codeFieldInitialization(MethodVisitor mv, String className) {
             codeClassForName(mv, fromClass);
@@ -916,13 +899,18 @@ final class ProxyGenerator extends ClassWriter {
          * Generate code to invoke the Class.forName with the name of the given
          * class to get its Class object at runtime.  The code is written to
          * the supplied stream.  Note that the code generated by this method
-         * may cause the checked ClassNotFoundException to be thrown.
+         * may cause the checked ClassNotFoundException to be thrown. A class
+         * loader is anticipated at local variable index 0.
          */
         private void codeClassForName(MethodVisitor mv, Class<?> cl) {
             mv.visitLdcInsn(cl.getName());
+            mv.visitInsn(ICONST_0); // false
+            mv.visitVarInsn(ALOAD, 0); // classLoader
             mv.visitMethodInsn(INVOKESTATIC,
                     JL_CLASS,
-                    "forName", "(Ljava/lang/String;)Ljava/lang/Class;", false);
+                    "forName",
+                    "(Ljava/lang/String;Z" + LJL_CLASSLOADER + ")Ljava/lang/Class;",
+                    false);
         }
 
         /**
@@ -950,65 +938,73 @@ final class ProxyGenerator extends ClassWriter {
     }
 
     /**
-     * A PrimitiveTypeInfo object contains assorted information about
-     * a primitive type in its public fields.  The struct for a particular
+     * A PrimitiveTypeInfo object contains bytecode-related information about
+     * a primitive type in its instance fields. The struct for a particular
      * primitive type can be obtained using the static "get" method.
      */
-    private static class PrimitiveTypeInfo {
-
-        private static Map<Class<?>, PrimitiveTypeInfo> table = new HashMap<>();
-
-        static {
-            add(byte.class, Byte.class);
-            add(char.class, Character.class);
-            add(double.class, Double.class);
-            add(float.class, Float.class);
-            add(int.class, Integer.class);
-            add(long.class, Long.class);
-            add(short.class, Short.class);
-            add(boolean.class, Boolean.class);
-        }
+    private enum PrimitiveTypeInfo {
+        BYTE(byte.class, ILOAD, IRETURN),
+        CHAR(char.class, ILOAD, IRETURN),
+        DOUBLE(double.class, DLOAD, DRETURN),
+        FLOAT(float.class, FLOAD, FRETURN),
+        INT(int.class, ILOAD, IRETURN),
+        LONG(long.class, LLOAD, LRETURN),
+        SHORT(short.class, ILOAD, IRETURN),
+        BOOLEAN(boolean.class, ILOAD, IRETURN);
 
         /**
-         * name of corresponding wrapper class
+         * internal name of corresponding wrapper class
          */
-        private String wrapperClassName;
+        private final String wrapperClassName;
         /**
          * method descriptor for wrapper class "valueOf" factory method
          */
-        private String wrapperValueOfDesc;
+        private final String wrapperValueOfDesc;
         /**
          * name of wrapper class method for retrieving primitive value
          */
-        private String unwrapMethodName;
+        private final String unwrapMethodName;
         /**
          * descriptor of same method
          */
-        private String unwrapMethodDesc;
+        private final String unwrapMethodDesc;
+        /**
+         * Load opcode used by this primitive
+         */
+        private final int loadOpcode;
+        /**
+         * Return opcode used by this primitive
+         */
+        private final int returnOpcode;
 
-        private PrimitiveTypeInfo(Class<?> primitiveClass, Class<?> wrapperClass) {
+        PrimitiveTypeInfo(Class<?> primitiveClass, int loadOpcode, int returnOpcode) {
             assert primitiveClass.isPrimitive();
+            assert returnOpcode - IRETURN == loadOpcode - ILOAD;
 
-            /**
-             * "base type" used in various descriptors (see JVMS section 4.3.2)
-             */
-            String baseTypeString =
-                    Array.newInstance(primitiveClass, 0)
-                            .getClass().getName().substring(1);
-            wrapperClassName = dotToSlash(wrapperClass.getName());
+            Wrapper wrapper = Wrapper.forPrimitiveType(primitiveClass);
+            // single-char BaseType descriptor (see JVMS section 4.3.2)
+            String baseTypeString = wrapper.basicTypeString();
+            var wrapperType = wrapper.wrapperType();
+            wrapperClassName = dotToSlash(wrapperType.getName());
             wrapperValueOfDesc =
-                    "(" + baseTypeString + ")L" + wrapperClassName + ";";
+                    "(" + baseTypeString + ")" + wrapperType.descriptorString();
             unwrapMethodName = primitiveClass.getName() + "Value";
             unwrapMethodDesc = "()" + baseTypeString;
-        }
-
-        private static void add(Class<?> primitiveClass, Class<?> wrapperClass) {
-            table.put(primitiveClass,
-                    new PrimitiveTypeInfo(primitiveClass, wrapperClass));
+            this.loadOpcode = loadOpcode;
+            this.returnOpcode = returnOpcode;
         }
 
         public static PrimitiveTypeInfo get(Class<?> cl) {
-            return table.get(cl);
+            // Uses if chain for speed: 8284880
+            if (cl == int.class)     return INT;
+            if (cl == long.class)    return LONG;
+            if (cl == boolean.class) return BOOLEAN;
+            if (cl == short.class)   return SHORT;
+            if (cl == byte.class)    return BYTE;
+            if (cl == char.class)    return CHAR;
+            if (cl == float.class)   return FLOAT;
+            if (cl == double.class)  return DOUBLE;
+            throw new AssertionError(cl);
         }
     }
 }
