@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -30,12 +30,10 @@ import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.io.ObjectStreamField;
 import java.io.Serializable;
-import java.net.InetAddress;
 import java.security.AccessController;
 import java.security.Permission;
 import java.security.PermissionCollection;
 import java.security.PrivilegedAction;
-import java.security.Security;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.Map;
@@ -245,6 +243,7 @@ public final class SocketPermission extends Permission
     };
 
     static {
+        @SuppressWarnings("removal")
         Boolean tmp = java.security.AccessController.doPrivileged(
                 new sun.security.action.GetBooleanAction("sun.net.trustNameService"));
         trustNameService = tmp.booleanValue();
@@ -332,7 +331,7 @@ public final class SocketPermission extends Permission
                         ind = host.lastIndexOf(':');
                         host = "[" + host.substring(0, ind) + "]" +
                             host.substring(ind);
-                    } else if (tokens == 8 && host.indexOf("::") == -1) {
+                    } else if (tokens == 8 && !host.contains("::")) {
                         // IPv6 address only, not followed by port
                         host = "[" + host + "]";
                     } else {
@@ -469,7 +468,7 @@ public final class SocketPermission extends Permission
             if (!host.isEmpty()) {
                 // see if we are being initialized with an IP address.
                 char ch = host.charAt(0);
-                if (ch == ':' || Character.digit(ch, 16) != -1) {
+                if (ch == ':' || IPAddressUtil.digit(ch, 16) != -1) {
                     byte ip[] = IPAddressUtil.textToNumericFormatV4(host);
                     if (ip == null) {
                         ip = IPAddressUtil.textToNumericFormatV6(host);
@@ -860,21 +859,19 @@ public final class SocketPermission extends Permission
     public boolean implies(Permission p) {
         int i,j;
 
-        if (!(p instanceof SocketPermission))
+        if (!(p instanceof SocketPermission that))
             return false;
 
         if (p == this)
             return true;
 
-        SocketPermission that = (SocketPermission) p;
-
         return ((this.mask & that.mask) == that.mask) &&
                                         impliesIgnoreMask(that);
     }
 
     /**
      * Checks if the incoming Permission's action are a proper subset of
-     * the this object's actions.
+     * this object's actions.
      * <P>
      * Check, in the following order:
      * <ul>
@@ -1040,11 +1037,9 @@ public final class SocketPermission extends Permission
         if (obj == this)
             return true;
 
-        if (! (obj instanceof SocketPermission))
+        if (! (obj instanceof SocketPermission that))
             return false;
 
-        SocketPermission that = (SocketPermission) obj;
-
         //this is (overly?) complex!!!
 
         // check the mask first
@@ -1189,9 +1184,12 @@ public final class SocketPermission extends Permission
     }
 
     /**
-     * WriteObject is called to save the state of the SocketPermission
-     * to a stream. The actions are serialized, and the superclass
-     * takes care of the name.
+     * {@code writeObject} is called to save the state of the
+     * {@code SocketPermission} to a stream. The actions are serialized,
+     * and the superclass takes care of the name.
+     *
+     * @param  s the {@code ObjectOutputStream} to which data is written
+     * @throws IOException if an I/O error occurs
      */
     @java.io.Serial
     private synchronized void writeObject(java.io.ObjectOutputStream s)
@@ -1205,8 +1203,12 @@ public final class SocketPermission extends Permission
     }
 
     /**
-     * readObject is called to restore the state of the SocketPermission from
-     * a stream.
+     * {@code readObject} is called to restore the state of the
+     * {@code SocketPermission} from a stream.
+     *
+     * @param  s the {@code ObjectInputStream} from which data is read
+     * @throws IOException if an I/O error occurs
+     * @throws ClassNotFoundException if a serialized class cannot be loaded
      */
     @java.io.Serial
     private synchronized void readObject(java.io.ObjectInputStream s)
@@ -1221,6 +1223,7 @@ public final class SocketPermission extends Permission
      * Check the system/security property for the ephemeral port range
      * for this system. The suffix is either "high" or "low"
      */
+    @SuppressWarnings("removal")
     private static int initEphemeralPorts(String suffix, int defval) {
         return AccessController.doPrivileged(
             new PrivilegedAction<>(){
@@ -1292,7 +1295,7 @@ public final class SocketPermission extends Permission
     /*
     public String toString()
     {
-        StringBuffer s = new StringBuffer(super.toString() + "\n" +
+        StringBuilder s = new StringBuilder(super.toString() + "\n" +
             "cname = " + cname + "\n" +
             "wildcard = " + wildcard + "\n" +
             "invalid = " + invalid + "\n" +
@@ -1372,15 +1375,13 @@ final class SocketPermissionCollection extends PermissionCollection
      */
     @Override
     public void add(Permission permission) {
-        if (! (permission instanceof SocketPermission))
+        if (! (permission instanceof SocketPermission sp))
             throw new IllegalArgumentException("invalid permission: "+
                                                permission);
         if (isReadOnly())
             throw new SecurityException(
                 "attempt to add a Permission to a readonly PermissionCollection");
 
-        SocketPermission sp = (SocketPermission)permission;
-
         // Add permission to map if it is absent, or replace with new
         // permission if applicable. NOTE: cannot use lambda for
         // remappingFunction parameter until JDK-8076596 is fixed.
@@ -1419,11 +1420,9 @@ final class SocketPermissionCollection extends PermissionCollection
     @Override
     public boolean implies(Permission permission)
     {
-        if (! (permission instanceof SocketPermission))
+        if (! (permission instanceof SocketPermission np))
                 return false;
 
-        SocketPermission np = (SocketPermission) permission;
-
         int desired = np.getMask();
         int effective = 0;
         int needed = desired;
@@ -1488,7 +1487,11 @@ final class SocketPermissionCollection extends PermissionCollection
     };
 
     /**
+     * Writes the state of this object to the stream.
      * @serialData "permissions" field (a Vector containing the SocketPermissions).
+     *
+     * @param  out the {@code ObjectOutputStream} to which data is written
+     * @throws IOException if an I/O error occurs
      */
     /*
      * Writes the contents of the perms field out as a Vector for
@@ -1506,8 +1509,13 @@ final class SocketPermissionCollection extends PermissionCollection
         out.writeFields();
     }
 
-    /*
-     * Reads in a Vector of SocketPermissions and saves them in the perms field.
+    /**
+     * Reads in a {@code Vector} of {@code SocketPermission} and saves
+     * them in the perms field.
+     *
+     * @param  in the {@code ObjectInputStream} from which data is read
+     * @throws IOException if an I/O error occurs
+     * @throws ClassNotFoundException if a serialized class cannot be loaded
      */
     @java.io.Serial
     private void readObject(ObjectInputStream in)
