@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,30 +25,32 @@
 
 package java.security;
 
-import java.util.*;
-import java.util.regex.*;
-
-import java.security.Provider.Service;
-
-import sun.security.jca.*;
+import jdk.internal.util.random.RandomSupport.RandomGeneratorProperties;
+import sun.security.jca.GetInstance;
 import sun.security.jca.GetInstance.Instance;
+import sun.security.jca.Providers;
 import sun.security.provider.SunEntries;
 import sun.security.util.Debug;
 
+import java.security.Provider.Service;
+import java.util.Objects;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
+
 /**
  * This class provides a cryptographically strong random number
  * generator (RNG).
  *
  * <p>A cryptographically strong random number minimally complies with the
  * statistical random number generator tests specified in
- * <a href="http://nvlpubs.nist.gov/nistpubs/FIPS/NIST.FIPS.140-2.pdf">
+ * <a href="https://nvlpubs.nist.gov/nistpubs/FIPS/NIST.FIPS.140-2.pdf">
  * <i>FIPS 140-2, Security Requirements for Cryptographic Modules</i></a>,
  * section 4.9.1.
  * Additionally, {@code SecureRandom} must produce non-deterministic output.
- * Therefore any seed material passed to a {@code SecureRandom} object must be
+ * Therefore, any seed material passed to a {@code SecureRandom} object must be
  * unpredictable, and all {@code SecureRandom} output sequences must be
  * cryptographically strong, as described in
- * <a href="http://tools.ietf.org/html/rfc4086">
+ * <a href="https://tools.ietf.org/html/rfc4086">
  * <i>RFC 4086: Randomness Requirements for Security</i></a>.
  *
  * <p> Many {@code SecureRandom} implementations are in the form of a
@@ -147,6 +149,10 @@ import sun.security.util.Debug;
  * @since 1.1
  */
 
+@RandomGeneratorProperties(
+        name = "SecureRandom",
+        isStochastic = true
+)
 public class SecureRandom extends java.util.Random {
 
     private static final Debug pdebug =
@@ -178,8 +184,8 @@ public class SecureRandom extends java.util.Random {
      */
     private final boolean threadSafe;
 
-    /*
-     * The algorithm name of null if unknown.
+    /**
+     * The algorithm name or {@code null} if unknown.
      *
      * @serial
      * @since 1.5
@@ -196,9 +202,9 @@ public class SecureRandom extends java.util.Random {
      * <p> This constructor traverses the list of registered security Providers,
      * starting with the most preferred Provider.
      * A new {@code SecureRandom} object encapsulating the
-     * {@code SecureRandomSpi} implementation from the first
-     * Provider that supports a {@code SecureRandom} (RNG) algorithm is returned.
-     * If none of the Providers support a RNG algorithm,
+     * {@code SecureRandomSpi} implementation from the first provider
+     * that supports a {@code SecureRandom} (RNG) algorithm is returned.
+     * If none of the providers support an RNG algorithm,
      * then an implementation-specific default is returned.
      *
      * <p> Note that the list of registered providers may be retrieved via
@@ -237,9 +243,9 @@ public class SecureRandom extends java.util.Random {
      * <p> This constructor traverses the list of registered security Providers,
      * starting with the most preferred Provider.
      * A new {@code SecureRandom} object encapsulating the
-     * {@code SecureRandomSpi} implementation from the first
-     * Provider that supports a {@code SecureRandom} (RNG) algorithm is returned.
-     * If none of the Providers support a RNG algorithm,
+     * {@code SecureRandomSpi} implementation from the first provider
+     * that supports a {@code SecureRandom} (RNG) algorithm is returned.
+     * If none of the providers support an RNG algorithm,
      * then an implementation-specific default is returned.
      *
      * <p> Note that the list of registered providers may be retrieved via
@@ -278,7 +284,7 @@ public class SecureRandom extends java.util.Random {
                 }
             }
         }
-        // per javadoc, if none of the Providers support a RNG algorithm,
+        // per javadoc, if none of the Providers support an RNG algorithm,
         // then an implementation-specific default is returned.
         if (prngService == null) {
             prngAlgorithm = "SHA1PRNG";
@@ -344,7 +350,7 @@ public class SecureRandom extends java.util.Random {
      * starting with the most preferred Provider.
      * A new {@code SecureRandom} object encapsulating the
      * {@code SecureRandomSpi} implementation from the first
-     * Provider that supports the specified algorithm is returned.
+     * provider that supports the specified algorithm is returned.
      *
      * <p> Note that the list of registered providers may be retrieved via
      * the {@link Security#getProviders() Security.getProviders()} method.
@@ -354,7 +360,7 @@ public class SecureRandom extends java.util.Random {
      * {@code jdk.security.provider.preferred}
      * {@link Security#getProperty(String) Security} property to determine
      * the preferred provider order for the specified algorithm. This
-     * may be different than the order of providers returned by
+     * may be different from the order of providers returned by
      * {@link Security#getProviders() Security.getProviders()}.
      *
      * @param algorithm the name of the RNG algorithm.
@@ -436,9 +442,9 @@ public class SecureRandom extends java.util.Random {
      * Random Number Generator (RNG) algorithm.
      *
      * <p> A new {@code SecureRandom} object encapsulating the
-     * {@code SecureRandomSpi} implementation from the specified {@code Provider}
-     * object is returned.  Note that the specified {@code Provider} object
-     * does not have to be registered in the provider list.
+     * {@code SecureRandomSpi} implementation from the specified provider
+     * is returned.  Note that the specified provider does not
+     * have to be registered in the provider list.
      *
      * @param algorithm the name of the RNG algorithm.
      * See the {@code SecureRandom} section in the <a href=
@@ -477,11 +483,11 @@ public class SecureRandom extends java.util.Random {
      * Random Number Generator (RNG) algorithm and supports the specified
      * {@code SecureRandomParameters} request.
      *
-     * <p> This method traverses the list of registered security Providers,
-     * starting with the most preferred Provider.
+     * <p> This method traverses the list of registered security providers,
+     * starting with the most preferred provider.
      * A new {@code SecureRandom} object encapsulating the
      * {@code SecureRandomSpi} implementation from the first
-     * Provider that supports the specified algorithm and the specified
+     * provider that supports the specified algorithm and the specified
      * {@code SecureRandomParameters} is returned.
      *
      * <p> Note that the list of registered providers may be retrieved via
@@ -491,7 +497,7 @@ public class SecureRandom extends java.util.Random {
      * The JDK Reference Implementation additionally uses the
      * {@code jdk.security.provider.preferred} property to determine
      * the preferred provider order for the specified algorithm. This
-     * may be different than the order of providers returned by
+     * may be different from the order of providers returned by
      * {@link Security#getProviders() Security.getProviders()}.
      *
      * @param algorithm the name of the RNG algorithm.
@@ -593,9 +599,8 @@ public class SecureRandom extends java.util.Random {
      *
      * <p> A new {@code SecureRandom} object encapsulating the
      * {@code SecureRandomSpi} implementation from the specified
-     * {@code Provider} object is returned.  Note that the specified
-     * {@code Provider} object does not have to be registered in the
-     * provider list.
+     * provider is returned.  Note that the specified provider
+     * does not have to be registered in the provider list.
      *
      * @param algorithm the name of the RNG algorithm.
      * See the {@code SecureRandom} section in the <a href=
@@ -719,6 +724,11 @@ public class SecureRandom extends java.util.Random {
      * in the given {@code long seed}. The given seed supplements,
      * rather than replaces, the existing seed. Thus, repeated calls
      * are guaranteed never to reduce randomness.
+     * <p>
+     * A PRNG {@code SecureRandom} will not seed itself automatically if
+     * {@code setSeed} is called before any {@code nextBytes} or {@code reseed}
+     * calls. The caller should make sure that the {@code seed} argument
+     * contains enough entropy for the security of this {@code SecureRandom}.
      *
      * <p>This method is defined for compatibility with
      * {@code java.util.Random}.
@@ -730,10 +740,11 @@ public class SecureRandom extends java.util.Random {
     @Override
     public void setSeed(long seed) {
         /*
-         * Ignore call from super constructor (as well as any other calls
-         * unfortunate enough to be passing 0).  It's critical that we
-         * ignore call from superclass constructor, as digest has not
-         * yet been initialized at that point.
+         * Ignore call from super constructor as well as any other calls
+         * unfortunate enough to be passing 0. All SecureRandom
+         * constructors call `super(0)` which leads to `setSeed(0)`.
+         * We either keep the object unseeded (in `new SecureRandom()`)
+         * or we seed the object explicitly (in `new SecureRandom(byte[])`).
          */
         if (seed != 0) {
             setSeed(longToByteArray(seed));
@@ -789,7 +800,7 @@ public class SecureRandom extends java.util.Random {
      * Generates an integer containing the user-specified number of
      * pseudo-random bits (right justified, with leading zeros).  This
      * method overrides a {@code java.util.Random} method, and serves
-     * to provide a source of random bits to all of the methods inherited
+     * to provide a source of random bits to all the methods inherited
      * from that class (for example, {@code nextInt},
      * {@code nextLong}, and {@code nextFloat}).
      *
@@ -893,9 +904,9 @@ public class SecureRandom extends java.util.Random {
          *     4 - ,nextEntry (optional)
          *     5 - nextEntry (optional)
          */
-        private static Pattern pattern =
+        private static final Pattern pattern =
             Pattern.compile(
-                "\\s*([\\S&&[^:,]]*)(\\:([\\S&&[^,]]*))?\\s*(\\,(.*))?");
+                "\\s*([\\S&&[^:,]]*)(:([\\S&&[^,]]*))?\\s*(,(.*))?");
     }
 
     /**
@@ -926,14 +937,10 @@ public class SecureRandom extends java.util.Random {
     public static SecureRandom getInstanceStrong()
             throws NoSuchAlgorithmException {
 
+        @SuppressWarnings("removal")
         String property = AccessController.doPrivileged(
-            new PrivilegedAction<>() {
-                @Override
-                public String run() {
-                    return Security.getProperty(
-                        "securerandom.strongAlgorithms");
-                }
-            });
+                (PrivilegedAction<String>) () -> Security.getProperty(
+                    "securerandom.strongAlgorithms"));
 
         if (property == null || property.isEmpty()) {
             throw new NoSuchAlgorithmException(
