@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1995, 2021, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -46,6 +46,7 @@ import java.util.function.BiConsumer;
 import java.util.function.BiFunction;
 import java.util.function.Function;
 
+import jdk.internal.util.StaticProperty;
 import sun.nio.cs.ISO_8859_1;
 import sun.nio.cs.UTF_8;
 
@@ -94,7 +95,7 @@ import jdk.internal.util.xml.PropertiesDefaultHandler;
  * methods work the same way as the load(Reader)/store(Writer, String) pair, except
  * the input/output stream is encoded in ISO 8859-1 character encoding.
  * Characters that cannot be directly represented in this encoding can be written using
- * Unicode escapes as defined in section 3.3 of
+ * Unicode escapes as defined in section {@jls 3.3} of
  * <cite>The Java Language Specification</cite>;
  * only a single 'u' character is allowed in an escape
  * sequence.
@@ -235,7 +236,7 @@ public class Properties extends Hashtable<Object,Object> {
      * character stream in a simple line-oriented format.
      * <p>
      * Properties are processed in terms of lines. There are two
-     * kinds of line, <i>natural lines</i> and <i>logical lines</i>.
+     * kinds of lines, <i>natural lines</i> and <i>logical lines</i>.
      * A natural line is defined as a line of
      * characters that is terminated either by a set of line terminator
      * characters ({@code \n} or {@code \r} or {@code \r\n})
@@ -252,8 +253,8 @@ public class Properties extends Hashtable<Object,Object> {
      * <p>
      * A natural line that contains only white space characters is
      * considered blank and is ignored.  A comment line has an ASCII
-     * {@code '#'} or {@code '!'} as its first non-white
-     * space character; comment lines are also ignored and do not
+     * {@code '#'} or {@code '!'} as its first non-whitespace
+     * character; comment lines are also ignored and do not
      * encode key-element information.  In addition to line
      * terminators, this format considers the characters space
      * ({@code ' '}, {@code '\u005Cu0020'}), tab
@@ -265,7 +266,7 @@ public class Properties extends Hashtable<Object,Object> {
      * If a logical line is spread across several natural lines, the
      * backslash escaping the line terminator sequence, the line
      * terminator sequence, and any white space at the start of the
-     * following line have no affect on the key or element values.
+     * following line have no effect on the key or element values.
      * The remainder of the discussion of key and element parsing
      * (when loading) will assume all the characters constituting
      * the key and element appear on a single natural line after
@@ -281,7 +282,7 @@ public class Properties extends Hashtable<Object,Object> {
      *
      * <p>
      * The key contains all of the characters in the line starting
-     * with the first non-white space character and up to, but not
+     * with the first non-whitespace character and up to, but not
      * including, the first unescaped {@code '='},
      * {@code ':'}, or white space character other than a line
      * terminator. All of these key termination characters may be
@@ -293,7 +294,7 @@ public class Properties extends Hashtable<Object,Object> {
      * would be the two-character key {@code ":="}.  Line
      * terminator characters can be included using {@code \r} and
      * {@code \n} escape sequences.  Any white space after the
-     * key is skipped; if the first non-white space character after
+     * key is skipped; if the first non-whitespace character after
      * the key is {@code '='} or {@code ':'}, then it is
      * ignored and any white space characters after it are also
      * skipped.  All remaining characters on the line become part of
@@ -336,7 +337,7 @@ public class Properties extends Hashtable<Object,Object> {
      * <a id="unicodeescapes"></a>
      * Characters in keys and elements can be represented in escape
      * sequences similar to those used for character and string literals
-     * (see sections 3.3 and 3.10.6 of
+     * (see sections {@jls 3.3} and {@jls 3.10.6} of
      * <cite>The Java Language Specification</cite>).
      *
      * The differences from the character escape sequences and Unicode
@@ -390,7 +391,7 @@ public class Properties extends Hashtable<Object,Object> {
      * the ISO 8859-1 character encoding; that is each byte is one Latin1
      * character. Characters not in Latin1, and certain special characters,
      * are represented in keys and elements using Unicode escapes as defined in
-     * section 3.3 of
+     * section {@jls 3.3} of
      * <cite>The Java Language Specification</cite>.
      * <p>
      * The specified stream remains open after this method returns.
@@ -655,23 +656,12 @@ public class Properties extends Hashtable<Object,Object> {
                     int value = 0;
                     for (int i = 0; i < 4; i++) {
                         aChar = in[off++];
-                        switch (aChar) {
-                          case '0': case '1': case '2': case '3': case '4':
-                          case '5': case '6': case '7': case '8': case '9':
-                             value = (value << 4) + aChar - '0';
-                             break;
-                          case 'a': case 'b': case 'c':
-                          case 'd': case 'e': case 'f':
-                             value = (value << 4) + 10 + aChar - 'a';
-                             break;
-                          case 'A': case 'B': case 'C':
-                          case 'D': case 'E': case 'F':
-                             value = (value << 4) + 10 + aChar - 'A';
-                             break;
-                          default:
-                              throw new IllegalArgumentException(
-                                           "Malformed \\uxxxx encoding.");
-                        }
+                        value = switch (aChar) {
+                            case '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' -> (value << 4) + aChar - '0';
+                            case 'a', 'b', 'c', 'd', 'e', 'f'                     -> (value << 4) + 10 + aChar - 'a';
+                            case 'A', 'B', 'C', 'D', 'E', 'F'                     -> (value << 4) + 10 + aChar - 'A';
+                            default -> throw new IllegalArgumentException("Malformed \\uxxxx encoding.");
+                        };
                     }
                     out.append((char)value);
                 } else {
@@ -701,7 +691,7 @@ public class Properties extends Hashtable<Object,Object> {
             bufLen = Integer.MAX_VALUE;
         }
         StringBuilder outBuffer = new StringBuilder(bufLen);
-
+        HexFormat hex = HexFormat.of().withUpperCase();
         for(int x=0; x<len; x++) {
             char aChar = theString.charAt(x);
             // Handle common case first, selecting largest block that
@@ -736,12 +726,8 @@ public class Properties extends Hashtable<Object,Object> {
                     break;
                 default:
                     if (((aChar < 0x0020) || (aChar > 0x007e)) & escapeUnicode ) {
-                        outBuffer.append('\\');
-                        outBuffer.append('u');
-                        outBuffer.append(toHex((aChar >> 12) & 0xF));
-                        outBuffer.append(toHex((aChar >>  8) & 0xF));
-                        outBuffer.append(toHex((aChar >>  4) & 0xF));
-                        outBuffer.append(toHex( aChar        & 0xF));
+                        outBuffer.append("\\u");
+                        outBuffer.append(hex.toHexDigits(aChar));
                     } else {
                         outBuffer.append(aChar);
                     }
@@ -752,24 +738,19 @@ public class Properties extends Hashtable<Object,Object> {
 
     private static void writeComments(BufferedWriter bw, String comments)
         throws IOException {
+        HexFormat hex = HexFormat.of().withUpperCase();
         bw.write("#");
         int len = comments.length();
         int current = 0;
         int last = 0;
-        char[] uu = new char[6];
-        uu[0] = '\\';
-        uu[1] = 'u';
         while (current < len) {
             char c = comments.charAt(current);
             if (c > '\u00ff' || c == '\n' || c == '\r') {
                 if (last != current)
                     bw.write(comments.substring(last, current));
                 if (c > '\u00ff') {
-                    uu[2] = toHex((c >> 12) & 0xf);
-                    uu[3] = toHex((c >>  8) & 0xf);
-                    uu[4] = toHex((c >>  4) & 0xf);
-                    uu[5] = toHex( c        & 0xf);
-                    bw.write(new String(uu));
+                    bw.write("\\u");
+                    bw.write(hex.toHexDigits(c));
                 } else {
                     bw.newLine();
                     if (c == '\r' &&
@@ -827,17 +808,25 @@ public class Properties extends Hashtable<Object,Object> {
      * If the comments argument is not null, then an ASCII {@code #}
      * character, the comments string, and a line separator are first written
      * to the output stream. Thus, the {@code comments} can serve as an
-     * identifying comment. Any one of a line feed ('\n'), a carriage
-     * return ('\r'), or a carriage return followed immediately by a line feed
-     * in comments is replaced by a line separator generated by the {@code Writer}
-     * and if the next character in comments is not character {@code #} or
-     * character {@code !} then an ASCII {@code #} is written out
-     * after that line separator.
+     * identifying comment. Any one of a line feed ({@code \n}), a carriage
+     * return ({@code \r}), or a carriage return followed immediately by a line feed
+     * ({@code \r\n}) in comments is replaced by a
+     * {@link System#lineSeparator() line separator} and if the next
+     * character in comments is not character {@code #} or character {@code !} then
+     * an ASCII {@code #} is written out after that line separator.
      * <p>
-     * Next, a comment line is always written, consisting of an ASCII
-     * {@code #} character, the current date and time (as if produced
-     * by the {@code toString} method of {@code Date} for the
-     * current time), and a line separator as generated by the {@code Writer}.
+     * If the {@systemProperty java.properties.date} is set on the command line
+     * and is non-empty (as determined by {@link String#isEmpty()  String.isEmpty}),
+     * a comment line is written as follows.
+     * First, a {@code #} character is written, followed by the contents
+     * of the property, followed by a line separator. Any line terminator characters
+     * in the value of the system property are treated the same way as noted above
+     * for the comments argument.
+     * If the system property is not set or is empty, a comment line is written
+     * as follows.
+     * First, a {@code #} character is written, followed by the current date and time
+     * formatted as if by the {@link Date#toString() Date.toString} method,
+     * followed by a line separator.
      * <p>
      * Then every entry in this {@code Properties} table is
      * written out, one per line. For each entry the key string is
@@ -853,6 +842,10 @@ public class Properties extends Hashtable<Object,Object> {
      * After the entries have been written, the output stream is flushed.
      * The output stream remains open after this method returns.
      *
+     * @implSpec The keys and elements are written in the natural sort order
+     * of the keys in the {@code entrySet()} unless {@code entrySet()} is
+     * overridden by a subclass to return a different value than {@code super.entrySet()}.
+     *
      * @param   writer      an output character stream writer.
      * @param   comments   a description of the property list.
      * @throws     IOException if writing this property list to the specified
@@ -923,12 +916,25 @@ public class Properties extends Hashtable<Object,Object> {
         if (comments != null) {
             writeComments(bw, comments);
         }
-        bw.write("#" + new Date().toString());
-        bw.newLine();
+        writeDateComment(bw);
+
         synchronized (this) {
-            for (Map.Entry<Object, Object> e : entrySet()) {
-                String key = (String)e.getKey();
-                String val = (String)e.getValue();
+            @SuppressWarnings("unchecked")
+            Collection<Map.Entry<String, String>> entries = (Set<Map.Entry<String, String>>) (Set) entrySet();
+            // entrySet() can be overridden by subclasses. Here we check to see if
+            // the returned instance type is the one returned by the Properties.entrySet()
+            // implementation. If yes, then we sort those entries in the natural order
+            // of their key. Else, we consider that the subclassed implementation may
+            // potentially have returned a differently ordered entries and so we just
+            // use the iteration order of the returned instance.
+            if (entries instanceof Collections.SynchronizedSet<?> ss
+                    && ss.c instanceof EntrySet) {
+                entries = new ArrayList<>(entries);
+                ((List<Map.Entry<String, String>>) entries).sort(Map.Entry.comparingByKey());
+            }
+            for (Map.Entry<String, String> e : entries) {
+                String key = e.getKey();
+                String val = e.getValue();
                 key = saveConvert(key, true, escUnicode);
                 /* No need to escape embedded and trailing spaces for value, hence
                  * pass false to flag.
@@ -941,6 +947,19 @@ public class Properties extends Hashtable<Object,Object> {
         bw.flush();
     }
 
+    private static void writeDateComment(BufferedWriter bw) throws IOException {
+        // value of java.properties.date system property isn't sensitive
+        // and so doesn't need any security manager checks to make the value accessible
+        // to the callers
+        String sysPropVal = StaticProperty.javaPropertiesDate();
+        if (sysPropVal != null && !sysPropVal.isEmpty()) {
+            writeComments(bw, sysPropVal);
+        } else {
+            bw.write("#" + new Date());
+            bw.newLine();
+        }
+    }
+
     /**
      * Loads all of the properties represented by the XML document on the
      * specified input stream into this properties table.
@@ -1271,19 +1290,6 @@ public class Properties extends Hashtable<Object,Object> {
         }
     }
 
-    /**
-     * Convert a nibble to a hex character
-     * @param   nibble  the nibble to convert.
-     */
-    private static char toHex(int nibble) {
-        return hexDigit[(nibble & 0xF)];
-    }
-
-    /** A table of hex digits */
-    private static final char[] hexDigit = {
-        '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'
-    };
-
     //
     // Hashtable methods overridden and delegated to a ConcurrentHashMap instance
 
@@ -1404,6 +1410,21 @@ public class Properties extends Hashtable<Object,Object> {
             return entrySet.containsAll(c);
         }
 
+        @Override
+        public boolean equals(Object o) {
+            return o == this || entrySet.equals(o);
+        }
+
+        @Override
+        public int hashCode() {
+            return entrySet.hashCode();
+        }
+
+        @Override
+        public String toString() {
+            return entrySet.toString();
+        }
+
         @Override
         public boolean removeAll(Collection<?> c) {
             return entrySet.removeAll(c);
