@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2007, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -47,11 +47,11 @@ public abstract class ImageOutputStreamImpl
 
     public abstract void write(int b) throws IOException;
 
-    public void write(byte b[]) throws IOException {
+    public void write(byte[] b) throws IOException {
         write(b, 0, b.length);
     }
 
-    public abstract void write(byte b[], int off, int len) throws IOException;
+    public abstract void write(byte[] b, int off, int len) throws IOException;
 
     public void writeBoolean(boolean v) throws IOException {
         write(v ? 1 : 0);
@@ -156,6 +156,9 @@ public abstract class ImageOutputStreamImpl
         write(b, 0, len*2);
     }
 
+    /**
+     * @throws UTFDataFormatException {@inheritDoc}
+     */
     public void writeUTF(String s) throws IOException {
         int strlen = s.length();
         int utflen = 0;
@@ -484,7 +487,7 @@ public abstract class ImageOutputStreamImpl
      * beginning of the {@code write(int)} and
      * {@code write(byte[], int, int)} methods.
      *
-     * @exception IOException if an I/O error occurs.
+     * @throws IOException if an I/O error occurs.
      */
     protected final void flushBits() throws IOException {
         checkClosed();
