@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -256,6 +256,7 @@ public class BasicMenuItemUI extends MenuItemUI
     }
 
     /**
+     * Registers the subcomponents of the menu.
      *
      * @param menuItem a menu item
      * @since 1.3
@@ -568,7 +569,7 @@ public class BasicMenuItemUI extends MenuItemUI
         result.width = lh.getLeadingGap();
         MenuItemLayoutHelper.addMaxWidth(lh.getCheckSize(),
                 lh.getAfterCheckIconGap(), result);
-        // Take into account mimimal text offset.
+        // Take into account minimal text offset.
         if ((!lh.isTopLevelMenu())
                 && (lh.getMinTextOffset() > 0)
                 && (result.width < lh.getMinTextOffset())) {
@@ -921,7 +922,13 @@ public class BasicMenuItemUI extends MenuItemUI
         if (dumpStack == true)
             Thread.dumpStack();
     }
-    /** Mouse input handler */
+    /**
+     * Mouse input handler.
+     * This class exists only for backward compatibility.
+     * All its functionality has been moved into Handler.
+     * @deprecated
+     */
+    @Deprecated(since = "17", forRemoval = true)
     protected class MouseInputHandler implements MouseInputListener {
         // NOTE: This class exists only for backward compatibility. All
         // its functionality has been moved into Handler. If you need to add
