@@ -3,9 +3,9 @@ package jdk.internal.jshell.tool.resources;
 public final class version extends java.util.ListResourceBundle {
     protected final Object[][] getContents() {
         return new Object[][] {
-            { "full", "17.0.12+8-LTS-286" },
-            { "jdk", "17.0.12" },
-            { "release", "17.0.12" },
+            { "full", "20.0.2+9-78" },
+            { "jdk", "20.0.2" },
+            { "release", "20.0.2" },
         };
     }
 }
