@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -28,7 +28,11 @@ package jdk.internal.net.http;
 import java.io.EOFException;
 import java.io.IOException;
 import java.io.UncheckedIOException;
+import java.lang.invoke.MethodHandles;
+import java.lang.invoke.VarHandle;
 import java.net.URI;
+import java.net.http.HttpResponse.BodyHandler;
+import java.net.http.HttpResponse.ResponseInfo;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Collections;
@@ -39,7 +43,6 @@ import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.Executor;
 import java.util.concurrent.Flow;
 import java.util.concurrent.Flow.Subscription;
-import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.BiPredicate;
 import java.net.http.HttpClient;
@@ -96,11 +99,12 @@ import jdk.internal.net.http.hpack.DecodingCallback;
  */
 class Stream<T> extends ExchangeImpl<T> {
 
+    private static final String COOKIE_HEADER = "Cookie";
     final Logger debug = Utils.getDebugLogger(this::dbgString, Utils.DEBUG);
 
     final ConcurrentLinkedQueue<Http2Frame> inputQ = new ConcurrentLinkedQueue<>();
     final SequentialScheduler sched =
-            SequentialScheduler.synchronizedScheduler(this::schedule);
+            SequentialScheduler.lockingScheduler(this::schedule);
     final SubscriptionBase userSubscription =
             new SubscriptionBase(sched, this::cancel, this::onSubscriptionError);
 
@@ -133,12 +137,18 @@ class Stream<T> extends ExchangeImpl<T> {
     private volatile boolean remotelyClosed;
     private volatile boolean closed;
     private volatile boolean endStreamSent;
-
-    final AtomicBoolean deRegistered = new AtomicBoolean(false);
+    // Indicates the first reason that was invoked when sending a ResetFrame
+    // to the server. A streamState of 0 indicates that no reset was sent.
+    // (see markStream(int code)
+    private volatile int streamState; // assigned using STREAM_STATE varhandle.
+    private volatile boolean deRegistered; // assigned using DEREGISTERED varhandle.
 
     // state flags
     private boolean requestSent, responseReceived;
 
+    // send lock: prevent sending DataFrames after reset occurred.
+    private final Object sendLock = new Object();
+
     /**
      * A reference to this Stream's connection Send Window controller. The
      * stream MUST acquire the appropriate amount of Send Window before
@@ -165,10 +175,9 @@ class Stream<T> extends ExchangeImpl<T> {
                 if (subscriber == null) {
                     // can't process anything yet
                     return;
-                } else {
-                    if (debug.on()) debug.log("subscribing user subscriber");
-                    subscriber.onSubscribe(userSubscription);
                 }
+                if (debug.on()) debug.log("subscribing user subscriber");
+                subscriber.onSubscribe(userSubscription);
             }
             while (!inputQ.isEmpty()) {
                 Http2Frame frame = inputQ.peek();
@@ -238,7 +247,7 @@ class Stream<T> extends ExchangeImpl<T> {
                         debug.log("already completed: dropping error %s", (Object) t);
                 }
             } catch (Throwable x) {
-                Log.logError("Subscriber::onError threw exception: {0}", (Object) t);
+                Log.logError("Subscriber::onError threw exception: {0}", t);
             } finally {
                 cancelImpl(t);
                 drainInputQueue();
@@ -264,6 +273,7 @@ class Stream<T> extends ExchangeImpl<T> {
         if (debug.on()) debug.log("nullBody: streamid=%d", streamid);
         // We should have an END_STREAM data frame waiting in the inputQ.
         // We need a subscriber to force the scheduler to process it.
+        assert pendingResponseSubscriber == null;
         pendingResponseSubscriber = HttpResponse.BodySubscribers.replacing(null);
         sched.runOrSchedule();
     }
@@ -291,8 +301,18 @@ class Stream<T> extends ExchangeImpl<T> {
         return endStream;
     }
 
+    @Override
+    void expectContinueFailed(int rcode) {
+        // Have to mark request as sent, due to no request body being sent in the
+        // event of a 417 Expectation Failed or some other non 100 response code
+        requestSent();
+    }
+
+    // This method is called by Http2Connection::decrementStreamCount in order
+    // to make sure that the stream count is decremented only once for
+    // a given stream.
     boolean deRegister() {
-        return deRegistered.compareAndSet(false, true);
+        return DEREGISTERED.compareAndSet(this, false, true);
     }
 
     @Override
@@ -303,7 +323,8 @@ class Stream<T> extends ExchangeImpl<T> {
         try {
             Log.logTrace("Reading body on stream {0}", streamid);
             debug.log("Getting BodySubscriber for: " + response);
-            BodySubscriber<T> bodySubscriber = handler.apply(new ResponseInfoImpl(response));
+            Http2StreamResponseSubscriber<T> bodySubscriber =
+                    createResponseSubscriber(handler, new ResponseInfoImpl(response));
             CompletableFuture<T> cf = receiveData(bodySubscriber, executor);
 
             PushGroup<?> pg = exchange.getPushGroup();
@@ -319,12 +340,27 @@ class Stream<T> extends ExchangeImpl<T> {
         }
     }
 
+    @Override
+    Http2StreamResponseSubscriber<T> createResponseSubscriber(BodyHandler<T> handler, ResponseInfo response) {
+        Http2StreamResponseSubscriber<T> subscriber =
+                new Http2StreamResponseSubscriber<>(handler.apply(response));
+        return subscriber;
+    }
+
+    // The Http2StreamResponseSubscriber is registered with the HttpClient
+    // to ensure that it gets completed if the SelectorManager aborts due
+    // to unexpected exceptions.
+    private void registerResponseSubscriber(Http2StreamResponseSubscriber<?> subscriber) {
+        client().registerSubscriber(subscriber);
+    }
+
+    private void unregisterResponseSubscriber(Http2StreamResponseSubscriber<?> subscriber) {
+        client().unregisterSubscriber(subscriber);
+    }
+
     @Override
     public String toString() {
-        StringBuilder sb = new StringBuilder();
-        sb.append("streamid: ")
-                .append(streamid);
-        return sb.toString();
+        return "streamid: " + streamid;
     }
 
     private void receiveDataFrame(DataFrame df) {
@@ -338,6 +374,36 @@ class Stream<T> extends ExchangeImpl<T> {
         sched.runOrSchedule();
     }
 
+    /**
+     * Records the first reason which was invoked when sending a ResetFrame
+     * to the server in the streamState, and return the previous value
+     * of the streamState. This is an atomic operation.
+     * A possible use of this method would be to send a ResetFrame only
+     * if no previous reset frame has been sent.
+     * For instance: <pre>{@code
+     *  if (markStream(ResetFrame.CANCEL) == 0) {
+     *      connection.sendResetFrame(streamId, ResetFrame.CANCEL);
+     *  }
+     *  }</pre>
+     * @param code the reason code as per HTTP/2 protocol
+     * @return the previous value of the stream state.
+     */
+    int  markStream(int code) {
+        if (code == 0) return streamState;
+        synchronized (sendLock) {
+            return (int) STREAM_STATE.compareAndExchange(this, 0, code);
+        }
+    }
+
+    private void sendDataFrame(DataFrame frame) {
+         synchronized (sendLock) {
+             // must not send DataFrame after reset.
+             if (streamState == 0) {
+                connection.sendDataFrame(frame);
+             }
+        }
+    }
+
     // pushes entire response body into response subscriber
     // blocking when required by local or remote flow control
     CompletableFuture<T> receiveData(BodySubscriber<T> bodySubscriber, Executor executor) {
@@ -349,10 +415,13 @@ class Stream<T> extends ExchangeImpl<T> {
         if (isCanceled()) {
             Throwable t = getCancelCause();
             responseBodyCF.completeExceptionally(t);
-        } else {
-            pendingResponseSubscriber = bodySubscriber;
-            sched.runOrSchedule(); // in case data waiting already to be processed
         }
+
+        // ensure that the body subscriber will be subscribed and onError() is
+        // invoked
+        pendingResponseSubscriber = bodySubscriber;
+        sched.runOrSchedule(); // in case data waiting already to be processed, or error
+
         return responseBodyCF;
     }
 
@@ -361,7 +430,6 @@ class Stream<T> extends ExchangeImpl<T> {
         return sendBodyImpl().thenApply( v -> this);
     }
 
-    @SuppressWarnings("unchecked")
     Stream(Http2Connection connection,
            Exchange<T> e,
            WindowController windowController)
@@ -377,6 +445,15 @@ class Stream<T> extends ExchangeImpl<T> {
         this.windowUpdater = new StreamWindowUpdateSender(connection);
     }
 
+    private boolean checkRequestCancelled() {
+        if (exchange.multi.requestCancelled()) {
+            if (errorRef.get() == null) cancel();
+            else sendResetStreamFrame(ResetFrame.CANCEL);
+            return true;
+        }
+        return false;
+    }
+
     /**
      * Entry point from Http2Connection reader thread.
      *
@@ -384,8 +461,9 @@ class Stream<T> extends ExchangeImpl<T> {
      */
     void incoming(Http2Frame frame) throws IOException {
         if (debug.on()) debug.log("incoming: %s", frame);
+        var cancelled = checkRequestCancelled() || closed;
         if ((frame instanceof HeaderFrame)) {
-            HeaderFrame hframe = (HeaderFrame)frame;
+            HeaderFrame hframe = (HeaderFrame) frame;
             if (hframe.endHeaders()) {
                 Log.logTrace("handling response (streamid={0})", streamid);
                 handleResponse();
@@ -395,26 +473,26 @@ class Stream<T> extends ExchangeImpl<T> {
                 receiveDataFrame(new DataFrame(streamid, DataFrame.END_STREAM, List.of()));
             }
         } else if (frame instanceof DataFrame) {
-            receiveDataFrame((DataFrame)frame);
+            if (cancelled) {
+                if (debug.on()) {
+                    debug.log("request cancelled or stream closed: dropping data frame");
+                }
+                connection.dropDataFrame((DataFrame) frame);
+            } else {
+                receiveDataFrame((DataFrame) frame);
+            }
         } else {
-            otherFrame(frame);
+            if (!cancelled) otherFrame(frame);
         }
     }
 
     void otherFrame(Http2Frame frame) throws IOException {
         switch (frame.type()) {
-            case WindowUpdateFrame.TYPE:
-                incoming_windowUpdate((WindowUpdateFrame) frame);
-                break;
-            case ResetFrame.TYPE:
-                incoming_reset((ResetFrame) frame);
-                break;
-            case PriorityFrame.TYPE:
-                incoming_priority((PriorityFrame) frame);
-                break;
-            default:
-                String msg = "Unexpected frame: " + frame.toString();
-                throw new IOException(msg);
+            case WindowUpdateFrame.TYPE ->  incoming_windowUpdate((WindowUpdateFrame) frame);
+            case ResetFrame.TYPE        ->  incoming_reset((ResetFrame) frame);
+            case PriorityFrame.TYPE     ->  incoming_priority((PriorityFrame) frame);
+
+            default -> throw new IOException("Unexpected frame: " + frame);
         }
     }
 
@@ -462,9 +540,9 @@ class Stream<T> extends ExchangeImpl<T> {
             Flow.Subscriber<?> subscriber =
                     responseSubscriber == null ? pendingResponseSubscriber : responseSubscriber;
             if (response == null && subscriber == null) {
-                // we haven't receive the headers yet, and won't receive any!
+                // we haven't received the headers yet, and won't receive any!
                 // handle reset now.
-                handleReset(frame, subscriber);
+                handleReset(frame, null);
             } else {
                 // put it in the input queue in order to read all
                 // pending data frames first. Indeed, a server may send
@@ -545,7 +623,7 @@ class Stream<T> extends ExchangeImpl<T> {
             Log.logRequest("PUSH_PROMISE: " + pushRequest.toString());
         }
         PushGroup<T> pushGroup = exchange.getPushGroup();
-        if (pushGroup == null) {
+        if (pushGroup == null || exchange.multi.requestCancelled()) {
             Log.logTrace("Rejecting push promise stream " + streamid);
             connection.resetStream(pushStream.streamid, ResetFrame.REFUSED_STREAM);
             pushStream.close();
@@ -611,10 +689,16 @@ class Stream<T> extends ExchangeImpl<T> {
         // Filter context restricted from userHeaders
         userh = HttpHeaders.of(userh.map(), Utils.CONTEXT_RESTRICTED(client()));
 
+        // Don't override Cookie values that have been set by the CookieHandler.
         final HttpHeaders uh = userh;
+        BiPredicate<String, String> overrides =
+                (k, v) -> COOKIE_HEADER.equalsIgnoreCase(k)
+                          || uh.firstValue(k).isEmpty();
 
         // Filter any headers from systemHeaders that are set in userHeaders
-        sysh = HttpHeaders.of(sysh.map(), (k,v) -> uh.firstValue(k).isEmpty());
+        //   except for "Cookie:" - user cookies will be appended to system
+        //   cookies
+        sysh = HttpHeaders.of(sysh.map(), overrides);
 
         OutgoingHeaders<Stream<T>> f = new OutgoingHeaders<>(sysh, userh, this);
         if (contentLength == 0) {
@@ -672,9 +756,14 @@ class Stream<T> extends ExchangeImpl<T> {
         hdrs.setHeader(":method", method);
         URI uri = request.uri();
         hdrs.setHeader(":scheme", uri.getScheme());
-        // TODO: userinfo deprecated. Needs to be removed
-        hdrs.setHeader(":authority", uri.getAuthority());
-        // TODO: ensure header names beginning with : not in user headers
+        String host = uri.getHost();
+        int port = uri.getPort();
+        assert host != null;
+        if (port != -1) {
+            hdrs.setHeader(":authority", host + ":" + port);
+        } else {
+            hdrs.setHeader(":authority", host);
+        }
         String query = uri.getRawQuery();
         String path = uri.getRawPath();
         if (path == null || path.isEmpty()) {
@@ -720,6 +809,16 @@ class Stream<T> extends ExchangeImpl<T> {
         } else {
             requestContentLen = 0;
         }
+
+        // At this point the stream doesn't have a streamid yet.
+        // It will be allocated if we send the request headers.
+        Throwable t = errorRef.get();
+        if (t != null) {
+            if (debug.on()) debug.log("stream already cancelled, headers not sent: %s", (Object)t);
+            return MinimalFuture.failedFuture(t);
+        }
+
+        // sending the headers will cause the allocation of the stream id
         OutgoingHeaders<Stream<T>> f = headerFrame(requestContentLen);
         connection.sendFrame(f);
         CompletableFuture<ExchangeImpl<T>> cf = new MinimalFuture<>();
@@ -745,10 +844,17 @@ class Stream<T> extends ExchangeImpl<T> {
         // been already closed (or will be closed shortly after).
     }
 
-    void registerStream(int id) {
-        this.streamid = id;
-        connection.putStream(this, streamid);
-        if (debug.on()) debug.log("Registered stream %d", id);
+    boolean registerStream(int id, boolean registerIfCancelled) {
+        boolean cancelled = closed || exchange.multi.requestCancelled();
+        if (!cancelled || registerIfCancelled) {
+            this.streamid = id;
+            connection.putStream(this, streamid);
+            if (debug.on()) {
+                debug.log("Stream %d registered (cancelled: %b, registerIfCancelled: %b)",
+                        streamid, cancelled, registerIfCancelled);
+            }
+        }
+        return !cancelled;
     }
 
     void signalWindowUpdate() {
@@ -782,7 +888,7 @@ class Stream<T> extends ExchangeImpl<T> {
             this.contentLength = contentLen;
             this.remainingContentLength = contentLen;
             this.sendScheduler =
-                    SequentialScheduler.synchronizedScheduler(this::trySend);
+                    SequentialScheduler.lockingScheduler(this::trySend);
         }
 
         @Override
@@ -853,6 +959,7 @@ class Stream<T> extends ExchangeImpl<T> {
                     cancelImpl(t);
                     return;
                 }
+                int state = streamState;
 
                 do {
                     // handle COMPLETED;
@@ -865,9 +972,8 @@ class Stream<T> extends ExchangeImpl<T> {
                     }
 
                     // handle bytes to send downstream
-                    while (item.hasRemaining()) {
+                    while (item.hasRemaining() && state == 0) {
                         if (debug.on()) debug.log("trySend: %d", item.remaining());
-                        assert !endStreamSent : "internal error, send data after END_STREAM flag";
                         DataFrame df = getDataFrame(item);
                         if (df == null) {
                             if (debug.on())
@@ -884,22 +990,37 @@ class Stream<T> extends ExchangeImpl<T> {
                                         + "Too many bytes in request body. Expected: "
                                         + contentLength + ", got: "
                                         + (contentLength - remainingContentLength);
+                                assert streamid > 0;
                                 connection.resetStream(streamid, ResetFrame.PROTOCOL_ERROR);
                                 throw new IOException(msg);
                             } else if (remainingContentLength == 0) {
+                                assert !endStreamSent : "internal error, send data after END_STREAM flag";
                                 df.setFlag(DataFrame.END_STREAM);
                                 endStreamSent = true;
                             }
+                        } else {
+                            assert !endStreamSent : "internal error, send data after END_STREAM flag";
+                        }
+                        if ((state = streamState) != 0) {
+                            if (debug.on()) debug.log("trySend: cancelled: %s", String.valueOf(t));
+                            break;
                         }
                         if (debug.on())
                             debug.log("trySend: sending: %d", df.getDataLength());
-                        connection.sendDataFrame(df);
+                        sendDataFrame(df);
                     }
+                    if (state != 0) break;
                     assert !item.hasRemaining();
                     ByteBuffer b = outgoing.removeFirst();
                     assert b == item;
                 } while (outgoing.peekFirst() != null);
 
+                if (state != 0) {
+                    t = errorRef.get();
+                    if (t == null) t = new IOException(ResetFrame.stringForCode(streamState));
+                    throw t;
+                }
+
                 if (debug.on()) debug.log("trySend: request 1");
                 subscription.request(1);
             } catch (Throwable ex) {
@@ -1102,7 +1223,11 @@ class Stream<T> extends ExchangeImpl<T> {
 
     @Override
     void cancel() {
-        cancel(new IOException("Stream " + streamid + " cancelled"));
+        if ((streamid == 0)) {
+            cancel(new IOException("Stream cancelled before streamid assigned"));
+        } else {
+            cancel(new IOException("Stream " + streamid + " cancelled"));
+        }
     }
 
     void onSubscriptionError(Throwable t) {
@@ -1123,6 +1248,16 @@ class Stream<T> extends ExchangeImpl<T> {
         cancelImpl(cause);
     }
 
+    @Override
+    void onProtocolError(final IOException cause) {
+        if (debug.on()) {
+            debug.log("cancelling exchange on stream %d due to protocol error: %s", streamid, cause.getMessage());
+        }
+        Log.logError("cancelling exchange on stream {0} due to protocol error: {1}\n", streamid, cause);
+        // send a RESET frame and close the stream
+        cancelImpl(cause, ResetFrame.PROTOCOL_ERROR);
+    }
+
     void connectionClosing(Throwable cause) {
         Flow.Subscriber<?> subscriber =
                 responseSubscriber == null ? pendingResponseSubscriber : responseSubscriber;
@@ -1134,10 +1269,18 @@ class Stream<T> extends ExchangeImpl<T> {
 
     // This method sends a RST_STREAM frame
     void cancelImpl(Throwable e) {
+        cancelImpl(e, ResetFrame.CANCEL);
+    }
+
+    private void cancelImpl(final Throwable e, final int resetFrameErrCode) {
         errorRef.compareAndSet(null, e);
-        if (debug.on()) debug.log("cancelling stream {0}: {1}", streamid, e);
+        if (debug.on()) {
+            if (streamid == 0) debug.log("cancelling stream: %s", (Object)e);
+            else debug.log("cancelling stream %d: %s", streamid, e);
+        }
         if (Log.trace()) {
-            Log.logTrace("cancelling stream {0}: {1}\n", streamid, e);
+            if (streamid == 0) Log.logTrace("cancelling stream: {0}\n", e);
+            else Log.logTrace("cancelling stream {0}: {1}\n", streamid, e);
         }
         boolean closing;
         if (closing = !closed) { // assigning closing to !closed
@@ -1147,10 +1290,24 @@ class Stream<T> extends ExchangeImpl<T> {
                 }
             }
         }
+
         if (closing) { // true if the stream has not been closed yet
-            if (responseSubscriber != null || pendingResponseSubscriber != null)
+            if (responseSubscriber != null || pendingResponseSubscriber != null) {
+                if (debug.on())
+                    debug.log("stream %s closing due to %s", streamid, (Object)errorRef.get());
                 sched.runOrSchedule();
+            } else {
+                if (debug.on())
+                    debug.log("stream %s closing due to %s before subscriber registered",
+                        streamid, (Object)errorRef.get());
+            }
+        } else {
+            if (debug.on()) {
+                debug.log("stream %s already closed due to %s",
+                        streamid, (Object)errorRef.get());
+            }
         }
+
         completeResponseExceptionally(e);
         if (!requestBodyCF.isDone()) {
             requestBodyCF.completeExceptionally(errorRef.get()); // we may be sending the body..
@@ -1160,14 +1317,15 @@ class Stream<T> extends ExchangeImpl<T> {
         }
         try {
             // will send a RST_STREAM frame
-            if (streamid != 0) {
-                connection.decrementStreamsCount(streamid);
-                e = Utils.getCompletionCause(e);
-                if (e instanceof EOFException) {
+            if (streamid != 0 && streamState == 0) {
+                final Throwable cause = Utils.getCompletionCause(e);
+                if (cause instanceof EOFException) {
                     // read EOF: no need to try & send reset
+                    connection.decrementStreamsCount(streamid);
                     connection.closeStream(streamid);
                 } else {
-                    connection.resetStream(streamid, ResetFrame.CANCEL);
+                    // no use to send CANCEL if already closed.
+                    sendResetStreamFrame(resetFrameErrCode);
                 }
             }
         } catch (Throwable ex) {
@@ -1175,6 +1333,14 @@ class Stream<T> extends ExchangeImpl<T> {
         }
     }
 
+    void sendResetStreamFrame(final int resetFrameErrCode) {
+        // do not reset a stream until it has a streamid.
+        if (streamid > 0 && markStream(resetFrameErrCode) == 0) {
+            connection.resetStream(streamid, resetFrameErrCode);
+        }
+        close();
+    }
+
     // This method doesn't send any frame
     void close() {
         if (closed) return;
@@ -1185,6 +1351,20 @@ class Stream<T> extends ExchangeImpl<T> {
         if (debug.on()) debug.log("close stream %d", streamid);
         Log.logTrace("Closing stream {0}", streamid);
         connection.closeStream(streamid);
+        var s = responseSubscriber == null
+                ? pendingResponseSubscriber
+                : responseSubscriber;
+        if (debug.on()) debug.log("subscriber is %s", s);
+        if (s instanceof Http2StreamResponseSubscriber<?> sw) {
+            if (debug.on()) debug.log("closing response subscriber stream %s", streamid);
+            // if the subscriber has already completed,
+            // there is nothing to do...
+            if (!sw.completed()) {
+                // otherwise make sure it will be completed
+                var cause = errorRef.get();
+                sw.complete(cause == null ? new IOException("stream closed") : cause);
+            }
+        }
         Log.logTrace("Stream {0} closed", streamid);
     }
 
@@ -1390,4 +1570,43 @@ class Stream<T> extends ExchangeImpl<T> {
             }
         }
     }
+
+    final class Http2StreamResponseSubscriber<U> extends HttpBodySubscriberWrapper<U> {
+        Http2StreamResponseSubscriber(BodySubscriber<U> subscriber) {
+            super(subscriber);
+        }
+
+        @Override
+        protected void onSubscribed() {
+            registerResponseSubscriber(this);
+        }
+
+        @Override
+        protected void complete(Throwable t) {
+            try {
+                unregisterResponseSubscriber(this);
+            } finally {
+                super.complete(t);
+            }
+        }
+
+        @Override
+        protected void onCancel() {
+            unregisterResponseSubscriber(this);
+        }
+
+    }
+
+    private static final VarHandle STREAM_STATE;
+    private static final VarHandle DEREGISTERED;
+    static {
+        try {
+            STREAM_STATE = MethodHandles.lookup()
+                    .findVarHandle(Stream.class, "streamState", int.class);
+            DEREGISTERED = MethodHandles.lookup()
+                    .findVarHandle(Stream.class, "deRegistered", boolean.class);
+        } catch (Exception x) {
+            throw new ExceptionInInitializerError(x);
+        }
+    }
 }
