@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,12 +26,13 @@
 package jdk.internal.util;
 
 import java.util.Properties;
+import java.nio.charset.Charset;
 
 /**
  * System Property access for internal use only.
  * Read-only access to System property values initialized during Phase 1
  * are cached.  Setting, clearing, or modifying the value using
- * {@link System#setProperty) or {@link System#getProperties()} is ignored.
+ * {@link System#setProperty} or {@link System#getProperties()} is ignored.
  * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
  * in these access methods. The caller of these methods should take care to ensure
  * that the returned property is not made accessible to untrusted code.</strong>
@@ -50,6 +51,11 @@ public final class StaticProperty {
     private static final String JDK_SERIAL_FILTER_FACTORY;
     private static final String JAVA_IO_TMPDIR;
     private static final String NATIVE_ENCODING;
+    private static final String FILE_ENCODING;
+    private static final String JAVA_PROPERTIES_DATE;
+    private static final String SUN_JNU_ENCODING;
+    private static final Charset jnuCharset;
+    private static final String JAVA_LOCALE_USE_OLD_ISO_CODES;
 
     private StaticProperty() {}
 
@@ -65,6 +71,11 @@ public final class StaticProperty {
         JDK_SERIAL_FILTER = getProperty(props, "jdk.serialFilter", null);
         JDK_SERIAL_FILTER_FACTORY = getProperty(props, "jdk.serialFilterFactory", null);
         NATIVE_ENCODING = getProperty(props, "native.encoding");
+        FILE_ENCODING = getProperty(props, "file.encoding");
+        JAVA_PROPERTIES_DATE = getProperty(props, "java.properties.date", null);
+        SUN_JNU_ENCODING = getProperty(props, "sun.jnu.encoding");
+        jnuCharset = Charset.forName(SUN_JNU_ENCODING, Charset.defaultCharset());
+        JAVA_LOCALE_USE_OLD_ISO_CODES = getProperty(props, "java.locale.useOldISOCodes", "");
     }
 
     private static String getProperty(Properties props, String key) {
@@ -82,91 +93,77 @@ public final class StaticProperty {
     }
 
     /**
-     * Return the {@code java.home} system property.
+     * {@return the {@code java.home} system property}
      *
      * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
      * in this method. The caller of this method should take care to ensure
      * that the returned property is not made accessible to untrusted code.</strong>
-     *
-     * @return the {@code java.home} system property
      */
     public static String javaHome() {
         return JAVA_HOME;
     }
 
     /**
-     * Return the {@code user.home} system property.
+     * {@return the {@code user.home} system property}
      *
      * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
      * in this method. The caller of this method should take care to ensure
      * that the returned property is not made accessible to untrusted code.</strong>
-     *
-     * @return the {@code user.home} system property
      */
     public static String userHome() {
         return USER_HOME;
     }
 
     /**
-     * Return the {@code user.dir} system property.
+     * {@return the {@code user.dir} system property}
      *
      * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
      * in this method. The caller of this method should take care to ensure
      * that the returned property is not made accessible to untrusted code.</strong>
-     *
-     * @return the {@code user.dir} system property
      */
     public static String userDir() {
         return USER_DIR;
     }
 
     /**
-     * Return the {@code user.name} system property.
+     * {@return the {@code user.name} system property}
      *
      * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
      * in this method. The caller of this method should take care to ensure
      * that the returned property is not made accessible to untrusted code.</strong>
-     *
-     * @return the {@code user.name} system property
      */
     public static String userName() {
         return USER_NAME;
     }
 
     /**
-     * Return the {@code java.library.path} system property.
+     * {@return the {@code java.library.path} system property}
      *
      * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
      * in this method. The caller of this method should take care to ensure
      * that the returned property is not made accessible to untrusted code.</strong>
-     *
-     * @return the {@code java.library.path} system property
      */
     public static String javaLibraryPath() {
         return JAVA_LIBRARY_PATH;
     }
 
     /**
-     * Return the {@code java.io.tmpdir} system property.
+     * {@return the {@code java.io.tmpdir} system property}
      *
      * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
      * in this method. The caller of this method should take care to ensure
      * that the returned property is not made accessible to untrusted code.</strong>
-     *
-     * @return the {@code java.io.tmpdir} system property
      */
     public static String javaIoTmpDir() {
         return JAVA_IO_TMPDIR;
     }
 
     /**
-     * Return the {@code sun.boot.library.path} system property.
+     * {@return the {@code sun.boot.library.path} system property}
      *
      * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
      * in this method. The caller of this method should take care to ensure
      * that the returned property is not made accessible to untrusted code.</strong>
-     *
-     * @return the {@code sun.boot.library.path} system property
      */
     public static String sunBootLibraryPath() {
         return SUN_BOOT_LIBRARY_PATH;
@@ -174,13 +171,11 @@ public final class StaticProperty {
 
 
     /**
-     * Return the {@code jdk.serialFilter} system property.
+     * {@return the {@code jdk.serialFilter} system property}
      *
      * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
      * in this method. The caller of this method should take care to ensure
      * that the returned property is not made accessible to untrusted code.</strong>
-     *
-     * @return the {@code jdk.serialFilter} system property
      */
     public static String jdkSerialFilter() {
         return JDK_SERIAL_FILTER;
@@ -188,28 +183,77 @@ public final class StaticProperty {
 
 
     /**
-     * Return the {@code jdk.serialFilterFactory} system property.
+     * {@return the {@code jdk.serialFilterFactory} system property}
      *
      * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
      * in this method. The caller of this method should take care to ensure
      * that the returned property is not made accessible to untrusted code.</strong>
-     *
-     * @return the {@code jdk.serialFilterFactory} system property
      */
     public static String jdkSerialFilterFactory() {
         return JDK_SERIAL_FILTER_FACTORY;
     }
 
     /**
-     * Return the {@code native.encoding} system property.
+     * {@return the {@code native.encoding} system property}
      *
      * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
      * in this method. The caller of this method should take care to ensure
      * that the returned property is not made accessible to untrusted code.</strong>
-     *
-     * @return the {@code native.encoding} system property
      */
     public static String nativeEncoding() {
         return NATIVE_ENCODING;
     }
+
+    /**
+     * {@return the {@code file.encoding} system property}
+     *
+     * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
+     * in this method. The caller of this method should take care to ensure
+     * that the returned property is not made accessible to untrusted code.</strong>
+     */
+    public static String fileEncoding() {
+        return FILE_ENCODING;
+    }
+
+    /**
+     * {@return the {@code java.properties.date} system property}
+     *
+     * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
+     * in this method.</strong>
+     */
+    public static String javaPropertiesDate() {
+        return JAVA_PROPERTIES_DATE;
+    }
+
+    /**
+     * {@return the {@code sun.jnu.encoding} system property}
+     *
+     * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
+     * in this method. The caller of this method should take care to ensure
+     * that the returned property is not made accessible to untrusted code.</strong>
+     */
+    public static String jnuEncoding() {
+        return SUN_JNU_ENCODING;
+    }
+
+    /**
+     * {@return {@code Charset} for {@code sun.jnu.encoding} system property}
+     *
+     * <strong>If {@code sun.jnu.encoding} system property has invalid
+     * encoding name, {@link Charset#defaultCharset()} is returned.</strong>
+     */
+    public static Charset jnuCharset() {
+        return jnuCharset;
+    }
+
+    /**
+     * {@return the {@code java.locale.useOldISOCodes} system property}
+     *
+     * <strong>{@link SecurityManager#checkPropertyAccess} is NOT checked
+     * in this method. The caller of this method should take care to ensure
+     * that the returned property is not made accessible to untrusted code.</strong>
+     */
+    public static String javaLocaleUseOldISOCodes() {
+        return JAVA_LOCALE_USE_OLD_ISO_CODES;
+    }
 }
