@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,30 +25,35 @@
 
 package jdk.javadoc.internal.doclets.toolkit.builders;
 
-import java.text.MessageFormat;
-import java.util.*;
-import java.util.stream.Collectors;
-
+import java.util.Collection;
+import java.util.Comparator;
+import java.util.EnumMap;
+import java.util.HashMap;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
+import java.util.Objects;
+import java.util.Optional;
+import java.util.SortedSet;
+import java.util.TreeSet;
 import javax.lang.model.element.Element;
 import javax.lang.model.element.ExecutableElement;
 import javax.lang.model.element.TypeElement;
 import javax.lang.model.element.VariableElement;
 import javax.lang.model.util.ElementFilter;
+import javax.tools.Diagnostic;
 
 import com.sun.source.doctree.DocCommentTree;
 import com.sun.source.doctree.DocTree;
-import com.sun.source.doctree.DocTree.Kind;
-import com.sun.source.doctree.SinceTree;
-import com.sun.source.doctree.UnknownBlockTagTree;
+import jdk.javadoc.internal.doclets.toolkit.BaseConfiguration;
 import jdk.javadoc.internal.doclets.toolkit.ClassWriter;
 import jdk.javadoc.internal.doclets.toolkit.Content;
 import jdk.javadoc.internal.doclets.toolkit.MemberSummaryWriter;
 import jdk.javadoc.internal.doclets.toolkit.WriterFactory;
-import jdk.javadoc.internal.doclets.toolkit.util.CommentHelper;
 import jdk.javadoc.internal.doclets.toolkit.util.DocFinder;
+import jdk.javadoc.internal.doclets.toolkit.util.DocFinder.Result;
 import jdk.javadoc.internal.doclets.toolkit.util.Utils;
 import jdk.javadoc.internal.doclets.toolkit.util.VisibleMemberTable;
-import jdk.javadoc.internal.doclets.toolkit.CommentUtils;
 
 import static jdk.javadoc.internal.doclets.toolkit.util.VisibleMemberTable.Kind.*;
 
@@ -57,11 +62,6 @@ import static jdk.javadoc.internal.doclets.toolkit.util.VisibleMemberTable.Kind.
  * There are two anonymous subtype variants of this builder, created
  * in the {@link #getInstance} methods. One is for general types;
  * the other is for annotation types.
- *
- *  <p><b>This is NOT part of any supported API.
- *  If you write code that depends on this, you do so at your own risk.
- *  This code and its internal interfaces are subject to change or
- *  deletion without notice.</b>
  */
 public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
 
@@ -102,15 +102,15 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
             ClassWriter classWriter, Context context) {
         MemberSummaryBuilder builder = new MemberSummaryBuilder(context, classWriter.getTypeElement()) {
             @Override
-            public void build(Content summariesList) {
-                buildPropertiesSummary(summariesList);
-                buildNestedClassesSummary(summariesList);
-                buildEnumConstantsSummary(summariesList);
-                buildAnnotationTypeRequiredMemberSummary(summariesList);
-                buildAnnotationTypeOptionalMemberSummary(summariesList);
-                buildFieldsSummary(summariesList);
-                buildConstructorsSummary(summariesList);
-                buildMethodsSummary(summariesList);
+            public void build(Content target) {
+                buildPropertiesSummary(target);
+                buildNestedClassesSummary(target);
+                buildEnumConstantsSummary(target);
+                buildAnnotationTypeRequiredMemberSummary(target);
+                buildAnnotationTypeOptionalMemberSummary(target);
+                buildFieldsSummary(target);
+                buildConstructorsSummary(target);
+                buildMethodsSummary(target);
             }
 
             @Override
@@ -166,15 +166,6 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
         return out;
     }
 
-    /**
-     * Returns true if there are members of the given kind, false otherwise.
-     * @param kind
-     * @return true if there are members of the given kind, false otherwise
-     */
-    public boolean hasMembers(VisibleMemberTable.Kind kind) {
-        return !getVisibleMembers(kind).isEmpty();
-    }
-
     /**
      * Builds the summary for any optional members of an annotation type.
      *
@@ -231,14 +222,14 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
      * @param summariesList the list of summaries to which the summary will be added
      */
     protected void buildNestedClassesSummary(Content summariesList) {
-        MemberSummaryWriter writer = memberSummaryWriters.get(INNER_CLASSES);
-        addSummary(writer, INNER_CLASSES, true, summariesList);
+        MemberSummaryWriter writer = memberSummaryWriters.get(NESTED_CLASSES);
+        addSummary(writer, NESTED_CLASSES, true, summariesList);
     }
 
     /**
      * Builds the summary for any methods.
      *
-     * @param summariesList the content tree to which the documentation will be added
+     * @param summariesList the content to which the documentation will be added
      */
     protected void buildMethodsSummary(Content summariesList) {
         MemberSummaryWriter writer = memberSummaryWriters.get(METHODS);
@@ -248,7 +239,7 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
     /**
      * Builds the summary for any constructors.
      *
-     * @param summariesList the content tree to which the documentation will be added
+     * @param summariesList the content to which the documentation will be added
      */
     protected void buildConstructorsSummary(Content summariesList) {
         MemberSummaryWriter writer = memberSummaryWriters.get(CONSTRUCTORS);
@@ -260,7 +251,7 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
      *
      * @param writer the summary writer to write the output.
      * @param kind the kind of  members to summarize.
-     * @param summaryTreeList list of content trees to which the documentation will be added
+     * @param summaryTreeList the list of contents to which the documentation will be added
      */
     private void buildSummary(MemberSummaryWriter writer,
             VisibleMemberTable.Kind kind, LinkedList<Content> summaryTreeList) {
@@ -268,137 +259,35 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
         if (!members.isEmpty()) {
             for (Element member : members) {
                 final Element property = pHelper.getPropertyElement(member);
-                if (property != null) {
-                    processProperty(member, property);
+                if (property != null && member instanceof ExecutableElement ee) {
+                    configuration.cmtUtils.updatePropertyMethodComment(ee, property);
                 }
-                List<? extends DocTree> firstSentenceTags = utils.getFirstSentenceTrees(member);
-                if (utils.isExecutableElement(member) && firstSentenceTags.isEmpty()) {
-                    //Inherit comments from overridden or implemented method if
-                    //necessary.
-                    DocFinder.Output inheritedDoc =
-                            DocFinder.search(configuration,
-                                    new DocFinder.Input(utils, member));
-                    if (inheritedDoc.holder != null
-                            && !utils.getFirstSentenceTrees(inheritedDoc.holder).isEmpty()) {
-                        // let the comment helper know of the overridden element
-                        CommentHelper ch = utils.getCommentHelper(member);
-                        ch.setOverrideElement(inheritedDoc.holder);
-                        firstSentenceTags = utils.getFirstSentenceTrees(inheritedDoc.holder);
-                    }
+                if (utils.isMethod(member)) {
+                    var docFinder = utils.docFinder();
+                    Optional<List<? extends DocTree>> r = docFinder.search((ExecutableElement) member, (m -> {
+                        var firstSentenceTrees = utils.getFirstSentenceTrees(m);
+                        Optional<List<? extends DocTree>> optional = firstSentenceTrees.isEmpty() ? Optional.empty() : Optional.of(firstSentenceTrees);
+                        return Result.fromOptional(optional);
+                    })).toOptional();
+                    // The fact that we use `member` for possibly unrelated tags is suspicious
+                    writer.addMemberSummary(typeElement, member, r.orElse(List.of()));
+                } else {
+                    writer.addMemberSummary(typeElement, member, utils.getFirstSentenceTrees(member));
                 }
-                writer.addMemberSummary(typeElement, member, firstSentenceTags);
             }
-            summaryTreeList.add(writer.getSummaryTableTree(typeElement));
+            summaryTreeList.add(writer.getSummaryTable(typeElement));
         }
     }
 
-    /**
-     * Process the property method, property setter and/or property getter
-     * comment text so that it contains the documentation from
-     * the property field. The method adds the leading sentence,
-     * copied documentation including the defaultValue tag and
-     * the see tags if the appropriate property getter and setter are
-     * available.
-     *
-     * @param member the member which is to be augmented.
-     * @param property the original property documentation.
-     */
-    private void processProperty(Element member,
-                                 Element property) {
-        CommentUtils cmtutils = configuration.cmtUtils;
-        final boolean isSetter = isSetter(member);
-        final boolean isGetter = isGetter(member);
-
-        List<DocTree> fullBody = new ArrayList<>();
-        List<DocTree> blockTags = new ArrayList<>();
-        if (isGetter || isSetter) {
-            //add "[GS]ets the value of the property PROPERTY_NAME."
-            if (isSetter) {
-                String text = MessageFormat.format(
-                        resources.getText("doclet.PropertySetterWithName"),
-                        utils.propertyName((ExecutableElement)member));
-                fullBody.addAll(cmtutils.makeFirstSentenceTree(text));
-            }
-            if (isGetter) {
-                String text = MessageFormat.format(
-                        resources.getText("doclet.PropertyGetterWithName"),
-                        utils.propertyName((ExecutableElement) member));
-                fullBody.addAll(cmtutils.makeFirstSentenceTree(text));
-            }
-            List<? extends DocTree> propertyTags = utils.getBlockTags(property,
-                    t -> (t instanceof UnknownBlockTagTree)
-                            && ((UnknownBlockTagTree) t).getTagName().equals("propertyDescription"));
-            if (propertyTags.isEmpty()) {
-                List<? extends DocTree> comment = utils.getFullBody(property);
-                blockTags.addAll(cmtutils.makePropertyDescriptionTree(comment));
-            }
-        } else {
-            fullBody.addAll(utils.getFullBody(property));
-        }
-
-        // copy certain tags
-        List<? extends SinceTree> tags = utils.getBlockTags(property, Kind.SINCE, SinceTree.class);
-        blockTags.addAll(tags);
-
-        List<? extends DocTree> bTags = utils.getBlockTags(property,
-                t -> (t instanceof UnknownBlockTagTree)
-                        && ((UnknownBlockTagTree) t).getTagName().equals("defaultValue"));
-        blockTags.addAll(bTags);
-
-        //add @see tags
-        if (!isGetter && !isSetter) {
-            ExecutableElement getter = pHelper.getGetterForProperty((ExecutableElement)member);
-            ExecutableElement setter = pHelper.getSetterForProperty((ExecutableElement)member);
-
-            if (null != getter) {
-                StringBuilder sb = new StringBuilder("#");
-                sb.append(utils.getSimpleName(getter)).append("()");
-                blockTags.add(cmtutils.makeSeeTree(sb.toString(), getter));
-            }
-
-            if (null != setter) {
-                VariableElement param = setter.getParameters().get(0);
-                StringBuilder sb = new StringBuilder("#");
-                sb.append(utils.getSimpleName(setter));
-                if (!utils.isTypeVariable(param.asType())) {
-                    sb.append("(").append(utils.getTypeSignature(param.asType(), false, true)).append(")");
-                }
-                blockTags.add(cmtutils.makeSeeTree(sb.toString(), setter));
-            }
-        }
-        cmtutils.setDocCommentTree(member, fullBody, blockTags);
-    }
-
-    /**
-     * Test whether the method is a getter.
-     * @param element property method documentation. Needs to be either property
-     * method, property getter, or property setter.
-     * @return true if the given documentation belongs to a getter.
-     */
-    private boolean isGetter(Element element) {
-        final String pedName = element.getSimpleName().toString();
-        return pedName.startsWith("get") || pedName.startsWith("is");
-    }
-
-    /**
-     * Test whether the method is a setter.
-     * @param element property method documentation. Needs to be either property
-     * method, property getter, or property setter.
-     * @return true if the given documentation belongs to a setter.
-     */
-    private boolean isSetter(Element element) {
-        return element.getSimpleName().toString().startsWith("set");
-    }
-
     /**
      * Build the inherited member summary for the given methods.
      *
      * @param writer the writer for this member summary.
      * @param kind the kind of members to document.
-     * @param summaryTreeList list of content trees to which the documentation will be added
+     * @param targets the list of contents to which the documentation will be added
      */
     private void buildInheritedSummary(MemberSummaryWriter writer,
-            VisibleMemberTable.Kind kind, LinkedList<Content> summaryTreeList) {
+            VisibleMemberTable.Kind kind, LinkedList<Content> targets) {
         VisibleMemberTable visibleMemberTable = getVisibleMemberTable();
         SortedSet<? extends Element> inheritedMembersFromMap = asSortedSet(visibleMemberTable.getAllVisibleMembers(kind));
 
@@ -409,29 +298,33 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
             if (inheritedClass == typeElement) {
                 continue;
             }
+            if (utils.hasHiddenTag(inheritedClass)) {
+                continue;
+            }
 
-            List<Element> members = inheritedMembersFromMap.stream()
+            List<? extends Element> members = inheritedMembersFromMap.stream()
                     .filter(e -> utils.getEnclosingTypeElement(e) == inheritedClass)
-                    .collect(Collectors.toList());
+                    .toList();
+
             if (!members.isEmpty()) {
                 SortedSet<Element> inheritedMembers = new TreeSet<>(comparator);
                 inheritedMembers.addAll(members);
-                Content inheritedTree = writer.getInheritedSummaryHeader(inheritedClass);
-                Content linksTree = writer.getInheritedSummaryLinksTree();
-                addSummaryFootNote(inheritedClass, inheritedMembers, linksTree, writer);
-                inheritedTree.add(linksTree);
-                summaryTreeList.add(inheritedTree);
+                Content inheritedHeader = writer.getInheritedSummaryHeader(inheritedClass);
+                Content links = writer.getInheritedSummaryLinks();
+                addSummaryFootNote(inheritedClass, inheritedMembers, links, writer);
+                inheritedHeader.add(links);
+                targets.add(inheritedHeader);
             }
         }
     }
 
     private void addSummaryFootNote(TypeElement inheritedClass, SortedSet<Element> inheritedMembers,
-                                    Content linksTree, MemberSummaryWriter writer) {
+                                    Content links, MemberSummaryWriter writer) {
         for (Element member : inheritedMembers) {
             TypeElement t = utils.isUndocumentedEnclosure(inheritedClass)
                     ? typeElement : inheritedClass;
             writer.addInheritedMemberSummary(t, member, inheritedMembers.first() == member,
-                    inheritedMembers.last() == member, linksTree);
+                    inheritedMembers.last() == member, links);
         }
     }
 
@@ -453,9 +346,9 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
         if (showInheritedSummary)
             buildInheritedSummary(writer, kind, summaryTreeList);
         if (!summaryTreeList.isEmpty()) {
-            Content memberTree = writer.getMemberSummaryHeader(typeElement, summariesList);
-            summaryTreeList.forEach(memberTree::add);
-            writer.addSummary(summariesList, memberTree);
+            Content member = writer.getMemberSummaryHeader(typeElement, summariesList);
+            summaryTreeList.forEach(member::add);
+            writer.addSummary(summariesList, member);
         }
     }
 
@@ -465,6 +358,19 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
         return out;
     }
 
+    /**
+     * A utility class to manage the property-related methods that should be
+     * synthesized or updated.
+     *
+     * A property may comprise a field (that is typically private, if present),
+     * a {@code fooProperty()} method (which is the defining characteristic for
+     * a property), a {@code getFoo()} method and/or a {@code setFoo(Foo foo)} method.
+     *
+     * Either the field (if present) or the {@code fooProperty()} method should have a
+     * comment. If there is no field, or no comment on the field, the description for
+     * the property will be derived from the description of the {@code fooProperty()}
+     * method. If any method does not have a comment, one will be provided.
+     */
     static class PropertyHelper {
 
         private final Map<Element, Element> classPropertiesMap = new HashMap<>();
@@ -492,22 +398,28 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
                                         VariableElement field,
                                         ExecutableElement getter,
                                         ExecutableElement setter) {
-            if (field == null || builder.utils.getDocCommentTree(field) == null) {
-                addToPropertiesMap(propertyMethod, propertyMethod);
-                addToPropertiesMap(getter, propertyMethod);
-                addToPropertiesMap(setter, propertyMethod);
-            } else {
-                addToPropertiesMap(propertyMethod, field);
-                addToPropertiesMap(getter, field);
-                addToPropertiesMap(setter, field);
+            // determine the preferred element from which to derive the property description
+            Element e = field == null || !builder.utils.hasDocCommentTree(field)
+                    ? propertyMethod : field;
+
+            if (e == field && builder.utils.hasDocCommentTree(propertyMethod)) {
+                BaseConfiguration configuration = builder.configuration;
+                configuration.getReporter().print(Diagnostic.Kind.WARNING,
+                        propertyMethod, configuration.getDocResources().getText("doclet.duplicate.comment.for.property"));
             }
+
+            addToPropertiesMap(propertyMethod, e);
+            addToPropertiesMap(getter, e);
+            addToPropertiesMap(setter, e);
         }
 
         private void addToPropertiesMap(Element propertyMethod,
                                         Element commentSource) {
-            if (null == propertyMethod || null == commentSource) {
+            Objects.requireNonNull(commentSource);
+            if (propertyMethod == null) {
                 return;
             }
+
             Utils utils = builder.utils;
             DocCommentTree docTree = utils.hasDocCommentTree(propertyMethod)
                     ? utils.getDocCommentTree(propertyMethod)
@@ -523,30 +435,12 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
         }
 
         /**
-         * Returns the property field documentation belonging to the given member.
+         * Returns the element for the property documentation belonging to the given member.
          * @param element the member for which the property documentation is needed.
-         * @return the property field documentation, null if there is none.
+         * @return the element for the property documentation, null if there is none.
          */
         public Element getPropertyElement(Element element) {
             return classPropertiesMap.get(element);
         }
-
-        /**
-         * Returns the getter documentation belonging to the given property method.
-         * @param propertyMethod the method for which the getter is needed.
-         * @return the getter documentation, null if there is none.
-         */
-        public ExecutableElement getGetterForProperty(ExecutableElement propertyMethod) {
-            return builder.getVisibleMemberTable().getPropertyGetter(propertyMethod);
-        }
-
-        /**
-         * Returns the setter documentation belonging to the given property method.
-         * @param propertyMethod the method for which the setter is needed.
-         * @return the setter documentation, null if there is none.
-         */
-        public ExecutableElement getSetterForProperty(ExecutableElement propertyMethod) {
-            return builder.getVisibleMemberTable().getPropertySetter(propertyMethod);
-        }
     }
 }
