@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -58,11 +58,6 @@ import jdk.javadoc.internal.doclets.toolkit.Resources;
  * generates "package-list"(lists all the packages which are getting
  * documented) file in the current or the destination directory, while
  * generating the documentation.
- *
- *  <p><b>This is NOT part of any supported API.
- *  If you write code that depends on this, you do so at your own risk.
- *  This code and its internal interfaces are subject to change or
- *  deletion without notice.</b>
  */
 public class Extern {
 
@@ -108,11 +103,6 @@ public class Extern {
          */
         final boolean relative;
 
-        /**
-         * Indicates that docs use old-form of anchors.
-         */
-        final boolean useOldFormId;
-
         /**
          * Constructor to build a Extern Item object and map it with the element name.
          * If the same element name is found in the map, then the first mapped
@@ -123,11 +113,10 @@ public class Extern {
          * file is picked.
          * @param relative    True if path is URL, false if directory path.
          */
-        Item(String elementName, DocPath path, boolean relative, boolean useOldFormId) {
+        Item(String elementName, DocPath path, boolean relative) {
             this.elementName = elementName;
             this.path = path;
             this.relative = relative;
-            this.useOldFormId = useOldFormId;
         }
 
         /**
@@ -197,7 +186,7 @@ public class Extern {
         DocPath p = fnd.relative ?
                 relativepath.resolve(fnd.path).resolve(filename) :
                 fnd.path.resolve(filename);
-        return new DocLink(p, fnd.useOldFormId ? getOldFormHtmlName(memberName) : memberName);
+        return new DocLink(p, memberName);
     }
 
     /**
@@ -232,7 +221,7 @@ public class Extern {
 
     /**
      * Check whether links to platform documentation are configured. If not then configure
-     * links using the the documentation URL defined in {@code linkPlatformProperties} or the
+     * links using the documentation URL defined in {@code linkPlatformProperties} or the
      * default documentation URL if that parameter is {@code null}.
      *
      * @param linkPlatformProperties path or URL to properties file containing
@@ -267,7 +256,7 @@ public class Extern {
                 reporter.print(Kind.WARNING, resources.getText("doclet.Resource_error", elementListPath.getPath()));
             } else {
                 try (InputStream in = open(elementListUrl)) {
-                    readElementList(in, docUrl, false, versionNumber, isOldFormPlatformDocs(versionNumber));
+                    readElementList(in, docUrl, false, versionNumber);
                 } catch (IOException exc) {
                     throw new Fault(resources.getText(
                             "doclet.Resource_error", elementListPath.getPath()), exc);
@@ -278,18 +267,6 @@ public class Extern {
         }
     }
 
-    /**
-     * Checks if platform docs for the specified version use old-form anchors.
-     * Old-form anchors are used by Oracle docs for JDKs 8 and 9.
-     * It can be checked on https://docs.oracle.com/javase/<version>/docs/api
-     *
-     * @param version
-     * @return True if docs use old-form anchors
-     */
-    private boolean isOldFormPlatformDocs(int version) {
-        return 8 == version || 9 == version;
-    }
-
     /**
      * Return the resource path for the package or element list for the given {@code version}.
      * @param version the platform version number
@@ -442,7 +419,7 @@ public class Extern {
         try {
             URL link = elemlisturlpath.toURI().resolve(DocPaths.ELEMENT_LIST.getPath()).toURL();
             try (InputStream in = open(link)) {
-                readElementList(in, urlpath, false, 0, false);
+                readElementList(in, urlpath, false, 0);
             }
         } catch (URISyntaxException | MalformedURLException exc) {
             throw new Fault(resources.getText("doclet.MalformedURL", elemlisturlpath.toString()), exc);
@@ -461,7 +438,7 @@ public class Extern {
         try {
             URL link = elemlisturlpath.toURI().resolve(DocPaths.PACKAGE_LIST.getPath()).toURL();
             try (InputStream in = open(link)) {
-                readElementList(in, urlpath, false, 0, true);
+                readElementList(in, urlpath, false, 0);
             }
         } catch (URISyntaxException | MalformedURLException exc) {
             throw new Fault(resources.getText("doclet.MalformedURL", elemlisturlpath.toString()), exc);
@@ -485,27 +462,27 @@ public class Extern {
             file = file.resolveAgainst(DocumentationTool.Location.DOCUMENTATION_OUTPUT);
         }
         if (file.exists()) {
-            readElementList(file, path, false);
+            readElementList(file, path);
         } else {
             DocFile file1 = elemListPath.resolve(DocPaths.PACKAGE_LIST);
             if (!(file1.isAbsolute() || linkoffline)) {
                 file1 = file1.resolveAgainst(DocumentationTool.Location.DOCUMENTATION_OUTPUT);
             }
             if (file1.exists()) {
-                readElementList(file1, path, true);
+                readElementList(file1, path);
             } else {
                 throw new Fault(resources.getText("doclet.File_error", file.getPath()), null);
             }
         }
     }
 
-    private void readElementList(DocFile file, String path, boolean isOldFormDoc) throws Fault, DocFileIOException {
+    private void readElementList(DocFile file, String path) throws Fault, DocFileIOException {
         try {
             if (file.canRead()) {
                 boolean pathIsRelative
                         = !isUrl(path)
                         && !DocFile.createFileForInput(configuration, path).isAbsolute();
-                readElementList(file.openInputStream(), path, pathIsRelative, 0, isOldFormDoc);
+                readElementList(file.openInputStream(), path, pathIsRelative, 0);
             } else {
                 throw new Fault(resources.getText("doclet.File_error", file.getPath()), null);
             }
@@ -525,21 +502,20 @@ public class Extern {
      *                        or {@code 0} if it does not belong to a platform libraries doc bundle.
      * @throws IOException if there is a problem reading or closing the stream
      */
-    private void readElementList(InputStream input, String path, boolean relative, int platformVersion,
-                                 boolean isOldFormDoc)
+    private void readElementList(InputStream input, String path, boolean relative, int platformVersion)
                          throws IOException {
         try (BufferedReader in = new BufferedReader(new InputStreamReader(input))) {
             String elemname;
             DocPath elempath;
             String moduleName = null;
             DocPath basePath  = DocPath.create(path);
-            boolean issueWarning = true;
+            boolean showDiagnostic = true;
             while ((elemname = in.readLine()) != null) {
                 if (elemname.length() > 0) {
                     elempath = basePath;
                     if (elemname.startsWith(DocletConstants.MODULE_PREFIX)) {
                         moduleName = elemname.replace(DocletConstants.MODULE_PREFIX, "");
-                        Item item = new Item(moduleName, elempath, relative, isOldFormDoc);
+                        Item item = new Item(moduleName, elempath, relative);
                         moduleItems.put(moduleName, item);
                     } else {
                         DocPath pkgPath = DocPath.create(elemname.replace('.', '/'));
@@ -553,14 +529,14 @@ public class Extern {
                         // For user provided libraries we check whether modularity matches the actual library.
                         // We trust modularity to be correct for platform library element lists.
                         if (platformVersion == 0) {
-                            actualModuleName = checkLinkCompatibility(elemname, moduleName, path, issueWarning);
+                            actualModuleName = checkLinkCompatibility(elemname, moduleName, path, showDiagnostic);
                         } else {
                             actualModuleName = moduleName == null ? DocletConstants.DEFAULT_ELEMENT_NAME : moduleName;
                         }
-                        Item item = new Item(elemname, elempath, relative, isOldFormDoc);
+                        Item item = new Item(elemname, elempath, relative);
                         packageItems.computeIfAbsent(actualModuleName, k -> new TreeMap<>())
                             .putIfAbsent(elemname, item); // first-one-wins semantics
-                        issueWarning = false;
+                        showDiagnostic = false;
                     }
                 }
             }
@@ -575,25 +551,23 @@ public class Extern {
      * @param packageName the package name
      * @param moduleName the module name or null
      * @param path the documentation path
-     * @param issueWarning whether to print a warning in case of modularity mismatch
+     * @param showDiagnostic whether to print a diagnostic message in case of modularity mismatch
      * @return the module name to use according to actual modularity of the package
      */
-    private String checkLinkCompatibility(String packageName, String moduleName, String path, boolean issueWarning)  {
+    private String checkLinkCompatibility(String packageName, String moduleName, String path, boolean showDiagnostic)  {
         PackageElement pe = utils.elementUtils.getPackageElement(packageName);
         if (pe != null) {
             ModuleElement me = (ModuleElement)pe.getEnclosingElement();
             if (me == null || me.isUnnamed()) {
-                if (moduleName != null && issueWarning) {
-                    configuration.getReporter().print(Kind.WARNING,
-                            resources.getText("doclet.linkMismatch_PackagedLinkedtoModule", path));
+                if (moduleName != null && showDiagnostic) {
+                    printModularityMismatchDiagnostic("doclet.linkMismatch_PackagedLinkedtoModule", path);
                 }
                 // library is not modular, ignore module name even if documentation is modular
                 return DocletConstants.DEFAULT_ELEMENT_NAME;
             } else if (moduleName == null) {
-                // suppress the warning message in the case of automatic modules
-                if (!utils.elementUtils.isAutomaticModule(me) && issueWarning) {
-                    configuration.getReporter().print(Kind.WARNING,
-                            resources.getText("doclet.linkMismatch_ModuleLinkedtoPackage", path));
+                // suppress the diagnostic message in the case of automatic modules
+                if (!utils.elementUtils.isAutomaticModule(me) && showDiagnostic) {
+                    printModularityMismatchDiagnostic("doclet.linkMismatch_ModuleLinkedtoPackage", path);
                 }
                 // library is modular, use module name for lookup even though documentation is not
                 return utils.getModuleName(me);
@@ -604,7 +578,8 @@ public class Extern {
 
     public boolean isUrl (String urlCandidate) {
         try {
-            new URL(urlCandidate);
+            @SuppressWarnings("deprecation")
+            var _unused = new URL(urlCandidate);
             //No exception was thrown, so this must really be a URL.
             return true;
         } catch (MalformedURLException e) {
@@ -613,6 +588,7 @@ public class Extern {
         }
     }
 
+    @SuppressWarnings("deprecation")
     private URL toURL(String url) throws Fault {
         try {
             return new URL(url);
@@ -658,7 +634,8 @@ public class Extern {
                         String loc = http.getHeaderField("Location");
                         URL target = null;
                         if (loc != null) {
-                            target = new URL(base, loc);
+                            @SuppressWarnings("deprecation")
+                            var _unused = target = new URL(base, loc);
                         }
                         http.disconnect();
                         if (target == null || redirects >= 5) {
@@ -679,64 +656,10 @@ public class Extern {
         return in;
     }
 
-    /**
-     * Converts a name to an old-form HTML name (old-form id).
-     *
-     * @param name the string that needs to be converted to a valid HTML name
-     * @return old-form HTML name
-     */
-    private String getOldFormHtmlName(String name) {
-        /* The HTML 4 spec at http://www.w3.org/TR/html4/types.html#h-6.2 mentions
-         * that the name/id should begin with a letter followed by other valid characters.
-         * The HTML 5 spec (draft) is more permissive on names/ids where the only restriction
-         * is that it should be at least one character long and should not contain spaces.
-         * The spec draft is @ http://www.w3.org/html/wg/drafts/html/master/dom.html#the-id-attribute.
-         *
-         * For HTML 4, we need to check for non-characters at the beginning of the name and
-         * substitute it accordingly, "_" and "$" can appear at the beginning of a member name.
-         * The method substitutes "$" with "Z:Z:D" and will prefix "_" with "Z:Z".
-         */
-
-        if (null == name)
-            return name;
-
-        StringBuilder sb = new StringBuilder();
-        for (int i = 0; i < name.length(); i++) {
-            char ch = name.charAt(i);
-            switch (ch) {
-                case '(':
-                case ')':
-                case '<':
-                case '>':
-                case ',':
-                    sb.append('-');
-                    break;
-                case ' ':
-                case '[':
-                    break;
-                case ']':
-                    sb.append(":A");
-                    break;
-                // Any appearance of $ needs to be substituted with ":D" and not with hyphen
-                // since a field name "P$$ and a method P(), both valid member names, can end
-                // up as "P--". A member name beginning with $ needs to be substituted with
-                // "Z:Z:D".
-                case '$':
-                    if (i == 0)
-                        sb.append("Z:Z");
-                    sb.append(":D");
-                    break;
-                // A member name beginning with _ needs to be prefixed with "Z:Z" since valid anchor
-                // names can only begin with a letter.
-                case '_':
-                    if (i == 0)
-                        sb.append("Z:Z");
-                    sb.append(ch);
-                    break;
-                default:
-                    sb.append(ch);
-            }
+    private void printModularityMismatchDiagnostic(String key, Object arg) {
+        switch (configuration.getOptions().linkModularityMismatch()) {
+            case INFO -> configuration.getMessages().notice(key, arg);
+            case WARN -> configuration.getMessages().warning(key, arg);
         }
-        return sb.toString();
     }
 }
