@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -26,7 +26,6 @@
 package jdk.jfr;
 
 import java.lang.annotation.Annotation;
-import java.util.Arrays;
 import java.util.Collections;
 import java.util.LinkedHashMap;
 import java.util.List;
@@ -45,8 +44,8 @@ import jdk.jfr.internal.Utils;
  * @since 9
  */
 public final class EventType {
+    private static final List<String> UNCATEGORIZED = List.of("Uncategorized");
     private final PlatformEventType platformEventType;
-    private final List<String> UNCATEGORIZED = Collections.singletonList("Uncategorized");
     private Map<String, ValueDescriptor> cache; // create lazy to avoid memory overhead
     // helper constructor
     EventType(PlatformEventType platformEventType) {
@@ -66,21 +65,31 @@ public final class EventType {
     /**
      * Returns the field with the specified name, or {@code null} if it doesn't
      * exist.
+     * <p>
+     * It's possible to index into a nested field by using {@code "."} (for
+     * instance {@code "thread.group.parent.name}").
+     *
+     * @param name of the field to get, not {@code null}
      *
      * @return a value descriptor that describes the field, or {@code null} if
      *         the field with the specified name doesn't exist
      */
     public ValueDescriptor getField(String name) {
-        Objects.requireNonNull(name);
+        Objects.requireNonNull(name, "name");
         if (cache == null) {
             List<ValueDescriptor> fields = getFields();
-            Map<String, ValueDescriptor> newCache = new LinkedHashMap<String, ValueDescriptor>(fields.size());
+            Map<String, ValueDescriptor> newCache = LinkedHashMap.newLinkedHashMap(fields.size());
             for (ValueDescriptor v :fields) {
                 newCache.put(v.getName(), v);
             }
-            cache = newCache;
+            cache = Map.copyOf(newCache);
         }
-        return cache.get(name);
+        ValueDescriptor result = cache.get(name);
+        if (result == null) {
+            // Cache doesn't contain subfields
+            result = platformEventType.getField(name);
+        }
+        return result;
     }
 
     /**
@@ -172,7 +181,7 @@ public final class EventType {
      *         directly present, else {@code null}
      */
     public <A extends Annotation> A getAnnotation(Class<A> annotationClass) {
-        Objects.requireNonNull(annotationClass);
+        Objects.requireNonNull(annotationClass, "annotationClass");
         return platformEventType.getAnnotation(annotationClass);
     }
 
@@ -189,7 +198,7 @@ public final class EventType {
      *         {@code Registered(false)}, but not manually registered
      */
     public static EventType getEventType(Class<? extends Event> eventClass) {
-        Objects.requireNonNull(eventClass);
+        Objects.requireNonNull(eventClass, "eventClass");
         Utils.ensureValidEventSubclass(eventClass);
         JVMSupport.ensureWithInternalError();
         return MetadataRepository.getInstance().getEventType(eventClass);
@@ -220,7 +229,7 @@ public final class EventType {
         if (c == null) {
             return UNCATEGORIZED;
         }
-        return Collections.unmodifiableList(Arrays.asList(c.value()));
+        return List.of(c.value());
     }
 
     // package private
@@ -232,4 +241,9 @@ public final class EventType {
     PlatformEventType getPlatformEventType() {
         return platformEventType;
     }
+
+    // package private
+    boolean isVisible() {
+        return platformEventType.isVisible();
+    }
 }
