@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -33,8 +33,8 @@
  *      file system provider} to enumerate and read the class and resource
  *      files in a run-time image.
  *      The jrt file system can be created by calling
- *      {@link java.nio.file.FileSystems#newFileSystem
- *      FileSystems.newFileSystem(URI.create("jrt:/"))}.
+ *      {@link java.nio.file.FileSystems#getFileSystem
+ *      FileSystems.getFileSystem(URI.create("jrt:/"))}.
  *      </dd>
  * </dl>
  *
@@ -79,6 +79,7 @@ module java.base {
     exports java.lang;
     exports java.lang.annotation;
     exports java.lang.constant;
+    exports java.lang.foreign;
     exports java.lang.invoke;
     exports java.lang.module;
     exports java.lang.ref;
@@ -137,17 +138,28 @@ module java.base {
         java.naming,
         java.rmi,
         jdk.charsets,
-        jdk.incubator.foreign,
+        jdk.crypto.cryptoki,
+        jdk.incubator.concurrent,
         jdk.jartool,
+        jdk.jfr,
         jdk.jlink,
-        jdk.net;
-    exports jdk.internal.access.foreign to jdk.incubator.foreign;
+        jdk.net,
+        jdk.sctp;
     exports jdk.internal.event to jdk.jfr;
-    exports jdk.internal.invoke to jdk.incubator.foreign;
+    exports jdk.internal.foreign to jdk.incubator.vector;
+    exports jdk.internal.io to
+        jdk.internal.le,
+        jdk.jshell;
     exports jdk.internal.javac to
         java.compiler,
+        java.management,
         jdk.compiler,
-        jdk.jshell;
+        jdk.incubator.concurrent,
+        jdk.incubator.vector,
+        jdk.jdi,
+        jdk.jfr,
+        jdk.jshell,
+        jdk.management;
     exports jdk.internal.jimage to jdk.jlink;
     exports jdk.internal.jimage.decompressor to jdk.jlink;
     exports jdk.internal.jmod to
@@ -156,8 +168,7 @@ module java.base {
     exports jdk.internal.loader to
         java.instrument,
         java.logging,
-        java.naming,
-        jdk.incubator.foreign;
+        java.naming;
     exports jdk.internal.logger to java.logging;
     exports jdk.internal.misc to
         java.desktop,
@@ -171,7 +182,7 @@ module java.base {
         jdk.charsets,
         jdk.compiler,
         jdk.crypto.cryptoki,
-        jdk.incubator.foreign,
+        jdk.incubator.concurrent,
         jdk.incubator.vector,
         jdk.internal.vm.ci,
         jdk.jfr,
@@ -181,7 +192,6 @@ module java.base {
     exports jdk.internal.module to
         java.instrument,
         java.management.rmi,
-        jdk.incubator.foreign,
         jdk.jartool,
         jdk.jfr,
         jdk.jlink,
@@ -189,7 +199,8 @@ module java.base {
     exports jdk.internal.org.objectweb.asm to
         jdk.jartool,
         jdk.jfr,
-        jdk.jlink;
+        jdk.jlink,
+        jdk.jshell;
     exports jdk.internal.org.objectweb.asm.commons to jdk.jfr;
     exports jdk.internal.org.objectweb.asm.tree to
         jdk.jfr,
@@ -206,34 +217,33 @@ module java.base {
         jdk.management;
     exports jdk.internal.ref to
         java.desktop,
-        jdk.incubator.foreign;
+        java.net.http;
     exports jdk.internal.reflect to
         java.logging,
         java.sql,
         java.sql.rowset,
         jdk.dynalink,
-        jdk.incubator.foreign,
         jdk.internal.vm.ci,
         jdk.unsupported;
-    exports jdk.internal.util to jdk.incubator.foreign;
     exports jdk.internal.util.jar to jdk.jartool;
     exports jdk.internal.util.random to jdk.random;
     exports jdk.internal.util.xml to jdk.jfr;
     exports jdk.internal.util.xml.impl to jdk.jfr;
     exports jdk.internal.vm to
+        java.management,
+        jdk.incubator.concurrent,
         jdk.internal.jvmstat,
+        jdk.management,
         jdk.management.agent;
     exports jdk.internal.vm.annotation to
         java.instrument,
-        jdk.incubator.foreign,
+        jdk.incubator.concurrent,
         jdk.incubator.vector,
         jdk.internal.vm.ci,
         jdk.jfr,
         jdk.unsupported;
     exports jdk.internal.vm.vector to jdk.incubator.vector;
-    exports sun.invoke.util to
-        jdk.compiler,
-        jdk.incubator.foreign;
+    exports sun.invoke.util to jdk.compiler;
     exports sun.net to
         java.net.http,
         jdk.naming.dns;
@@ -253,7 +263,6 @@ module java.base {
     exports sun.nio.ch to
         java.management,
         jdk.crypto.cryptoki,
-        jdk.incubator.foreign,
         jdk.net,
         jdk.sctp;
     exports sun.nio.cs to
@@ -273,7 +282,7 @@ module java.base {
         java.desktop,
         java.security.jgss,
         jdk.crypto.ec,
-        jdk.incubator.foreign;
+        jdk.incubator.concurrent;
     exports sun.security.internal.interfaces to jdk.crypto.cryptoki;
     exports sun.security.internal.spec to jdk.crypto.cryptoki;
     exports sun.security.jca to
@@ -356,6 +365,7 @@ module java.base {
     uses java.util.spi.TimeZoneNameProvider;
     uses java.util.spi.ToolProvider;
     uses javax.security.auth.spi.LoginModule;
+    uses jdk.internal.io.JdkConsoleProvider;
     uses jdk.internal.logger.DefaultLoggerFinder;
     uses sun.text.spi.JavaTimeDateTimePatternProvider;
     uses sun.util.locale.provider.LocaleDataMetaInfo;
