@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -45,6 +45,9 @@
  * @toolGuide javadoc
  *
  * @provides java.util.spi.ToolProvider
+ *     Use {@link java.util.spi.ToolProvider#findFirst ToolProvider.findFirst("javadoc")}
+ *     to obtain an instance of a {@code ToolProvider} that provides the equivalent
+ *     of command-line access to the {@code javadoc} tool.
  * @provides javax.tools.DocumentationTool
  * @provides javax.tools.Tool
  *
@@ -70,4 +73,7 @@ module jdk.javadoc {
 
     provides javax.tools.Tool with
         jdk.javadoc.internal.api.JavadocTool;
+
+    provides com.sun.tools.doclint.DocLint with
+            jdk.javadoc.internal.doclint.DocLint;
 }
