@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -55,7 +55,7 @@ public class ZCollectedHeap extends CollectedHeap {
 
     public ZHeap heap() {
         Address heapAddr = addr.addOffsetTo(zHeapFieldOffset);
-        return (ZHeap)VMObjectFactory.newObject(ZHeap.class, heapAddr);
+        return VMObjectFactory.newObject(ZHeap.class, heapAddr);
     }
 
     @Override
@@ -82,7 +82,10 @@ public class ZCollectedHeap extends CollectedHeap {
         return heap().used();
     }
 
-
+    @Override
+    public boolean isInReserved(Address a) {
+        return heap().isIn(a);
+    }
 
     private OopHandle oop_load_barrier(Address oopAddress) {
         oopAddress = ZBarrier.weak_barrier(oopAddress);
@@ -106,7 +109,6 @@ public class ZCollectedHeap extends CollectedHeap {
     @Override
     public OopHandle oop_load_in_native(Address addr) {
         Address oopAddress = addr.getAddressAt(0);
-
         return oop_load_barrier(oopAddress);
     }
 
