@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -75,10 +75,6 @@ class UnixUriUtils {
         int pos = 0;
         while (pos < len) {
             char c = p.charAt(pos++);
-            if ((c == '/') && (pos < len) && (p.charAt(pos) == '/')) {
-                // skip redundant slashes
-                continue;
-            }
             byte b;
             if (c == '%') {
                 assert (pos+2) <= len;
@@ -92,6 +88,10 @@ class UnixUriUtils {
                     throw new IllegalArgumentException("Bad escape");
                 b = (byte)c;
             }
+            if (b == '/' && rlen > 0 && result[rlen-1] == '/') {
+                // skip redundant slashes
+                continue;
+            }
             result[rlen++] = b;
         }
         if (rlen != result.length)
@@ -122,10 +122,11 @@ class UnixUriUtils {
         if (sb.charAt(sb.length()-1) != '/') {
             try {
                 up.checkRead();
-                int mode = UnixNativeDispatcher.stat(up);
-                if ((mode & UnixConstants.S_IFMT) == UnixConstants.S_IFDIR)
+                UnixFileAttributes attrs = UnixFileAttributes.getIfExists(up);
+                if (attrs != null
+                        && ((attrs.mode() & UnixConstants.S_IFMT) == UnixConstants.S_IFDIR))
                     sb.append('/');
-            } catch (SecurityException ignore) { }
+            } catch (UnixException | SecurityException ignore) { }
         }
 
         try {
