@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2019, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -35,35 +35,50 @@ import java.security.spec.*;
 import javax.crypto.*;
 import javax.crypto.spec.*;
 
+import jdk.internal.access.JavaNioAccess;
+import jdk.internal.access.SharedSecrets;
 import sun.nio.ch.DirectBuffer;
 import sun.security.jca.JCAUtil;
 import sun.security.pkcs11.wrapper.*;
 import static sun.security.pkcs11.wrapper.PKCS11Constants.*;
+import static sun.security.pkcs11.wrapper.PKCS11Exception.RV.*;
 
 /**
  * P11 AEAD Cipher implementation class. This class currently supports
- * AES with GCM mode.
+ * AES cipher in GCM mode and CHACHA20-POLY1305 cipher.
  *
  * Note that AEAD modes do not use padding, so this class does not have
- * its own padding impl. In addition, NSS CKM_AES_GCM only supports single-part
- * encryption/decryption, thus the current impl uses PKCS#11 C_Encrypt/C_Decrypt
- * calls and buffers data until doFinal is called.
- *
- * Note that PKCS#11 standard currently only supports GCM and CCM AEAD modes.
- * There are no provisions for other AEAD modes yet.
+ * its own padding impl. In addition, some vendors such as NSS may not support
+ * multi-part encryption/decryption for AEAD cipher algorithms, thus the
+ * current impl uses PKCS#11 C_Encrypt/C_Decrypt calls and buffers data until
+ * doFinal is called.
  *
  * @since   13
  */
 final class P11AEADCipher extends CipherSpi {
 
-    // mode constant for GCM mode
-    private static final int MODE_GCM = 10;
+    private static final JavaNioAccess NIO_ACCESS = SharedSecrets.getJavaNioAccess();
 
-    // default constants for GCM
-    private static final int GCM_DEFAULT_TAG_LEN = 16;
-    private static final int GCM_DEFAULT_IV_LEN = 16;
+    // supported AEAD algorithms/transformations
+    private enum Transformation {
+        AES_GCM("AES", "GCM", "NOPADDING", 16, 16),
+        CHACHA20_POLY1305("CHACHA20", "NONE", "NOPADDING", 12, 16);
 
-    private static final String ALGO = "AES";
+        final String keyAlgo;
+        final String mode;
+        final String padding;
+        final int defIvLen; // in bytes
+        final int defTagLen; // in bytes
+
+        Transformation(String keyAlgo, String mode, String padding,
+                int defIvLen, int defTagLen) {
+            this.keyAlgo = keyAlgo;
+            this.mode = mode;
+            this.padding = padding;
+            this.defIvLen = defIvLen;
+            this.defTagLen = defTagLen;
+        }
+    }
 
     // token instance
     private final Token token;
@@ -71,10 +86,10 @@ final class P11AEADCipher extends CipherSpi {
     // mechanism id
     private final long mechanism;
 
-    // mode, one of MODE_* above
-    private final int blockMode;
+    // type of this AEAD cipher, one of Transformation enum above
+    private final Transformation type;
 
-    // acceptable key size, -1 if more than 1 key sizes are accepted
+    // acceptable key size in bytes, -1 if more than 1 key sizes are accepted
     private final int fixedKeySize;
 
     // associated session, if any
@@ -86,7 +101,7 @@ final class P11AEADCipher extends CipherSpi {
     // flag indicating whether an operation is initialized
     private boolean initialized = false;
 
-    // falg indicating encrypt or decrypt mode
+    // flag indicating encrypt or decrypt mode
     private boolean encrypt = true;
 
     // parameters
@@ -111,99 +126,128 @@ final class P11AEADCipher extends CipherSpi {
         this.mechanism = mechanism;
 
         String[] algoParts = algorithm.split("/");
-        if (algoParts.length != 3) {
-            throw new ProviderException("Unsupported Transformation format: " +
-                    algorithm);
-        }
-        if (!algoParts[0].startsWith("AES")) {
-            throw new ProviderException("Only support AES for AEAD cipher mode");
-        }
-        int index = algoParts[0].indexOf('_');
-        if (index != -1) {
-            // should be well-formed since we specify what we support
-            fixedKeySize = Integer.parseInt(algoParts[0].substring(index+1)) >> 3;
+        if (algoParts[0].startsWith("AES")) {
+            // for AES_GCM, need 3 parts
+            if (algoParts.length != 3) {
+                throw new AssertionError("Invalid Transformation format: " +
+                        algorithm);
+            }
+            int index = algoParts[0].indexOf('_');
+            if (index != -1) {
+                // should be well-formed since we specify what we support
+                fixedKeySize = Integer.parseInt(algoParts[0].substring(index+1)) >> 3;
+            } else {
+                fixedKeySize = -1;
+            }
+            this.type = Transformation.AES_GCM;
+            engineSetMode(algoParts[1]);
+            try {
+                engineSetPadding(algoParts[2]);
+            } catch (NoSuchPaddingException e) {
+                throw new NoSuchAlgorithmException(e);
+            }
+        } else if (algoParts[0].equals("ChaCha20-Poly1305")) {
+            fixedKeySize = 32;
+            this.type = Transformation.CHACHA20_POLY1305;
+            if (algoParts.length > 3) {
+                throw new AssertionError(
+                        "Invalid Transformation format: " + algorithm);
+            } else {
+                if (algoParts.length > 1) {
+                    engineSetMode(algoParts[1]);
+                }
+                try {
+                    if (algoParts.length > 2) {
+                        engineSetPadding(algoParts[2]);
+                    }
+                } catch (NoSuchPaddingException e) {
+                    throw new NoSuchAlgorithmException();
+                }
+            }
         } else {
-            fixedKeySize = -1;
-        }
-        this.blockMode = parseMode(algoParts[1]);
-        if (!algoParts[2].equals("NoPadding")) {
-            throw new ProviderException("Only NoPadding is supported for AEAD cipher mode");
+            throw new AssertionError("Unsupported transformation " + algorithm);
         }
     }
 
+    @Override
     protected void engineSetMode(String mode) throws NoSuchAlgorithmException {
-        // Disallow change of mode for now since currently it's explicitly
-        // defined in transformation strings
-        throw new NoSuchAlgorithmException("Unsupported mode " + mode);
-    }
-
-    private int parseMode(String mode) throws NoSuchAlgorithmException {
-        mode = mode.toUpperCase(Locale.ENGLISH);
-        int result;
-        if (mode.equals("GCM")) {
-            result = MODE_GCM;
-        } else {
+        if (!mode.toUpperCase(Locale.ENGLISH).equals(type.mode)) {
             throw new NoSuchAlgorithmException("Unsupported mode " + mode);
         }
-        return result;
     }
 
     // see JCE spec
+    @Override
     protected void engineSetPadding(String padding)
             throws NoSuchPaddingException {
-        // Disallow change of padding for now since currently it's explicitly
-        // defined in transformation strings
-        throw new NoSuchPaddingException("Unsupported padding " + padding);
+        if (!padding.toUpperCase(Locale.ENGLISH).equals(type.padding)) {
+            throw new NoSuchPaddingException("Unsupported padding " + padding);
+        }
     }
 
     // see JCE spec
+    @Override
     protected int engineGetBlockSize() {
-        return 16; // constant; only AES is supported
+        return switch (type) {
+            case AES_GCM -> 16;
+            case CHACHA20_POLY1305 -> 0;
+            default -> throw new AssertionError("Unsupported type " + type);
+        };
     }
 
     // see JCE spec
+    @Override
     protected int engineGetOutputSize(int inputLen) {
         return doFinalLength(inputLen);
     }
 
     // see JCE spec
+    @Override
     protected byte[] engineGetIV() {
         return (iv == null) ? null : iv.clone();
     }
 
     // see JCE spec
     protected AlgorithmParameters engineGetParameters() {
-        if (encrypt && iv == null && tagLen == -1) {
-            switch (blockMode) {
-                case MODE_GCM:
-                    iv = new byte[GCM_DEFAULT_IV_LEN];
-                    tagLen = GCM_DEFAULT_TAG_LEN;
-                    break;
-                default:
-                    throw new ProviderException("Unsupported mode");
-            }
-            random.nextBytes(iv);
-        }
-        try {
-            AlgorithmParameterSpec spec;
-            String apAlgo;
-            switch (blockMode) {
-                case MODE_GCM:
-                    apAlgo = "GCM";
+        String apAlgo;
+        AlgorithmParameterSpec spec = null;
+        switch (type) {
+            case AES_GCM -> {
+                apAlgo = "GCM";
+                if (encrypt && iv == null && tagLen == -1) {
+                    iv = new byte[type.defIvLen];
+                    tagLen = type.defTagLen;
+                    random.nextBytes(iv);
+                }
+                if (iv != null) {
                     spec = new GCMParameterSpec(tagLen << 3, iv);
-                    break;
-                default:
-                    throw new ProviderException("Unsupported mode");
+                }
             }
-            AlgorithmParameters params =
-                AlgorithmParameters.getInstance(apAlgo);
-            params.init(spec);
-            return params;
-        } catch (GeneralSecurityException e) {
-            // NoSuchAlgorithmException, NoSuchProviderException
-            // InvalidParameterSpecException
-            throw new ProviderException("Could not encode parameters", e);
+            case CHACHA20_POLY1305 -> {
+                if (encrypt && iv == null) {
+                    iv = new byte[type.defIvLen];
+                    random.nextBytes(iv);
+                }
+                apAlgo = "ChaCha20-Poly1305";
+                if (iv != null) {
+                    spec = new IvParameterSpec(iv);
+                }
+            }
+            default -> throw new AssertionError("Unsupported type " + type);
         }
+        if (spec != null) {
+            try {
+                AlgorithmParameters params =
+                    AlgorithmParameters.getInstance(apAlgo);
+                params.init(spec);
+                return params;
+            } catch (GeneralSecurityException e) {
+                // NoSuchAlgorithmException, NoSuchProviderException
+                // InvalidParameterSpecException
+                throw new ProviderException("Could not encode parameters", e);
+            }
+        }
+        return null;
     }
 
     // see JCE spec
@@ -231,20 +275,30 @@ final class P11AEADCipher extends CipherSpi {
         updateCalled = false;
         byte[] ivValue = null;
         int tagLen = -1;
-        if (params != null) {
-            switch (blockMode) {
-            case MODE_GCM:
-                if (!(params instanceof GCMParameterSpec)) {
-                    throw new InvalidAlgorithmParameterException
-                        ("Only GCMParameterSpec is supported");
+        switch (type) {
+            case AES_GCM:
+                if (params != null) {
+                    if (!(params instanceof GCMParameterSpec)) {
+                        throw new InvalidAlgorithmParameterException
+                                ("Only GCMParameterSpec is supported");
+                    }
+                    ivValue = ((GCMParameterSpec) params).getIV();
+                    tagLen = ((GCMParameterSpec) params).getTLen() >> 3;
+                }
+            break;
+            case CHACHA20_POLY1305:
+                if (params != null) {
+                    if (!(params instanceof IvParameterSpec)) {
+                        throw new InvalidAlgorithmParameterException
+                                ("Only IvParameterSpec is supported");
+                    }
+                    ivValue = ((IvParameterSpec) params).getIV();
+                    tagLen = type.defTagLen;
                 }
-                ivValue = ((GCMParameterSpec) params).getIV();
-                tagLen = ((GCMParameterSpec) params).getTLen() >> 3;
             break;
             default:
-                throw new ProviderException("Unsupported mode");
-            }
-        }
+                throw new AssertionError("Unsupported type " + type);
+        };
         implInit(opmode, key, ivValue, tagLen, sr);
     }
 
@@ -260,14 +314,10 @@ final class P11AEADCipher extends CipherSpi {
         try {
             AlgorithmParameterSpec paramSpec = null;
             if (params != null) {
-                switch (blockMode) {
-                    case MODE_GCM:
-                        paramSpec =
-                            params.getParameterSpec(GCMParameterSpec.class);
-                        break;
-                    default:
-                        throw new ProviderException("Unsupported mode");
-                }
+                paramSpec = switch (type) {
+                    case AES_GCM -> params.getParameterSpec(GCMParameterSpec.class);
+                    case CHACHA20_POLY1305 -> params.getParameterSpec(IvParameterSpec.class);
+                };
             }
             engineInit(opmode, key, paramSpec, sr);
         } catch (InvalidParameterSpecException ex) {
@@ -285,41 +335,39 @@ final class P11AEADCipher extends CipherSpi {
                             key.getEncoded().length) != fixedKeySize) {
             throw new InvalidKeyException("Key size is invalid");
         }
-        P11Key newKey = P11SecretKeyFactory.convertKey(token, key, ALGO);
+        P11Key newKey = P11SecretKeyFactory.convertKey(token, key,
+                type.keyAlgo);
         switch (opmode) {
-            case Cipher.ENCRYPT_MODE:
+            case Cipher.ENCRYPT_MODE -> {
                 encrypt = true;
                 requireReinit = Arrays.equals(iv, lastEncIv) &&
                         (newKey == lastEncKey);
                 if (requireReinit) {
-                    throw new InvalidAlgorithmParameterException
-                        ("Cannot reuse iv for GCM encryption");
+                    throw new InvalidAlgorithmParameterException(
+                            "Cannot reuse the same key and iv pair");
                 }
-                break;
-            case Cipher.DECRYPT_MODE:
+            }
+            case Cipher.DECRYPT_MODE -> {
                 encrypt = false;
                 requireReinit = false;
-                break;
-            default:
-                throw new InvalidAlgorithmParameterException
-                        ("Unsupported mode: " + opmode);
+            }
+            case Cipher.WRAP_MODE, Cipher.UNWRAP_MODE -> throw new UnsupportedOperationException
+                    ("Unsupported mode: " + opmode);
+            default ->
+                // should never happen; checked by Cipher.init()
+                    throw new AssertionError("Unknown mode: " + opmode);
         }
 
         // decryption without parameters is checked in all engineInit() calls
         if (sr != null) {
             this.random = sr;
         }
+
         if (iv == null && tagLen == -1) {
             // generate default values
-            switch (blockMode) {
-                case MODE_GCM:
-                    iv = new byte[GCM_DEFAULT_IV_LEN];
-                    this.random.nextBytes(iv);
-                    tagLen = GCM_DEFAULT_TAG_LEN;
-                    break;
-                default:
-                    throw new ProviderException("Unsupported mode");
-            }
+            iv = new byte[type.defIvLen];
+            this.random.nextBytes(iv);
+            tagLen = type.defTagLen;
         }
         this.iv = iv;
         this.tagLen = tagLen;
@@ -327,7 +375,7 @@ final class P11AEADCipher extends CipherSpi {
         try {
             initialize();
         } catch (PKCS11Exception e) {
-            if (e.getErrorCode() == CKR_MECHANISM_PARAM_INVALID) {
+            if (e.match(CKR_MECHANISM_PARAM_INVALID)) {
                 throw new InvalidAlgorithmParameterException("Bad params", e);
             }
             throw new InvalidKeyException("Could not initialize cipher", e);
@@ -335,7 +383,13 @@ final class P11AEADCipher extends CipherSpi {
     }
 
     private void cancelOperation() {
-        // cancel operation by finishing it; avoid killSession as some
+        token.ensureValid();
+        if (P11Util.trySessionCancel(token, session,
+                (encrypt ? CKF_ENCRYPT : CKF_DECRYPT))) {
+            return;
+        }
+
+        // cancel by finishing operations; avoid killSession as some
         // hardware vendors may require re-login
         int bufLen = doFinalLength(0);
         byte[] buffer = new byte[bufLen];
@@ -350,6 +404,13 @@ final class P11AEADCipher extends CipherSpi {
                         0, buffer, 0, bufLen);
             }
         } catch (PKCS11Exception e) {
+            if (e.match(CKR_OPERATION_NOT_INITIALIZED)) {
+                // Cancel Operation may be invoked after an error on a PKCS#11
+                // call. If the operation inside the token was already cancelled,
+                // do not fail here. This is part of a defensive mechanism for
+                // PKCS#11 libraries that do not strictly follow the standard.
+                return;
+            }
             if (encrypt) {
                 throw new ProviderException("Cancel failed", e);
             }
@@ -375,24 +436,21 @@ final class P11AEADCipher extends CipherSpi {
         }
         if (requireReinit) {
             throw new IllegalStateException
-                ("Must use either different key or iv for GCM encryption");
+                ("Must use either different key or iv");
         }
 
         token.ensureValid();
 
-        byte[] aad = (aadBuffer.size() > 0? aadBuffer.toByteArray() : null);
+        byte[] aad = (aadBuffer.size() > 0 ? aadBuffer.toByteArray() : null);
 
         long p11KeyID = p11Key.getKeyID();
         try {
-            CK_MECHANISM mechWithParams;
-            switch (blockMode) {
-                case MODE_GCM:
-                    mechWithParams = new CK_MECHANISM(mechanism,
+            CK_MECHANISM mechWithParams = switch (type) {
+                case AES_GCM -> new CK_MECHANISM(mechanism,
                         new CK_GCM_PARAMS(tagLen << 3, iv, aad));
-                    break;
-                default:
-                    throw new ProviderException("Unsupported mode: " + blockMode);
-            }
+                case CHACHA20_POLY1305 -> new CK_MECHANISM(mechanism,
+                        new CK_SALSA20_CHACHA20_POLY1305_PARAMS(iv, aad));
+            };
             if (session == null) {
                 session = token.getOpSession();
             }
@@ -424,10 +482,13 @@ final class P11AEADCipher extends CipherSpi {
         if (encrypt) {
             result += tagLen;
         } else {
-            // PKCS11Exception: CKR_BUFFER_TOO_SMALL
-            //result -= tagLen;
+            // In earlier NSS versions, AES_GCM would report
+            // CKR_BUFFER_TOO_SMALL error if minus tagLen
+            if (type == Transformation.CHACHA20_POLY1305) {
+                result -= tagLen;
+            }
         }
-        return result;
+        return (Math.max(result, 0));
     }
 
     // reset the states to the pre-initialized values
@@ -485,7 +546,7 @@ final class P11AEADCipher extends CipherSpi {
         }
         if (requireReinit) {
             throw new IllegalStateException
-                ("Must use either different key or iv for GCM encryption");
+                ("Must use either different key or iv for encryption");
         }
         if (p11Key == null) {
             throw new IllegalStateException("Need to initialize Cipher first");
@@ -588,6 +649,7 @@ final class P11AEADCipher extends CipherSpi {
         if (outLen < requiredOutLen) {
             throw new ShortBufferException();
         }
+
         boolean doCancel = true;
         try {
             ensureInitialized();
@@ -616,6 +678,12 @@ final class P11AEADCipher extends CipherSpi {
             }
             return k;
         } catch (PKCS11Exception e) {
+            // As per the PKCS#11 standard, C_Encrypt and C_Decrypt may only
+            // keep the operation active on CKR_BUFFER_TOO_SMALL errors or
+            // successful calls to determine the output length. However,
+            // these cases are not expected here because the output length
+            // is checked in the OpenJDK side before making the PKCS#11 call.
+            // Thus, doCancel can safely be 'false'.
             doCancel = false;
             handleException(e);
             throw new ProviderException("doFinal() failed", e);
@@ -641,96 +709,112 @@ final class P11AEADCipher extends CipherSpi {
         }
 
         boolean doCancel = true;
+        NIO_ACCESS.acquireSession(inBuffer);
         try {
-            ensureInitialized();
+            NIO_ACCESS.acquireSession(outBuffer);
+            try {
+                try {
+                    ensureInitialized();
 
-            long inAddr = 0;
-            byte[] in = null;
-            int inOfs = 0;
-            if (dataBuffer.size() > 0) {
-                if (inLen > 0) {
-                    byte[] temp = new byte[inLen];
-                    inBuffer.get(temp);
-                    dataBuffer.write(temp, 0, temp.length);
-                }
-                in = dataBuffer.toByteArray();
-                inOfs = 0;
-                inLen = in.length;
-            } else {
-                if (inBuffer instanceof DirectBuffer) {
-                    inAddr = ((DirectBuffer) inBuffer).address();
-                    inOfs = inBuffer.position();
-                } else {
-                    if (inBuffer.hasArray()) {
-                        in = inBuffer.array();
-                        inOfs = inBuffer.position() + inBuffer.arrayOffset();
+                    long inAddr = 0;
+                    byte[] in = null;
+                    int inOfs = 0;
+                    if (dataBuffer.size() > 0) {
+                        if (inLen > 0) {
+                            byte[] temp = new byte[inLen];
+                            inBuffer.get(temp);
+                            dataBuffer.write(temp, 0, temp.length);
+                        }
+                        in = dataBuffer.toByteArray();
+                        inOfs = 0;
+                        inLen = in.length;
                     } else {
-                        in = new byte[inLen];
-                        inBuffer.get(in);
+                        if (inBuffer instanceof DirectBuffer dInBuffer) {
+                            inAddr = dInBuffer.address();
+                            inOfs = inBuffer.position();
+                        } else {
+                            if (inBuffer.hasArray()) {
+                                in = inBuffer.array();
+                                inOfs = inBuffer.position() + inBuffer.arrayOffset();
+                            } else {
+                                in = new byte[inLen];
+                                inBuffer.get(in);
+                            }
+                        }
+                    }
+                    long outAddr = 0;
+                    byte[] outArray = null;
+                    int outOfs = 0;
+                    if (outBuffer instanceof DirectBuffer dOutBuffer) {
+                        outAddr = dOutBuffer.address();
+                        outOfs = outBuffer.position();
+                    } else {
+                        if (outBuffer.hasArray()) {
+                            outArray = outBuffer.array();
+                            outOfs = outBuffer.position() + outBuffer.arrayOffset();
+                        } else {
+                            outArray = new byte[outLen];
+                        }
                     }
-                }
-            }
-            long outAddr = 0;
-            byte[] outArray = null;
-            int outOfs = 0;
-            if (outBuffer instanceof DirectBuffer) {
-                outAddr = ((DirectBuffer) outBuffer).address();
-                outOfs = outBuffer.position();
-            } else {
-                if (outBuffer.hasArray()) {
-                    outArray = outBuffer.array();
-                    outOfs = outBuffer.position() + outBuffer.arrayOffset();
-                } else {
-                    outArray = new byte[outLen];
-                }
-            }
 
-            int k = 0;
-            if (encrypt) {
-                k = token.p11.C_Encrypt(session.id(), inAddr, in, inOfs, inLen,
-                        outAddr, outArray, outOfs, outLen);
-                doCancel = false;
-            } else {
-                // Special handling to match SunJCE provider behavior
-                if (inLen == 0) {
-                    return 0;
+                    int k = 0;
+                    if (encrypt) {
+                        k = token.p11.C_Encrypt(session.id(), inAddr, in, inOfs, inLen,
+                                outAddr, outArray, outOfs, outLen);
+                        doCancel = false;
+                    } else {
+                        // Special handling to match SunJCE provider behavior
+                        if (inLen == 0) {
+                            return 0;
+                        }
+                        k = token.p11.C_Decrypt(session.id(), inAddr, in, inOfs, inLen,
+                                outAddr, outArray, outOfs, outLen);
+                        doCancel = false;
+                    }
+                    inBuffer.position(inBuffer.limit());
+                    outBuffer.position(outBuffer.position() + k);
+                    return k;
+                } catch (PKCS11Exception e) {
+                    // As per the PKCS#11 standard, C_Encrypt and C_Decrypt may only
+                    // keep the operation active on CKR_BUFFER_TOO_SMALL errors or
+                    // successful calls to determine the output length. However,
+                    // these cases are not expected here because the output length
+                    // is checked in the OpenJDK side before making the PKCS#11 call.
+                    // Thus, doCancel can safely be 'false'.
+                    doCancel = false;
+                    handleException(e);
+                    throw new ProviderException("doFinal() failed", e);
+                } finally {
+                    if (encrypt) {
+                        lastEncKey = this.p11Key;
+                        lastEncIv = this.iv;
+                        requireReinit = true;
+                    }
+                    reset(doCancel);
                 }
-                k = token.p11.C_Decrypt(session.id(), inAddr, in, inOfs, inLen,
-                        outAddr, outArray, outOfs, outLen);
-                doCancel = false;
+            } finally {
+                NIO_ACCESS.releaseSession(outBuffer);
             }
-            outBuffer.position(outBuffer.position() + k);
-            return k;
-        } catch (PKCS11Exception e) {
-            doCancel = false;
-            handleException(e);
-            throw new ProviderException("doFinal() failed", e);
         } finally {
-            if (encrypt) {
-                lastEncKey = this.p11Key;
-                lastEncIv = this.iv;
-                requireReinit = true;
-            }
-            reset(doCancel);
+            NIO_ACCESS.releaseSession(inBuffer);
         }
     }
 
     private void handleException(PKCS11Exception e)
             throws ShortBufferException, IllegalBlockSizeException,
             BadPaddingException {
-        long errorCode = e.getErrorCode();
-        if (errorCode == CKR_BUFFER_TOO_SMALL) {
+        if (e.match(CKR_BUFFER_TOO_SMALL)) {
             throw (ShortBufferException)
                     (new ShortBufferException().initCause(e));
-        } else if (errorCode == CKR_DATA_LEN_RANGE ||
-                   errorCode == CKR_ENCRYPTED_DATA_LEN_RANGE) {
+        } else if (e.match(CKR_DATA_LEN_RANGE) ||
+                e.match(CKR_ENCRYPTED_DATA_LEN_RANGE)) {
             throw (IllegalBlockSizeException)
                     (new IllegalBlockSizeException(e.toString()).initCause(e));
-        } else if (errorCode == CKR_ENCRYPTED_DATA_INVALID ||
-                // Solaris-specific
-                errorCode == CKR_GENERAL_ERROR) {
-            throw (BadPaddingException)
-                    (new BadPaddingException(e.toString()).initCause(e));
+        } else if (e.match(CKR_ENCRYPTED_DATA_INVALID) ||
+                e.match(CKR_GENERAL_ERROR)) {
+            // CKR_GENERAL_ERROR is Solaris-specific workaround
+            throw (AEADBadTagException)
+                    (new AEADBadTagException(e.toString()).initCause(e));
         }
     }
 
@@ -753,7 +837,7 @@ final class P11AEADCipher extends CipherSpi {
     @Override
     protected int engineGetKeySize(Key key) throws InvalidKeyException {
         int n = P11SecretKeyFactory.convertKey
-                (token, key, ALGO).length();
+                (token, key, type.keyAlgo).length();
         return n;
     }
 }
