@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -31,7 +31,8 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.IOException;
 import java.io.ByteArrayInputStream;
-import java.io.UnsupportedEncodingException;
+
+import static java.nio.charset.StandardCharsets.UTF_8;
 
 import java.util.Arrays;
 import java.util.Collections;
@@ -56,6 +57,7 @@ import java.security.spec.*;
 import javax.crypto.SecretKey;
 import javax.crypto.interfaces.*;
 
+import javax.security.auth.DestroyFailedException;
 import javax.security.auth.x500.X500Principal;
 import javax.security.auth.login.LoginException;
 import javax.security.auth.callback.Callback;
@@ -72,6 +74,7 @@ import static sun.security.pkcs11.P11Util.*;
 
 import sun.security.pkcs11.wrapper.*;
 import static sun.security.pkcs11.wrapper.PKCS11Constants.*;
+import static sun.security.pkcs11.wrapper.PKCS11Exception.RV.*;
 
 import sun.security.rsa.RSAKeyFactory;
 
@@ -229,6 +232,7 @@ final class P11KeyStore extends KeyStoreSpi {
         private PasswordCallbackHandler(char[] password) {
             if (password != null) {
                 this.password = password.clone();
+                P11Util.cleaner.register(this, releaserFor(this.password));
             }
         }
 
@@ -241,12 +245,10 @@ final class P11KeyStore extends KeyStoreSpi {
             pc.setPassword(password);  // this clones the password if not null
         }
 
-        @SuppressWarnings("deprecation")
-        protected void finalize() throws Throwable {
-            if (password != null) {
+        private static Runnable releaserFor(char[] password) {
+            return () -> {
                 Arrays.fill(password, ' ');
-            }
-            super.finalize();
+            };
         }
     }
 
@@ -453,7 +455,18 @@ final class P11KeyStore extends KeyStoreSpi {
         } catch (NullPointerException | IllegalArgumentException e) {
             throw new KeyStoreException(e);
         }
-        engineSetEntry(alias, entry, new KeyStore.PasswordProtection(password));
+
+        KeyStore.PasswordProtection passwordProtection =
+                new KeyStore.PasswordProtection(password);
+        try {
+            engineSetEntry(alias, entry, passwordProtection);
+        } finally {
+            try {
+                passwordProtection.destroy();
+            } catch (DestroyFailedException dfe) {
+                // ignore
+            }
+        }
     }
 
     /**
@@ -755,7 +768,7 @@ final class P11KeyStore extends KeyStoreSpi {
             Throwable cause = e.getCause();
             if (cause instanceof PKCS11Exception) {
                 PKCS11Exception pe = (PKCS11Exception) cause;
-                if (pe.getErrorCode() == CKR_PIN_INCORRECT) {
+                if (pe.match(CKR_PIN_INCORRECT)) {
                     // if password is wrong, the cause of the IOException
                     // should be an UnrecoverableKeyException
                     throw new IOException("load failed",
@@ -766,12 +779,14 @@ final class P11KeyStore extends KeyStoreSpi {
         }
 
         try {
-            if (mapLabels() == true) {
+            if (mapLabels()) {
                 // CKA_LABELs are shared by multiple certs
                 writeDisabled = true;
             }
             if (debug != null) {
                 dumpTokenMap();
+                debug.println("P11KeyStore load. Entry count: " +
+                        aliasMap.size());
             }
         } catch (KeyStoreException | PKCS11Exception e) {
             throw new IOException("load failed", e);
@@ -846,7 +861,7 @@ final class P11KeyStore extends KeyStoreSpi {
 
         try {
             login(handler);
-            if (mapLabels() == true) {
+            if (mapLabels()) {
                 // CKA_LABELs are shared by multiple certs
                 writeDisabled = true;
             }
@@ -900,9 +915,8 @@ final class P11KeyStore extends KeyStoreSpi {
 
         token.ensureValid();
 
-        if (protParam != null &&
-            protParam instanceof KeyStore.PasswordProtection &&
-            ((KeyStore.PasswordProtection)protParam).getPassword() != null &&
+        if (protParam instanceof PasswordProtection pp &&
+            pp.getPassword() != null &&
             !token.config.getKeyStoreCompatibilityMode()) {
             throw new KeyStoreException("ProtectionParameter must be null");
         }
@@ -1003,9 +1017,8 @@ final class P11KeyStore extends KeyStoreSpi {
         token.ensureValid();
         checkWrite();
 
-        if (protParam != null &&
-            protParam instanceof KeyStore.PasswordProtection &&
-            ((KeyStore.PasswordProtection)protParam).getPassword() != null &&
+        if (protParam instanceof PasswordProtection pp &&
+            pp.getPassword() != null &&
             !token.config.getKeyStoreCompatibilityMode()) {
             throw new KeyStoreException(new UnsupportedOperationException
                                 ("ProtectionParameter must be null"));
@@ -1017,7 +1030,7 @@ final class P11KeyStore extends KeyStoreSpi {
 
         if (entry instanceof KeyStore.TrustedCertificateEntry) {
 
-            if (useSecmodTrust == false) {
+            if (!useSecmodTrust) {
                 // PKCS #11 does not allow app to modify trusted certs -
                 throw new KeyStoreException(new UnsupportedOperationException
                                     ("trusted certificates may only be set by " +
@@ -1030,10 +1043,9 @@ final class P11KeyStore extends KeyStoreSpi {
                     + "added to the NSS KeyStore module");
             }
             Certificate cert = ((TrustedCertificateEntry)entry).getTrustedCertificate();
-            if (cert instanceof X509Certificate == false) {
+            if (!(cert instanceof X509Certificate xcert)) {
                 throw new KeyStoreException("Certificate must be an X509Certificate");
             }
-            X509Certificate xcert = (X509Certificate)cert;
             AliasInfo info = aliasMap.get(alias);
             if (info != null) {
                 // XXX try to update
@@ -1109,9 +1121,8 @@ final class P11KeyStore extends KeyStoreSpi {
                     throw new KeyStoreException(pe);
                 }
 
-            } else if (entry instanceof KeyStore.SecretKeyEntry) {
+            } else if (entry instanceof SecretKeyEntry ske) {
 
-                KeyStore.SecretKeyEntry ske = (KeyStore.SecretKeyEntry)entry;
                 SecretKey skey = ske.getSecretKey();
 
                 try {
@@ -1202,7 +1213,7 @@ final class P11KeyStore extends KeyStoreSpi {
             // self signed
             return new X509Certificate[] { endCert };
         } else {
-            lChain = new ArrayList<X509Certificate>();
+            lChain = new ArrayList<>();
             lChain.add(endCert);
         }
 
@@ -1319,7 +1330,7 @@ final class P11KeyStore extends KeyStoreSpi {
                 RSAKeyFactory.checkKeyLengths(keyLength, null,
                     -1, Integer.MAX_VALUE);
             } catch (InvalidKeyException e) {
-                throw new KeyStoreException(e.getMessage());
+                throw new KeyStoreException(e.getMessage(), e);
             }
 
             return P11Key.privateKey(session,
@@ -1431,7 +1442,7 @@ final class P11KeyStore extends KeyStoreSpi {
                 throw new KeyStoreException
                         ("expected but could not find private key " +
                         "with CKA_ID " +
-                        getID(cka_id));
+                        getIDNullSafe(cka_id));
             }
 
             // next find existing end entity cert
@@ -1441,7 +1452,7 @@ final class P11KeyStore extends KeyStoreSpi {
                 throw new KeyStoreException
                         ("expected but could not find certificate " +
                         "with CKA_ID " +
-                        getID(cka_id));
+                        getIDNullSafe(cka_id));
             } else {
                 if (replaceCert) {
                     // replacing existing cert and chain
@@ -1485,6 +1496,7 @@ final class P11KeyStore extends KeyStoreSpi {
         }
     }
 
+    // retrieves the native key handle and either update it directly or make a copy
     private void updateP11Pkey(String alias, CK_ATTRIBUTE attribute, P11Key key)
                 throws PKCS11Exception {
 
@@ -1492,23 +1504,22 @@ final class P11KeyStore extends KeyStoreSpi {
         // if session key, convert to token key.
 
         Session session = null;
+        long keyID = key.getKeyID();
         try {
             session = token.getOpSession();
-            if (key.tokenObject == true) {
-
+            if (key.tokenObject) {
                 // token key - set new CKA_ID
 
                 CK_ATTRIBUTE[] attrs = new CK_ATTRIBUTE[] {
                                 new CK_ATTRIBUTE(CKA_ID, alias) };
                 token.p11.C_SetAttributeValue
-                                (session.id(), key.keyID, attrs);
+                                (session.id(), keyID, attrs);
                 if (debug != null) {
                     debug.println("updateP11Pkey set new alias [" +
                                 alias +
                                 "] for key entry");
                 }
             } else {
-
                 // session key - convert to token key and set CKA_ID
 
                 CK_ATTRIBUTE[] attrs = new CK_ATTRIBUTE[] {
@@ -1518,7 +1529,8 @@ final class P11KeyStore extends KeyStoreSpi {
                 if (attribute != null) {
                     attrs = addAttribute(attrs, attribute);
                 }
-                token.p11.C_CopyObject(session.id(), key.keyID, attrs);
+                // creates a new token key with the desired CKA_ID
+                token.p11.C_CopyObject(session.id(), keyID, attrs);
                 if (debug != null) {
                     debug.println("updateP11Pkey copied private session key " +
                                 "for [" +
@@ -1528,13 +1540,14 @@ final class P11KeyStore extends KeyStoreSpi {
             }
         } finally {
             token.releaseSession(session);
+            key.releaseKeyID();
         }
     }
 
     private void storeCert(String alias, X509Certificate cert)
                 throws PKCS11Exception, CertificateException {
 
-        ArrayList<CK_ATTRIBUTE> attrList = new ArrayList<CK_ATTRIBUTE>();
+        ArrayList<CK_ATTRIBUTE> attrList = new ArrayList<>();
         attrList.add(ATTR_TOKEN_TRUE);
         attrList.add(ATTR_CLASS_CERT);
         attrList.add(ATTR_X509_CERT_TYPE);
@@ -1658,8 +1671,7 @@ final class P11KeyStore extends KeyStoreSpi {
         // If the key is a token object on this token, update it instead
         // of creating a duplicate key object.
         // Otherwise, treat a P11Key like any other key, if it is extractable.
-        if (key instanceof P11Key) {
-            P11Key p11Key = (P11Key)key;
+        if (key instanceof P11Key p11Key) {
             if (p11Key.tokenObject && (p11Key.token == this.token)) {
                 updateP11Pkey(alias, null, p11Key);
                 storeChain(alias, (X509Certificate[])pke.getCertificateChain());
@@ -1676,9 +1688,7 @@ final class P11KeyStore extends KeyStoreSpi {
             attrs = getRsaPrivKeyAttrs
                 (alias, (RSAPrivateKey)key, cert.getSubjectX500Principal());
 
-        } else if (key instanceof DSAPrivateKey) {
-
-            DSAPrivateKey dsaKey = (DSAPrivateKey)key;
+        } else if (key instanceof DSAPrivateKey dsaKey) {
 
             CK_ATTRIBUTE[] idAttrs = getIdAttributes(key, publicKey, false, useNDB);
             if (idAttrs[0] == null) {
@@ -1707,9 +1717,7 @@ final class P11KeyStore extends KeyStoreSpi {
                 debug.println("storePkey created DSA template");
             }
 
-        } else if (key instanceof DHPrivateKey) {
-
-            DHPrivateKey dhKey = (DHPrivateKey)key;
+        } else if (key instanceof DHPrivateKey dhKey) {
 
             CK_ATTRIBUTE[] idAttrs = getIdAttributes(key, publicKey, false, useNDB);
             if (idAttrs[0] == null) {
@@ -1733,9 +1741,7 @@ final class P11KeyStore extends KeyStoreSpi {
             attrs = token.getAttributes
                 (TemplateManager.O_IMPORT, CKO_PRIVATE_KEY, CKK_DH, attrs);
 
-        } else if (key instanceof ECPrivateKey) {
-
-            ECPrivateKey ecKey = (ECPrivateKey)key;
+        } else if (key instanceof ECPrivateKey ecKey) {
 
             CK_ATTRIBUTE[] idAttrs = getIdAttributes(key, publicKey, false, useNDB);
             if (idAttrs[0] == null) {
@@ -1764,9 +1770,8 @@ final class P11KeyStore extends KeyStoreSpi {
                 debug.println("storePkey created EC template");
             }
 
-        } else if (key instanceof P11Key) {
+        } else if (key instanceof P11Key p11Key) {
             // sensitive/non-extractable P11Key
-            P11Key p11Key = (P11Key)key;
             if (p11Key.token != this.token) {
                 throw new KeyStoreException
                     ("Cannot move sensitive keys across tokens");
@@ -1816,14 +1821,12 @@ final class P11KeyStore extends KeyStoreSpi {
         // subject is currently ignored - could be used to set CKA_SUBJECT
 
         CK_ATTRIBUTE[] attrs = null;
-        if (key instanceof RSAPrivateCrtKey) {
+        if (key instanceof RSAPrivateCrtKey rsaKey) {
 
             if (debug != null) {
                 debug.println("creating RSAPrivateCrtKey attrs");
             }
 
-            RSAPrivateCrtKey rsaKey = (RSAPrivateCrtKey)key;
-
             attrs = new CK_ATTRIBUTE[] {
                 ATTR_TOKEN_TRUE,
                 ATTR_CLASS_PKEY,
@@ -1855,8 +1858,6 @@ final class P11KeyStore extends KeyStoreSpi {
                 debug.println("creating RSAPrivateKey attrs");
             }
 
-            RSAPrivateKey rsaKey = key;
-
             attrs = new CK_ATTRIBUTE[] {
                 ATTR_TOKEN_TRUE,
                 ATTR_CLASS_PKEY,
@@ -1864,9 +1865,9 @@ final class P11KeyStore extends KeyStoreSpi {
                 new CK_ATTRIBUTE(CKA_KEY_TYPE, CKK_RSA),
                 new CK_ATTRIBUTE(CKA_ID, alias),
                 new CK_ATTRIBUTE(CKA_MODULUS,
-                                rsaKey.getModulus()),
+                                key.getModulus()),
                 new CK_ATTRIBUTE(CKA_PRIVATE_EXPONENT,
-                                rsaKey.getPrivateExponent()) };
+                                key.getPrivateExponent()) };
             attrs = token.getAttributes
                 (TemplateManager.O_IMPORT, CKO_PRIVATE_KEY, CKK_RSA, attrs);
         }
@@ -1890,14 +1891,16 @@ final class P11KeyStore extends KeyStoreSpi {
     private CK_ATTRIBUTE[] getIdAttributes(PrivateKey privateKey,
             PublicKey publicKey, boolean id, boolean netscapeDb) {
         CK_ATTRIBUTE[] attrs = new CK_ATTRIBUTE[2];
-        if ((id || netscapeDb) == false) {
+        if (!(id || netscapeDb)) {
             return attrs;
         }
         String alg = privateKey.getAlgorithm();
-        if (id && alg.equals("RSA") && (publicKey instanceof RSAPublicKey)) {
+        if (alg.equals("RSA") && (publicKey instanceof RSAPublicKey)) {
+            if (id) {
+                BigInteger n = ((RSAPublicKey)publicKey).getModulus();
+                attrs[0] = new CK_ATTRIBUTE(CKA_ID, sha1(getMagnitude(n)));
+            }
             // CKA_NETSCAPE_DB not needed for RSA public keys
-            BigInteger n = ((RSAPublicKey)publicKey).getModulus();
-            attrs[0] = new CK_ATTRIBUTE(CKA_ID, sha1(getMagnitude(n)));
         } else if (alg.equals("DSA") && (publicKey instanceof DSAPublicKey)) {
             BigInteger y = ((DSAPublicKey)publicKey).getY();
             if (id) {
@@ -1947,8 +1950,8 @@ final class P11KeyStore extends KeyStoreSpi {
             token.p11.C_DestroyObject(session.id(), h.handle);
             if (debug != null) {
                 debug.println("destroyCert destroyed cert with CKA_ID [" +
-                                                getID(cka_id) +
-                                                "]");
+                        getIDNullSafe(cka_id) +
+                        "]");
             }
             return true;
         } finally {
@@ -1982,7 +1985,7 @@ final class P11KeyStore extends KeyStoreSpi {
             if (debug != null) {
                 debug.println("destroyChain destroyed end entity cert " +
                         "with CKA_ID [" +
-                        getID(cka_id) +
+                        getIDNullSafe(cka_id) +
                         "]");
             }
 
@@ -1993,7 +1996,7 @@ final class P11KeyStore extends KeyStoreSpi {
 
                 if (next.getSubjectX500Principal().equals
                     (next.getIssuerX500Principal())) {
-                    // self signed - done
+                    // self-signed - done
                     break;
                 }
 
@@ -2107,7 +2110,7 @@ final class P11KeyStore extends KeyStoreSpi {
                 if (debug != null) {
                     debug.println
                         ("destroyPkey did not find private key with CKA_ID [" +
-                        getID(cka_id) +
+                        getIDNullSafe(cka_id) +
                         "]");
                 }
                 return false;
@@ -2148,14 +2151,17 @@ final class P11KeyStore extends KeyStoreSpi {
         if (!printable) {
             return "0x" + Functions.toHexString(bytes);
         } else {
-            try {
-                return new String(bytes, "UTF-8");
-            } catch (UnsupportedEncodingException uee) {
-                return "0x" + Functions.toHexString(bytes);
-            }
+            return new String(bytes, UTF_8);
         }
     }
 
+    /**
+     * Null safe version of getID.
+     */
+    private static String getIDNullSafe(byte[] bytes) {
+        return (bytes != null) ? getID(bytes) : "null";
+    }
+
     /**
      * find an object on the token
      *
@@ -2192,12 +2198,12 @@ final class P11KeyStore extends KeyStoreSpi {
                 } else if (type == ATTR_CLASS_CERT) {
                     debug.println
                         ("getTokenObject did not find cert with CKA_ID [" +
-                        getID(cka_id) +
+                        getIDNullSafe(cka_id) +
                         "]");
                 } else {
                     debug.println("getTokenObject did not find private key " +
                         "with CKA_ID [" +
-                        getID(cka_id) +
+                        getIDNullSafe(cka_id) +
                         "]");
                 }
             }
@@ -2213,7 +2219,7 @@ final class P11KeyStore extends KeyStoreSpi {
 
             if (type == ATTR_CLASS_SKEY) {
 
-                ArrayList<THandle> list = new ArrayList<THandle>(h.length);
+                ArrayList<THandle> list = new ArrayList<>(h.length);
                 for (int i = 0; i < h.length; i++) {
 
                     CK_ATTRIBUTE[] label = new CK_ATTRIBUTE[]
@@ -2240,13 +2246,13 @@ final class P11KeyStore extends KeyStoreSpi {
                         "found " +
                         h.length +
                         " certificates sharing CKA_ID " +
-                        getID(cka_id));
+                        getIDNullSafe(cka_id));
             } else {
                 throw new KeyStoreException("invalid KeyStore state: " +
                         "found " +
                         h.length +
                         " private keys sharing CKA_ID " +
-                        getID(cka_id));
+                        getIDNullSafe(cka_id));
             }
         }
         return new THandle(NO_HANDLE, null);
@@ -2279,7 +2285,7 @@ final class P11KeyStore extends KeyStoreSpi {
 
             // get all private key CKA_IDs
 
-            ArrayList<byte[]> pkeyIDs = new ArrayList<byte[]>();
+            ArrayList<byte[]> pkeyIDs = new ArrayList<>();
             CK_ATTRIBUTE[] attrs = new CK_ATTRIBUTE[] {
                 ATTR_TOKEN_TRUE,
                 ATTR_CLASS_PKEY,
@@ -2306,7 +2312,7 @@ final class P11KeyStore extends KeyStoreSpi {
             // (multiple certs may be mapped to a single CKA_LABEL)
 
             HashMap<String, HashSet<AliasInfo>> certMap =
-                                new HashMap<String, HashSet<AliasInfo>>();
+                                new HashMap<>();
 
             attrs = new CK_ATTRIBUTE[] {
                 ATTR_TOKEN_TRUE,
@@ -2326,7 +2332,7 @@ final class P11KeyStore extends KeyStoreSpi {
                         cka_label = new String(attrs[0].getCharArray());
                     }
                 } catch (PKCS11Exception pe) {
-                    if (pe.getErrorCode() != CKR_ATTRIBUTE_TYPE_INVALID) {
+                    if (!pe.match(CKR_ATTRIBUTE_TYPE_INVALID)) {
                         throw pe;
                     }
 
@@ -2367,7 +2373,7 @@ final class P11KeyStore extends KeyStoreSpi {
                                     (session.id(), handle, trustedAttr);
                             cka_trusted = trustedAttr[0].getBoolean();
                         } catch (PKCS11Exception pe) {
-                            if (pe.getErrorCode() == CKR_ATTRIBUTE_TYPE_INVALID) {
+                            if (pe.match(CKR_ATTRIBUTE_TYPE_INVALID)) {
                                 // XXX  NSS, ibutton, sca1000
                                 CKA_TRUSTED_SUPPORTED = false;
                                 if (debug != null) {
@@ -2379,11 +2385,7 @@ final class P11KeyStore extends KeyStoreSpi {
                     }
                 }
 
-                HashSet<AliasInfo> infoSet = certMap.get(cka_label);
-                if (infoSet == null) {
-                    infoSet = new HashSet<AliasInfo>(2);
-                    certMap.put(cka_label, infoSet);
-                }
+                HashSet<AliasInfo> infoSet = certMap.computeIfAbsent(cka_label, k -> new HashSet<>(2));
 
                 // initially create private key entry AliasInfo entries -
                 // these entries will get resolved into their true
@@ -2401,7 +2403,7 @@ final class P11KeyStore extends KeyStoreSpi {
             // or between a secret key and another object),
             // throw an exception
             HashMap<String, AliasInfo> sKeyMap =
-                    new HashMap<String, AliasInfo>();
+                    new HashMap<>();
 
             attrs = new CK_ATTRIBUTE[] {
                 ATTR_SKEY_TOKEN_TRUE,
@@ -2457,10 +2459,10 @@ final class P11KeyStore extends KeyStoreSpi {
                 throws PKCS11Exception, CertificateException {
 
         // reset global alias map
-        aliasMap = new HashMap<String, AliasInfo>();
+        aliasMap = new HashMap<>();
 
         // list of matched certs that we will return
-        ArrayList<AliasInfo> matchedCerts = new ArrayList<AliasInfo>();
+        ArrayList<AliasInfo> matchedCerts = new ArrayList<>();
 
         for (byte[] pkeyID : pkeyIDs) {
 
@@ -2502,7 +2504,7 @@ final class P11KeyStore extends KeyStoreSpi {
                 if (debug != null) {
                     debug.println
                         ("did not find match for private key with CKA_ID [" +
-                        getID(pkeyID) +
+                        getIDNullSafe(pkeyID) +
                         "] (ignoring entry)");
                 }
             }
@@ -2546,7 +2548,7 @@ final class P11KeyStore extends KeyStoreSpi {
             HashSet<AliasInfo> infoSet = certMap.get(certLabel);
             for (AliasInfo aliasInfo : infoSet) {
 
-                if (aliasInfo.matched == true) {
+                if (aliasInfo.matched) {
                     // already found a private key match for this cert -
                     // just continue
                     aliasInfo.trusted = false;
@@ -2663,7 +2665,7 @@ final class P11KeyStore extends KeyStoreSpi {
         }
     }
 
-    private final static long[] LONG0 = new long[0];
+    private static final long[] LONG0 = new long[0];
 
     private static long[] findObjects(Session session, CK_ATTRIBUTE[] attrs)
             throws PKCS11Exception {
