@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -57,7 +57,7 @@ import static java.security.DrbgParameters.Capability.*;
  * <p>
  * SecureRandom methods like reseed and nextBytes are not thread-safe.
  * An implementation is required to protect shared access to instantiate states
- * (instantiated, nonce) and DRBG states (v, c, key, reseedCounter, etc).
+ * (instantiated, nonce) and DRBG states (v, c, key, reseedCounter, etc.).
  */
 public abstract class AbstractDrbg {
 
@@ -78,7 +78,7 @@ public abstract class AbstractDrbg {
      * after each random bits generation and reset it in reseed. A mechanism
      * does <em>not</em> need to compare it to {@link #reseedInterval}.
      *
-     * Volatile, will be used in a double checked locking.
+     * Volatile, will be used in double-checked locking.
      */
     protected volatile int reseedCounter;
 
@@ -343,12 +343,10 @@ public abstract class AbstractDrbg {
         if (debug != null) {
             debug.println(this, "nextBytes");
         }
-        if (params instanceof DrbgParameters.NextBytes) {
+        if (params instanceof DrbgParameters.NextBytes dp) {
 
             // 800-90Ar1 9.3: Generate Process.
 
-            DrbgParameters.NextBytes dp = (DrbgParameters.NextBytes) params;
-
             // Step 2: max_number_of_bits_per_request
             if (result.length > maxNumberOfBytesPerRequest) {
                 // generateAlgorithm should be called multiple times to fill
@@ -378,7 +376,7 @@ public abstract class AbstractDrbg {
             instantiateIfNecessary(null);
 
             // Step 7: Auto reseed (reseedCounter might overflow)
-            // Double checked locking, safe because reseedCounter is volatile
+            // Double-checked locking, safe because reseedCounter is volatile
             if (reseedCounter < 0 || reseedCounter > reseedInterval || pr) {
                 synchronized (this) {
                     if (reseedCounter < 0 || reseedCounter > reseedInterval
@@ -410,8 +408,7 @@ public abstract class AbstractDrbg {
         if (params == null) {
             params = DrbgParameters.reseed(predictionResistanceFlag, null);
         }
-        if (params instanceof DrbgParameters.Reseed) {
-            DrbgParameters.Reseed dp = (DrbgParameters.Reseed) params;
+        if (params instanceof DrbgParameters.Reseed dp) {
 
             // 800-90Ar1 9.2: Reseed Process.
 
@@ -518,7 +515,7 @@ public abstract class AbstractDrbg {
      * assumed to support prediction resistance and always contains
      * full-entropy.
      */
-    private final static EntropySource defaultES =
+    private static final EntropySource defaultES =
             (minE, minLen, maxLen, pr) -> {
         byte[] result = new byte[minLen];
         SeedGenerator.generateSeed(result);
@@ -542,7 +539,7 @@ public abstract class AbstractDrbg {
             prseeder = defaultES;
             // According to SP800-90C section 7, a DRBG without live
             // entropy (drbg here, with pr being false) can instantiate
-            // another DRBG with weaker strength. So we choose highest
+            // another DRBG with weaker strength. So we choose the highest
             // strength we support.
             HashDrbg first = new HashDrbg(new MoreDrbgParameters(
                     prseeder, null, "SHA-256", null, false,
@@ -572,10 +569,10 @@ public abstract class AbstractDrbg {
     }
 
     /**
-     * A mechanism shall override this constructor to setup {@link #mechName},
+     * A mechanism shall override this constructor to set up {@link #mechName},
      * {@link #highestSupportedSecurityStrength},
      * {@link #supportPredictionResistance}, {@link #supportReseeding}
-     * or other features like {@link #DEFAULT_STRENGTH}. Finally it shall
+     * or other features like {@link #DEFAULT_STRENGTH}. Finally, it shall
      * call {@link #configure} on {@code params}.
      *
      * @param params the {@link SecureRandomParameters} object.
@@ -591,7 +588,7 @@ public abstract class AbstractDrbg {
      * Returns the current configuration as a {@link DrbgParameters.Instantiation}
      * object.
      *
-     * @return the curent configuration
+     * @return the current configuration
      */
     protected SecureRandomParameters engineGetParameters() {
         // Or read from variable.
@@ -619,8 +616,7 @@ public abstract class AbstractDrbg {
         if (params == null) {
             params = DrbgParameters.instantiation(-1, RESEED_ONLY, null);
         }
-        if (params instanceof MoreDrbgParameters) {
-            MoreDrbgParameters m = (MoreDrbgParameters)params;
+        if (params instanceof MoreDrbgParameters m) {
             this.requestedNonce = m.nonce;
             this.es = m.es;
             this.requestedAlgorithm = m.algorithm;
@@ -628,44 +624,40 @@ public abstract class AbstractDrbg {
             params = DrbgParameters.instantiation(m.strength,
                     m.capability, m.personalizationString);
         }
-        if (params != null) {
-            if (params instanceof DrbgParameters.Instantiation) {
-                DrbgParameters.Instantiation inst =
-                        (DrbgParameters.Instantiation) params;
+        if (params instanceof DrbgParameters.Instantiation inst) {
 
-                // 800-90Ar1 9.1: Instantiate Process. Steps 1-5.
+            // 800-90Ar1 9.1: Instantiate Process. Steps 1-5.
 
-                // Step 1: Check requested_instantiation_security_strength
-                if (inst.getStrength() > highestSupportedSecurityStrength) {
-                    throw new IllegalArgumentException("strength too big: "
-                            + inst.getStrength());
-                }
-
-                // Step 2: Check prediction_resistance_flag
-                if (inst.getCapability().supportsPredictionResistance()
-                        && !supportPredictionResistance) {
-                    throw new IllegalArgumentException("pr not supported");
-                }
-
-                // Step 3: Check personalization_string
-                byte[] ps = inst.getPersonalizationString();
-                if (ps != null && ps.length > maxPersonalizationStringLength) {
-                    throw new IllegalArgumentException("ps too long: "
-                            + ps.length);
-                }
-
-                if (inst.getCapability().supportsReseeding()
-                        && !supportReseeding) {
-                    throw new IllegalArgumentException("reseed not supported");
-                }
-                this.personalizationString = ps;
-                this.predictionResistanceFlag =
-                        inst.getCapability().supportsPredictionResistance();
-                this.requestedInstantiationSecurityStrength = inst.getStrength();
-            } else {
-                throw new IllegalArgumentException("unknown params: "
-                        + params.getClass());
+            // Step 1: Check requested_instantiation_security_strength
+            if (inst.getStrength() > highestSupportedSecurityStrength) {
+                throw new IllegalArgumentException("strength too big: "
+                        + inst.getStrength());
             }
+
+            // Step 2: Check prediction_resistance_flag
+            if (inst.getCapability().supportsPredictionResistance()
+                    && !supportPredictionResistance) {
+                throw new IllegalArgumentException("pr not supported");
+            }
+
+            // Step 3: Check personalization_string
+            byte[] ps = inst.getPersonalizationString();
+            if (ps != null && ps.length > maxPersonalizationStringLength) {
+                throw new IllegalArgumentException("ps too long: "
+                        + ps.length);
+            }
+
+            if (inst.getCapability().supportsReseeding()
+                    && !supportReseeding) {
+                throw new IllegalArgumentException("reseed not supported");
+            }
+            this.personalizationString = ps;
+            this.predictionResistanceFlag =
+                    inst.getCapability().supportsPredictionResistance();
+            this.requestedInstantiationSecurityStrength = inst.getStrength();
+        } else {
+            throw new IllegalArgumentException("unknown params: "
+                    + params.getClass());
         }
 
         // Step 4: Set security_strength
@@ -730,19 +722,6 @@ public abstract class AbstractDrbg {
 
     // Misc
 
-    /** A handy method returning hexdump string with no colon or new line.
-     *
-     * @param in input byte array
-     * @return the hexdump string
-     */
-    protected static String hex(byte[] in) {
-        StringBuilder sb = new StringBuilder();
-        for (byte b : in) {
-            sb.append(String.format("%02x", b&0xff));
-        }
-        return sb.toString();
-    }
-
     /**
      * Returns the smallest standard strength (112, 128, 192, 256) that is
      * greater or equal to the input.
