@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,18 +25,22 @@
 
 package sun.security.provider.certpath;
 
-import java.io.*;
+import java.io.IOException;
 import java.net.URI;
-import java.security.*;
+import java.security.GeneralSecurityException;
+import java.security.InvalidAlgorithmParameterException;
+import java.security.NoSuchAlgorithmException;
+import java.security.PublicKey;
 import java.security.cert.*;
-import javax.security.auth.x500.X500Principal;
 import java.util.*;
+import javax.security.auth.x500.X500Principal;
 
 import sun.security.util.Debug;
-import sun.security.validator.Validator;
-import static sun.security.x509.PKIXExtensions.*;
+import sun.security.util.Event;
 import sun.security.x509.*;
 
+import static sun.security.x509.PKIXExtensions.IssuingDistributionPoint_Id;
+
 /**
  * Class to obtain CRLs via the CRLDistributionPoints extension.
  * Note that the functionality of this class must be explicitly enabled
@@ -62,38 +66,6 @@ public class DistributionPointFetcher {
      */
     private DistributionPointFetcher() {}
 
-    /**
-     * Return the X509CRLs matching this selector. The selector must be
-     * an X509CRLSelector with certificateChecking set.
-     */
-    public static Collection<X509CRL> getCRLs(X509CRLSelector selector,
-            boolean signFlag, PublicKey prevKey, String provider,
-            List<CertStore> certStores, boolean[] reasonsMask,
-            Set<TrustAnchor> trustAnchors, Date validity, String variant)
-            throws CertStoreException
-    {
-        return getCRLs(selector, signFlag, prevKey, null, provider, certStores,
-                reasonsMask, trustAnchors, validity, variant);
-    }
-    /**
-     * Return the X509CRLs matching this selector. The selector must be
-     * an X509CRLSelector with certificateChecking set.
-     */
-    // Called by com.sun.deploy.security.RevocationChecker
-    public static Collection<X509CRL> getCRLs(X509CRLSelector selector,
-                                              boolean signFlag,
-                                              PublicKey prevKey,
-                                              String provider,
-                                              List<CertStore> certStores,
-                                              boolean[] reasonsMask,
-                                              Set<TrustAnchor> trustAnchors,
-                                              Date validity)
-        throws CertStoreException
-    {
-        return getCRLs(selector, signFlag, prevKey, null, provider, certStores,
-                reasonsMask, trustAnchors, validity, Validator.VAR_GENERIC);
-    }
-
     /**
      * Return the X509CRLs matching this selector. The selector must be
      * an X509CRLSelector with certificateChecking set.
@@ -107,7 +79,8 @@ public class DistributionPointFetcher {
                                               boolean[] reasonsMask,
                                               Set<TrustAnchor> trustAnchors,
                                               Date validity,
-                                              String variant)
+                                              String variant,
+                                              TrustAnchor anchor)
         throws CertStoreException
     {
         X509Certificate cert = selector.getCertificateChecking();
@@ -129,21 +102,21 @@ public class DistributionPointFetcher {
                 return Collections.emptySet();
             }
             List<DistributionPoint> points =
-                    ext.get(CRLDistributionPointsExtension.POINTS);
+                    ext.getDistributionPoints();
             Set<X509CRL> results = new HashSet<>();
             for (Iterator<DistributionPoint> t = points.iterator();
                  t.hasNext() && !Arrays.equals(reasonsMask, ALL_REASONS); ) {
                 DistributionPoint point = t.next();
                 Collection<X509CRL> crls = getCRLs(selector, certImpl,
                     point, reasonsMask, signFlag, prevKey, prevCert, provider,
-                    certStores, trustAnchors, validity, variant);
+                    certStores, trustAnchors, validity, variant, anchor);
                 results.addAll(crls);
             }
             if (debug != null) {
                 debug.println("Returning " + results.size() + " CRLs");
             }
             return results;
-        } catch (CertificateException | IOException e) {
+        } catch (CertificateException e) {
             return Collections.emptySet();
         }
     }
@@ -161,7 +134,8 @@ public class DistributionPointFetcher {
         X509CertImpl certImpl, DistributionPoint point, boolean[] reasonsMask,
         boolean signFlag, PublicKey prevKey, X509Certificate prevCert,
         String provider, List<CertStore> certStores,
-        Set<TrustAnchor> trustAnchors, Date validity, String variant)
+        Set<TrustAnchor> trustAnchors, Date validity, String variant,
+        TrustAnchor anchor)
             throws CertStoreException {
 
         // check for full name
@@ -224,7 +198,7 @@ public class DistributionPointFetcher {
                 selector.setIssuerNames(null);
                 if (selector.match(crl) && verifyCRL(certImpl, point, crl,
                         reasonsMask, signFlag, prevKey, prevCert, provider,
-                        trustAnchors, certStores, validity, variant)) {
+                        trustAnchors, certStores, validity, variant, anchor)) {
                     crls.add(crl);
                 }
             } catch (IOException | CRLException e) {
@@ -246,7 +220,9 @@ public class DistributionPointFetcher {
         if (debug != null) {
             debug.println("Trying to fetch CRL from DP " + uri);
         }
-        CertStore ucs = null;
+
+        Event.report(Event.ReporterCategory.CRLCHECK, "event.crl.check", uri.toString());
+        CertStore ucs;
         try {
             ucs = URICertStore.getInstance(new URICertStoreParameters(uri));
         } catch (InvalidAlgorithmParameterException |
@@ -332,7 +308,8 @@ public class DistributionPointFetcher {
         X509CRL crl, boolean[] reasonsMask, boolean signFlag,
         PublicKey prevKey, X509Certificate prevCert, String provider,
         Set<TrustAnchor> trustAnchors, List<CertStore> certStores,
-        Date validity, String variant) throws CRLException, IOException {
+        Date validity, String variant, TrustAnchor anchor)
+        throws CRLException, IOException {
 
         if (debug != null) {
             debug.println("DistributionPointFetcher.verifyCRL: " +
@@ -356,22 +333,19 @@ public class DistributionPointFetcher {
         GeneralNames pointCrlIssuers = point.getCRLIssuer();
         X500Name pointCrlIssuer = null;
         if (pointCrlIssuers != null) {
-            if (idpExt == null ||
-                ((Boolean) idpExt.get
-                    (IssuingDistributionPointExtension.INDIRECT_CRL)).equals
-                        (Boolean.FALSE)) {
+            if (idpExt == null || !idpExt.isIndirectCRL()) {
                 return false;
             }
             boolean match = false;
             for (Iterator<GeneralName> t = pointCrlIssuers.iterator();
                  !match && t.hasNext(); ) {
                 GeneralNameInterface name = t.next().getName();
-                if (crlIssuer.equals(name) == true) {
+                if (crlIssuer.equals(name)) {
                     pointCrlIssuer = (X500Name) name;
                     match = true;
                 }
             }
-            if (match == false) {
+            if (!match) {
                 return false;
             }
 
@@ -383,7 +357,7 @@ public class DistributionPointFetcher {
             } else {
                 indirectCRL = true;
             }
-        } else if (crlIssuer.equals(certIssuer) == false) {
+        } else if (!crlIssuer.equals(certIssuer)) {
             if (debug != null) {
                 debug.println("crl issuer does not equal cert issuer.\n" +
                               "crl issuer: " + crlIssuer + "\n" +
@@ -422,8 +396,7 @@ public class DistributionPointFetcher {
         }
 
         if (idpExt != null) {
-            DistributionPointName idpPoint = (DistributionPointName)
-                idpExt.get(IssuingDistributionPointExtension.POINT);
+            DistributionPointName idpPoint = idpExt.getDistributionPoint();
             if (idpPoint != null) {
                 GeneralNames idpNames = idpPoint.getFullName();
                 if (idpNames == null) {
@@ -503,10 +476,10 @@ public class DistributionPointFetcher {
                     // the names in the cRLIssuer of the cert's DP
                     boolean match = false;
                     for (Iterator<GeneralName> t = pointCrlIssuers.iterator();
-                         !match && t.hasNext(); ) {
+                            !match && t.hasNext(); ) {
                         GeneralNameInterface crlIssuerName = t.next().getName();
                         for (Iterator<GeneralName> i = idpNames.iterator();
-                             !match && i.hasNext(); ) {
+                                !match && i.hasNext(); ) {
                             GeneralNameInterface idpName = i.next().getName();
                             match = crlIssuerName.equals(idpName);
                         }
@@ -519,9 +492,8 @@ public class DistributionPointFetcher {
 
             // if the onlyContainsUserCerts boolean is asserted, verify that the
             // cert is not a CA cert
-            Boolean b = (Boolean)
-                idpExt.get(IssuingDistributionPointExtension.ONLY_USER_CERTS);
-            if (b.equals(Boolean.TRUE) && certImpl.getBasicConstraints() != -1) {
+            boolean b = idpExt.hasOnlyUserCerts();
+            if (b && certImpl.getBasicConstraints() != -1) {
                 if (debug != null) {
                     debug.println("cert must be a EE cert");
                 }
@@ -530,9 +502,8 @@ public class DistributionPointFetcher {
 
             // if the onlyContainsCACerts boolean is asserted, verify that the
             // cert is a CA cert
-            b = (Boolean)
-                idpExt.get(IssuingDistributionPointExtension.ONLY_CA_CERTS);
-            if (b.equals(Boolean.TRUE) && certImpl.getBasicConstraints() == -1) {
+            b = idpExt.hasOnlyCACerts();
+            if (b && certImpl.getBasicConstraints() == -1) {
                 if (debug != null) {
                     debug.println("cert must be a CA cert");
                 }
@@ -541,9 +512,8 @@ public class DistributionPointFetcher {
 
             // verify that the onlyContainsAttributeCerts boolean is not
             // asserted
-            b = (Boolean) idpExt.get
-                (IssuingDistributionPointExtension.ONLY_ATTRIBUTE_CERTS);
-            if (b.equals(Boolean.TRUE)) {
+            b = idpExt.hasOnlyAttributeCerts();
+            if (b) {
                 if (debug != null) {
                     debug.println("cert must not be an AA cert");
                 }
@@ -555,8 +525,7 @@ public class DistributionPointFetcher {
         boolean[] interimReasonsMask = new boolean[9];
         ReasonFlags reasons = null;
         if (idpExt != null) {
-            reasons = (ReasonFlags)
-                idpExt.get(IssuingDistributionPointExtension.REASONS);
+            reasons = idpExt.getRevocationReasons();
         }
 
         boolean[] pointReasonFlags = point.getReasonFlags();
@@ -576,7 +545,7 @@ public class DistributionPointFetcher {
                 // modify it)
                 interimReasonsMask = reasons.getFlags().clone();
             }
-        } else if (idpExt == null || reasons == null) {
+        } else {
             if (pointReasonFlags != null) {
                 // set interim reasons mask to the value of DP reasons
                 interimReasonsMask = pointReasonFlags.clone();
@@ -589,11 +558,11 @@ public class DistributionPointFetcher {
         // verify that interim reasons mask includes one or more reasons
         // not included in the reasons mask
         boolean oneOrMore = false;
-        for (int i = 0; i < interimReasonsMask.length && !oneOrMore; i++) {
+        for (int i = 0; i < interimReasonsMask.length; i++) {
             if (interimReasonsMask[i] &&
-                    !(i < reasonsMask.length && reasonsMask[i]))
-            {
+                    !(i < reasonsMask.length && reasonsMask[i])) {
                 oneOrMore = true;
+                break;
             }
         }
         if (!oneOrMore) {
@@ -609,7 +578,7 @@ public class DistributionPointFetcher {
             boolean[] crlSign = {false,false,false,false,false,false,true};
             certSel.setKeyUsage(crlSign);
 
-            // Currently by default, forward builder does not enable
+            // Currently, by default, forward builder does not enable
             // subject/authority key identifier identifying for target
             // certificate, instead, it only compares the CRL issuer and
             // the target certificate subject. If the certificate of the
@@ -627,8 +596,7 @@ public class DistributionPointFetcher {
                     certSel.setSubjectKeyIdentifier(kid);
                 }
 
-                SerialNumber asn = (SerialNumber)akidext.get(
-                        AuthorityKeyIdentifierExtension.SERIAL_NUMBER);
+                SerialNumber asn = akidext.getSerialNumber();
                 if (asn != null) {
                     certSel.setSerialNumber(asn.getNumber());
                 }
@@ -658,7 +626,7 @@ public class DistributionPointFetcher {
                 newTrustAnchors.add(temporary);
             }
 
-            PKIXBuilderParameters params = null;
+            PKIXBuilderParameters params;
             try {
                 params = new PKIXBuilderParameters(newTrustAnchors, certSel);
             } catch (InvalidAlgorithmParameterException iape) {
@@ -679,7 +647,8 @@ public class DistributionPointFetcher {
 
         // check the crl signature algorithm
         try {
-            AlgorithmChecker.check(prevKey, crl, variant);
+            AlgorithmChecker.check(prevKey, crlImpl.getSigAlgId(),
+                                   variant, anchor);
         } catch (CertPathValidatorException cpve) {
             if (debug != null) {
                 debug.println("CRL signature algorithm check failed: " + cpve);
@@ -748,7 +717,7 @@ public class DistributionPointFetcher {
     private static boolean issues(X509CertImpl cert, X509CRLImpl crl,
                                   String provider) throws IOException
     {
-        boolean matched = false;
+        boolean matched;
 
         AdaptableX509CertSelector issuerSelector =
                                     new AdaptableX509CertSelector();
@@ -782,7 +751,6 @@ public class DistributionPointFetcher {
                 cert.getAuthorityKeyIdentifierExtension() == null)) {
             try {
                 crl.verify(cert.getPublicKey(), provider);
-                matched = true;
             } catch (GeneralSecurityException e) {
                 matched = false;
             }
