@@ -33,7 +33,6 @@ import java.security.cert.*;
 import java.security.cert.CertPathValidatorException.BasicReason;
 import java.security.cert.PKIXReason;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
@@ -43,7 +42,6 @@ import javax.security.auth.x500.X500Principal;
 
 import sun.security.provider.certpath.PKIX.BuilderParams;
 import static sun.security.x509.PKIXExtensions.*;
-import sun.security.x509.SubjectAlternativeNameExtension;
 import sun.security.x509.X509CertImpl;
 import sun.security.util.Debug;
 
@@ -75,7 +73,7 @@ public final class SunCertPathBuilder extends CertPathBuilderSpi {
      * private objects shared by methods
      */
     private BuilderParams buildParams;
-    private CertificateFactory cf;
+    private final CertificateFactory cf;
     private boolean pathCompleted = false;
     private PolicyNode policyTreeResult;
     private TrustAnchor trustAnchor;
@@ -224,7 +222,7 @@ public final class SunCertPathBuilder extends CertPathBuilderSpi {
 
         /* Initialize adjacency list */
         adjacencyList.clear();
-        adjacencyList.add(new LinkedList<Vertex>());
+        adjacencyList.add(new LinkedList<>());
 
         currentState.untrustedChecker = new UntrustedChecker();
 
@@ -267,7 +265,7 @@ public final class SunCertPathBuilder extends CertPathBuilderSpi {
          */
         Collection<X509Certificate> certs =
             builder.getMatchingCerts(currentState, buildParams.certStores());
-        List<Vertex> vertices = addVertices(certs, adjList, cpList);
+        List<Vertex> vertices = addVertices(certs, adjList);
         if (debug != null) {
             debug.println("SunCertPathBuilder.depthFirstSearchForward(): "
                           + "certs.size=" + vertices.size());
@@ -282,9 +280,9 @@ public final class SunCertPathBuilder extends CertPathBuilderSpi {
 
                vertices:
         for (Vertex vertex : vertices) {
-            /**
+            /*
              * Restore state to currentState each time through the loop.
-             * This is important because some of the user-defined
+             * This is important because some user-defined
              * checkers modify the state, which MUST be restored if
              * the cert eventually fails to lead to the target and
              * the next matching cert is tried.
@@ -327,9 +325,33 @@ public final class SunCertPathBuilder extends CertPathBuilderSpi {
                  * cert (which is signed by the trusted public key), but
                  * don't add it yet to the cpList
                  */
-                PublicKey rootKey = cert.getPublicKey();
                 if (builder.trustAnchor.getTrustedCert() == null) {
                     appendedCerts.add(0, cert);
+                }
+
+                Set<String> initExpPolSet =
+                    Collections.singleton(PolicyChecker.ANY_POLICY);
+
+                PolicyNodeImpl rootNode = new PolicyNodeImpl(null,
+                    PolicyChecker.ANY_POLICY, null, false, initExpPolSet, false);
+
+                List<PKIXCertPathChecker> checkers = new ArrayList<>();
+                PolicyChecker policyChecker
+                    = new PolicyChecker(buildParams.initialPolicies(),
+                                        appendedCerts.size(),
+                                        buildParams.explicitPolicyRequired(),
+                                        buildParams.policyMappingInhibited(),
+                                        buildParams.anyPolicyInhibited(),
+                                        buildParams.policyQualifiersRejected(),
+                                        rootNode);
+                checkers.add(policyChecker);
+
+                // add the algorithm checker
+                checkers.add(new AlgorithmChecker(builder.trustAnchor,
+                        buildParams.timestamp(), buildParams.variant()));
+
+                PublicKey rootKey = cert.getPublicKey();
+                if (builder.trustAnchor.getTrustedCert() == null) {
                     rootKey = builder.trustAnchor.getCAPublicKey();
                     if (debug != null)
                         debug.println(
@@ -341,35 +363,11 @@ public final class SunCertPathBuilder extends CertPathBuilderSpi {
                     (cert.getSubjectX500Principal(), rootKey, null);
 
                 // add the basic checker
-                List<PKIXCertPathChecker> checkers = new ArrayList<>();
                 BasicChecker basicChecker = new BasicChecker(anchor,
                                                     buildParams.date(),
                                                     buildParams.sigProvider(),
                                                     true);
                 checkers.add(basicChecker);
-                Set<String> initExpPolSet =
-                    Collections.singleton(PolicyChecker.ANY_POLICY);
-
-                PolicyNodeImpl rootNode = new PolicyNodeImpl(null,
-                    PolicyChecker.ANY_POLICY, null, false, initExpPolSet, false);
-
-                PolicyChecker policyChecker
-                    = new PolicyChecker(buildParams.initialPolicies(),
-                                        appendedCerts.size(),
-                                        buildParams.explicitPolicyRequired(),
-                                        buildParams.policyMappingInhibited(),
-                                        buildParams.anyPolicyInhibited(),
-                                        buildParams.policyQualifiersRejected(),
-                                        rootNode);
-                checkers.add(policyChecker);
-
-                // add the constraints checker
-                checkers.add(new ConstraintsChecker(appendedCerts.size()));
-
-                // add the algorithm checker
-                checkers.add(new AlgorithmChecker(builder.trustAnchor,
-                        buildParams.timestamp(), buildParams.variant()));
-
 
                 buildParams.setCertPath(cf.generateCertPath(appendedCerts));
 
@@ -409,7 +407,7 @@ public final class SunCertPathBuilder extends CertPathBuilderSpi {
                     Set<String> unresCritExts =
                         currCert.getCriticalExtensionOIDs();
                     if (unresCritExts == null) {
-                        unresCritExts = Collections.<String>emptySet();
+                        unresCritExts = Collections.emptySet();
                     }
 
                     for (PKIXCertPathChecker currChecker : checkers) {
@@ -533,7 +531,7 @@ public final class SunCertPathBuilder extends CertPathBuilderSpi {
              * Append an entry for cert in adjacency list and
              * set index for current vertex.
              */
-            adjList.add(new LinkedList<Vertex>());
+            adjList.add(new LinkedList<>());
             vertex.setIndex(adjList.size() - 1);
 
             /* recursively search for matching certs at next dN */
@@ -565,79 +563,18 @@ public final class SunCertPathBuilder extends CertPathBuilderSpi {
      * adjacency list.
      */
     private static List<Vertex> addVertices(Collection<X509Certificate> certs,
-                                            List<List<Vertex>> adjList,
-                                            List<X509Certificate> cpList)
+                                            List<List<Vertex>> adjList)
     {
         List<Vertex> l = adjList.get(adjList.size() - 1);
 
         for (X509Certificate cert : certs) {
-            boolean repeated = false;
-            for (X509Certificate cpListCert : cpList) {
-                /*
-                 * Ignore if we encounter the same certificate or a
-                 * certificate with the same public key, subject DN, and
-                 * subjectAltNames as a cert that is already in path.
-                 */
-                if (repeated(cpListCert, cert)) {
-                    if (debug != null) {
-                        debug.println("cert with repeated subject, " +
-                            "public key, and subjectAltNames detected");
-                    }
-                    repeated = true;
-                    break;
-                }
-            }
-            if (!repeated) {
-                l.add(new Vertex(cert));
-            }
+            Vertex v = new Vertex(cert);
+            l.add(v);
         }
 
         return l;
     }
 
-    /**
-     * Return true if two certificates are equal or have the same subject,
-     * public key, and subject alternative names.
-     */
-    private static boolean repeated(
-            X509Certificate currCert, X509Certificate nextCert) {
-        if (currCert.equals(nextCert)) {
-            return true;
-        }
-        return (currCert.getSubjectX500Principal().equals(
-            nextCert.getSubjectX500Principal()) &&
-            currCert.getPublicKey().equals(nextCert.getPublicKey()) &&
-            altNamesEqual(currCert, nextCert));
-    }
-
-    /**
-     * Return true if two certificates have the same subject alternative names.
-     */
-    private static boolean altNamesEqual(
-            X509Certificate currCert, X509Certificate nextCert) {
-        X509CertImpl curr, next;
-        try {
-            curr = X509CertImpl.toImpl(currCert);
-            next = X509CertImpl.toImpl(nextCert);
-        } catch (CertificateException ce) {
-            return false;
-        }
-
-        SubjectAlternativeNameExtension currAltNameExt =
-            curr.getSubjectAlternativeNameExtension();
-        SubjectAlternativeNameExtension nextAltNameExt =
-            next.getSubjectAlternativeNameExtension();
-        if (currAltNameExt != null) {
-            if (nextAltNameExt == null) {
-                return false;
-            }
-            return Arrays.equals(currAltNameExt.getExtensionValue(),
-                nextAltNameExt.getExtensionValue());
-        } else {
-            return (nextAltNameExt == null);
-        }
-    }
-
     /**
      * Returns true if trust anchor certificate matches specified
      * certificate constraints.
