@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2006, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -113,7 +113,7 @@ class URICertStore extends CertStoreSpi {
     private long lastModified;
 
     // the URI of this CertStore
-    private URI uri;
+    private final URI uri;
 
     // true if URI is ldap
     private boolean ldap = false;
@@ -166,7 +166,7 @@ class URICertStore extends CertStoreSpi {
     /**
      * Creates a URICertStore.
      *
-     * @param parameters specifying the URI
+     * @param params specifying the URI
      */
     URICertStore(CertStoreParameters params)
         throws InvalidAlgorithmParameterException, NoSuchAlgorithmException {
@@ -284,9 +284,8 @@ class URICertStore extends CertStoreSpi {
                             debug.println("Not modified, using cached copy");
                         }
                         return getMatchingCerts(certs, selector);
-                    } else if (connection instanceof HttpURLConnection) {
+                    } else if (connection instanceof HttpURLConnection hconn) {
                         // some proxy servers omit last modified
-                        HttpURLConnection hconn = (HttpURLConnection)connection;
                         if (hconn.getResponseCode()
                                     == HttpURLConnection.HTTP_NOT_MODIFIED) {
                             if (debug != null) {
@@ -390,9 +389,8 @@ class URICertStore extends CertStoreSpi {
                             debug.println("Not modified, using cached copy");
                         }
                         return getMatchingCRLs(crl, selector);
-                    } else if (connection instanceof HttpURLConnection) {
+                    } else if (connection instanceof HttpURLConnection hconn) {
                         // some proxy servers omit last modified
-                        HttpURLConnection hconn = (HttpURLConnection)connection;
                         if (hconn.getResponseCode()
                                     == HttpURLConnection.HTTP_NOT_MODIFIED) {
                             if (debug != null) {
