@@ -27,29 +27,13 @@ package sun.security.util;
 
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
-import java.security.CodeSigner;
-import java.security.GeneralSecurityException;
-import java.security.MessageDigest;
-import java.security.NoSuchAlgorithmException;
-import java.security.PrivateKey;
-import java.security.SignatureException;
+import java.security.*;
 import java.security.cert.CertPath;
-import java.security.cert.X509Certificate;
 import java.security.cert.CertificateException;
 import java.security.cert.CertificateFactory;
-import java.util.ArrayList;
-import java.util.Base64;
-import java.util.HashMap;
-import java.util.Hashtable;
-import java.util.HexFormat;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Locale;
-import java.util.Map;
-import java.util.Set;
+import java.security.cert.X509Certificate;
+import java.util.*;
 import java.util.jar.Attributes;
-import java.util.jar.JarException;
-import java.util.jar.JarFile;
 import java.util.jar.Manifest;
 
 import sun.security.action.GetIntegerAction;
@@ -62,24 +46,24 @@ public class SignatureFileVerifier {
     /* Are we debugging ? */
     private static final Debug debug = Debug.getInstance("jar");
 
-    private ArrayList<CodeSigner[]> signerCache;
+    private final ArrayList<CodeSigner[]> signerCache;
 
     private static final String ATTR_DIGEST =
         "-DIGEST-" + ManifestDigester.MF_MAIN_ATTRS.toUpperCase(Locale.ENGLISH);
 
     /** the PKCS7 block for this .DSA/.RSA/.EC file */
-    private PKCS7 block;
+    private final PKCS7 block;
 
     /** the raw bytes of the .SF file */
     private byte[] sfBytes;
 
-    /** the name of the signature block file, uppercased and without
+    /** the name of the signature block file, uppercase and without
      *  the extension (.DSA/.RSA/.EC)
      */
-    private String name;
+    private final String name;
 
     /** the ManifestDigester */
-    private ManifestDigester md;
+    private final ManifestDigester md;
 
     /** cache of created MessageDigest objects */
     private HashMap<String, MessageDigest> createdDigests;
@@ -88,12 +72,12 @@ public class SignatureFileVerifier {
     private boolean workaround = false;
 
     /* for generating certpath objects */
-    private CertificateFactory certificateFactory = null;
+    private final CertificateFactory certificateFactory;
 
     /** Algorithms that have been previously checked against disabled
      *  constraints.
      */
-    private Map<String, Boolean> permittedAlgs = new HashMap<>();
+    private final Map<String, Boolean> permittedAlgs = new HashMap<>();
 
     /** ConstraintsParameters for checking disabled algorithms */
     private JarConstraintsParameters params;
@@ -251,8 +235,7 @@ public class SignatureFileVerifier {
 
     /** get digest from cache */
 
-    private MessageDigest getDigest(String algorithm)
-            throws SignatureException {
+    private MessageDigest getDigest(String algorithm) {
         if (createdDigests == null)
             createdDigests = new HashMap<>();
 
@@ -279,7 +262,7 @@ public class SignatureFileVerifier {
     public void process(Hashtable<String, CodeSigner[]> signers,
             List<Object> manifestDigests, String manifestName)
         throws IOException, SignatureException, NoSuchAlgorithmException,
-            JarException, CertificateException
+            CertificateException
     {
         // calls Signature.getInstance() and MessageDigest.getInstance()
         // need to use local providers here, see Providers class
@@ -296,7 +279,7 @@ public class SignatureFileVerifier {
     private void processImpl(Hashtable<String, CodeSigner[]> signers,
             List<Object> manifestDigests, String manifestName)
         throws IOException, SignatureException, NoSuchAlgorithmException,
-            JarException, CertificateException
+            CertificateException
     {
         Manifest sf = new Manifest();
         sf.read(new ByteArrayInputStream(sfBytes));
@@ -306,7 +289,7 @@ public class SignatureFileVerifier {
 
         if ((version == null) || !(version.equalsIgnoreCase("1.0"))) {
             // XXX: should this be an exception?
-            // for now we just ignore this signature file
+            // for now, we just ignore this signature file
             return;
         }
 
@@ -393,7 +376,8 @@ public class SignatureFileVerifier {
                     .jarConstraints().permits(algorithm, params, false);
             } catch (GeneralSecurityException e) {
                 permittedAlgs.put(algorithm, Boolean.FALSE);
-                permittedAlgs.put(key.toUpperCase(), Boolean.FALSE);
+                permittedAlgs.put(key.toUpperCase(Locale.ENGLISH),
+                        Boolean.FALSE);
                 if (debug != null) {
                     if (e.getMessage() != null) {
                         debug.println(key + ":  " + e.getMessage());
@@ -448,7 +432,7 @@ public class SignatureFileVerifier {
     private boolean verifyManifestHash(Manifest sf,
                                        ManifestDigester md,
                                        List<Object> manifestDigests)
-         throws IOException, SignatureException
+         throws SignatureException
     {
         Attributes mattr = sf.getMainAttributes();
         boolean manifestSigned = false;
@@ -519,7 +503,7 @@ public class SignatureFileVerifier {
     }
 
     private boolean verifyManifestMainAttrs(Manifest sf, ManifestDigester md)
-         throws IOException, SignatureException
+         throws SignatureException
     {
         Attributes mattr = sf.getMainAttributes();
         boolean attrsVerified = true;
@@ -617,7 +601,7 @@ public class SignatureFileVerifier {
     private boolean verifySection(Attributes sfAttr,
                                   String name,
                                   ManifestDigester md)
-         throws IOException, SignatureException
+         throws SignatureException
     {
         boolean oneDigestVerified = false;
         ManifestDigester.Entry mde = md.get(name,block.isOldStyle());
@@ -751,7 +735,7 @@ public class SignatureFileVerifier {
         }
 
         if (signers != null) {
-            return signers.toArray(new CodeSigner[signers.size()]);
+            return signers.toArray(new CodeSigner[0]);
         } else {
             return null;
         }
@@ -774,7 +758,6 @@ public class SignatureFileVerifier {
         if (set == subset)
             return true;
 
-        boolean match;
         for (int i = 0; i < subset.length; i++) {
             if (!contains(set, subset[i]))
                 return false;
@@ -794,8 +777,6 @@ public class SignatureFileVerifier {
         if ((oldSigners == null) && (signers == newSigners))
             return true;
 
-        boolean match;
-
         // make sure all oldSigners are in signers
         if ((oldSigners != null) && !isSubSet(oldSigners, signers))
             return false;
@@ -856,16 +837,16 @@ public class SignatureFileVerifier {
          * the maximum allowed number of bytes for the signature-related files
          * in a JAR file.
          */
-        int tmp = GetIntegerAction.privilegedGetProperty(
-                "jdk.jar.maxSignatureFileSize", 16000000);
+        Integer tmp = GetIntegerAction.privilegedGetProperty(
+                "jdk.jar.maxSignatureFileSize", 8000000);
         if (tmp < 0 || tmp > MAX_ARRAY_SIZE) {
             if (debug != null) {
-                debug.println("The default signature file size of 16000000 bytes " +
-                        "will be used for the jdk.jar.maxSignatureFileSize " +
-                        "system property since the specified value " +
+                debug.println("Default signature file size 8000000 bytes " +
+                        "is used as the specified size for the " +
+                        "jdk.jar.maxSignatureFileSize system property " +
                         "is out of range: " + tmp);
             }
-            tmp = 16000000;
+            tmp = 8000000;
         }
         return tmp;
     }
