@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2004, 2022, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -82,9 +82,9 @@ public class JConsole extends JFrame
     }
 
 
-    private final static String title =
+    private static final String title =
         Messages.JAVA_MONITORING___MANAGEMENT_CONSOLE;
-    public final static String ROOT_URL =
+    public static final String ROOT_URL =
         "service:jmx:";
 
     private static int updateInterval = 4000;
@@ -483,9 +483,7 @@ public class JConsole extends JFrame
             public void run() {
                 try {
                     addProxyClient(ProxyClient.getProxyClient(lvm), tile);
-                } catch (final SecurityException ex) {
-                    failed(ex, null, null, null);
-                } catch (final IOException ex) {
+                } catch (final SecurityException | IOException ex) {
                     failed(ex, null, null, null);
                 }
             }
@@ -502,11 +500,7 @@ public class JConsole extends JFrame
                 try {
                     addProxyClient(ProxyClient.getProxyClient(url, userName, password),
                                    tile);
-                } catch (final MalformedURLException ex) {
-                    failed(ex, url, userName, password);
-                } catch (final SecurityException ex) {
-                    failed(ex, url, userName, password);
-                } catch (final IOException ex) {
+                } catch (final IOException | SecurityException ex) {
                     failed(ex, url, userName, password);
                 }
             }
@@ -1053,7 +1047,9 @@ public class JConsole extends JFrame
         if (!file.isFile()) {
             name = name + "/";
         }
-        return new URL("file", "", name);
+        @SuppressWarnings("deprecation")
+        var result = new URL("file", "", name);
+        return result;
     }
 
 
